GADGET = {};

/**
 * Defines the ajax requests required for the template function
 */
GADGET.templateArgs = function(baseUrl) {
	//Assign wallboard class here so errors also get this class in the body
	if (GH.gadget.utils.getViewName() == 'wallboard') AJS.$('body').addClass('wallboard');
	return [
	    {
	    	key: "data",
	    	ajaxOptions: function () {
	    		var gadget = this;
	    		return {
	    	        url : baseUrl + "/rest/greenhopper/1.0/chartdata/generate.json",
	    	        data :
	    	        {
	    	            projectId : gadget.getPref("projectId"),
	    	            versionId : gadget.getPref("versionId"),
	    	            contextId   : gadget.getPref("contextId"),
	    	            chartType : "gh.chart.cacumulativeflow"
	    	        },
	    	        dataType : "json",
	    	        baseUrl : baseUrl,
	    	        contentType : "application/json",
	    	        global : true,
	    	        processData : true,
	    	        type : "get"
	    	    }
        	}
	    }
	];
};

/**
 * Gadget template function	 
 */
GADGET.template = function (gadget, args, baseUrl) {
	var chartClass = 'greenhopper-cumulative-flow-chart-gadget';
	var isWallboard = GH.gadget.utils.isWallboardView();

	// fetch the flot defaults and adapt some values
    var plotDefaults = CHART.getPlotDefaults(isWallboard, args.data);
    plotDefaults.series.lines.fill = 1;
    plotDefaults.series.lines.lineWidth=1;
    
    // remove last day to always fill out complete chart width
    if (isWallboard && args.data && args.data.endX) args.data.endX -= 1;
    
    // create the plotData for the burndown chart. in this case we display the chart as-is, but adapt the colors
    var plotData = CHART.identityPlotData(args.data);
    CHART.rewritePlotDataColors(plotData);
    
    // create the items for the title
    var titleElements = GH.gadget.utils.getProjectVersionHeaderElements(gadget, args.data);
    var showContextName = gadget.getPref('showContextName') == 'true';
	if (showContextName && args.data && args.data.charts && args.data.charts[0]) {
		titleElements.push(args.data.charts[0].context.name);
	}
    
    CHART.template(gadget, args.data, baseUrl, chartClass, isWallboard, plotDefaults, plotData, titleElements);
};

/**
 * Returns the ajax requests that should be executed prior to calling GADGET.descriptor.
 * The descriptor will get the results in the args parameter, with each request response added to the "key" field
 */
GADGET.descriptorArgs = function (baseUrl) {
	return [
        {
            key: "projectOptions",
            ajaxOptions:  baseUrl + "/rest/greenhopper/1.0/project-list"
        }
    ];
};

/**
 * Configuration screen descriptor for the days remaining gadget
 */ 
GADGET.descriptor = function (gadget, args, baseUrl) {
	//[GHS-2677] 
	//Root cause is that the existing content is not cleared when an error is added as a result of a gadget refresh
	//To fix this we hide existing content when this class is appended into the body
	//Calling descriptor should remove class indicating invalidness
	AJS.$("body").removeClass('config-unavailable');
	return {
        action: baseUrl + "/rest/greenhopper/1.0/chartdata/validate.json",
        theme : function()
        {
            if (gadgets.window.getViewportDimensions().width < 450)
            {
                return "gdt top-label";
            }
            else
            {
                return "gdt";
            }
        }(),
        fields:[
            GH.gadget.fields.projectPicker(gadget, "projectId", args.projectOptions.options),
            GH.gadget.fields.versionPicker(gadget, "projectId", "versionId", false, true),
            GH.gadget.fields.contextPicker(gadget, "projectId", "contextId", false),
            GH.gadget.fields.checkbox(gadget, "showProjectName", 'gh.gadget.common.options.projectname', gadget.getPref('showProjectName') == 'true'),
            GH.gadget.fields.checkbox(gadget, "showVersionName", 'gh.gadget.common.options.versionname', gadget.getPref('showVersionName') == 'true'),
            GH.gadget.fields.checkbox(gadget, "showContextName", 'gh.gadget.common.options.contextname', gadget.getPref('showContextName') == 'true'),
            AJS.gadget.fields.nowConfigured()
        ]
    };
};
