GADGET = {};

/**
 * Ajax requests for the template
 */
GADGET.templateArgs = function(baseUrl) {
	//Assign wallboard class here so errors also get this class in the body
	if (GH.gadget.utils.getViewName() == 'wallboard') AJS.$('body').addClass('wallboard');
	return [
	    {
	        key: "ghGadget",
	        ajaxOptions: function()
	        {
	     		var gadget = this;
	     		return {
	     		    url: baseUrl + "/rest/greenhopper/1.0/project-dashboard/generate",
	     		    data: {
	     		        selectedProjectId: gadgets.util.unescapeString(gadget.getPref("selectedProjectId")),
	     		        selectedBoardId: gadget.getPref("selectedBoardId"),
	     		        contextName: gadget.getPref("contextName"),
	     		        chartType: gadget.getPref("chartType"),
	     		        fieldId: gadget.getPref("fieldId"),
	     		        startDate: gadget.getPref("startDate"),
	     		        endDate: gadget.getPref("endDate"),
	     		        forMaster: gadget.getPref("forMaster"),
	     		        showLabels: gadget.getPref("showLabels"),
	     		        showLegend: gadget.getPref("showLegend"),
	     		        width: Math.min(gadgets.window.getViewportDimensions().width, 800)
	     		    }
	     		};
	        }
	    }
	];
};

/**
 * Template function
 */
GADGET.template = function (gadget, args, baseUrl) {
	
    gadget.projectOrFilterName = args.ghGadget.title;
    gadget.getView().addClass("chart").attr({id:"agile-gadget-view"}).empty();
    
	var renderActions = function(){
        var div = AJS.$("<div/>");
        AJS.$(args.ghGadget.actions.actionItems).each(function(){
        	var a = AJS.$("<a/>").addClass("btn").attr(
                        {
                            href: gadget.getBaseUrl() + this.location,
                            title: this.title
                        });
            a.css("background-image", "url('" + gadget.getBaseUrl() + this.iconUrl + "')").append("&nbsp;&nbsp;");
       		div.append(a);
        });
        return function ()
        {
            return div;
        };
	}();
    var renderProjectHeader = function(){
        var div = '';
        div = AJS.$("<div/>").append(renderActions().css("float","right"));
        var h = AJS.$("<h5/>").addClass("project-title");
        if(args.ghGadget.board.project){
            h.append(AJS.$("<a/>").text(args.ghGadget.board.project.value).attr({href: gadget.getBaseUrl() + '/browse/' + args.ghGadget.board.project.name})).append(AJS.$("<span/>").text("(" + args.ghGadget.board.project.name + ")"));
        }
        else{
        	h.text(gadget.getMsg("gh.gadget.filterbase"));
        }
        div.append(h);
        return function ()
        {
            return div;
        };
    }();
	var versionSelect = function() {
		var control = '';
		if(args.ghGadget.controls.versions){
    		control = AJS.$("<select id='select-version'/>").css({"width": "70%", "max-width":"300px"});
            AJS.$(args.ghGadget.controls.versions.all).each(function(){
            	var option  = AJS.$("<option/>").addClass(this.className).attr({ value: this.value });
            	option.append(this.name);
            	if(this.value == args.ghGadget.selections.selectedBoardId){
            		option.attr({selected: "selected"});
            	}
           		control.append(option);
            });
            control.change(function(event){
                gadget.savePref("selectedBoardId", AJS.$(this).val());
                gadget.showView(true);
            });
        }
		
        return function ()
        {
            return control;
        };       
	}();
	var filterSelect = function() {
		var control = '';
		if(args.ghGadget.controls.filters){
    		control = AJS.$("<select id='select-filter'/>").css({"width": "70%", "max-width":"300px"});
            AJS.$(args.ghGadget.controls.filters.all).each(function(){
            	var option  = AJS.$("<option/>").addClass(this.className).attr({ value: this.value });
            	option.append(this.name);
            	if(this.value == args.ghGadget.selections.selectedProjectId){
            		option.attr({selected: "selected"});
            	}
           		control.append(option);
            });
            control.change(function(event){
                gadget.savePref("selectedProjectId", AJS.$(this).val());
                gadget.showView(true);
            });
        }
		
        return function ()
        {
            return control;
        };       
	}();
	var renderBar = function() {
    	var div = '';
    	if(args.ghGadget.board.summary){
        	div = AJS.$("<div/>");
        	if(args.ghGadget.board.summary.progressStats){
        		var table = AJS.$("<table/>").addClass("bar");
        		var tr = AJS.$("<tr/>");
                AJS.$(args.ghGadget.board.summary.progressStats.barItems).each(function(){
                	if(this.value!="0"){
                		// keep this around for the click handler function
                	    var item = this;
                        tr.append(
                        	AJS.$("<td/>")
                        		.addClass(this.className)
                        		.css("width", this.ratio)
                        		.attr({ title: this.title })
                        		.click(function() {
                        			parent.location = gadget.getBaseUrl() + item.location;	
                        		})
                       	);
                    }
                });
               	div.append(table.append(tr));
            }
            else{
            	div.addClass("empty").text(gadget.getMsg("gh.gadget.noissues"));
            }
		}
        return function ()
        {
            return div;
        };   
	}();
	var contextSelect = function() {
		var control = '';
		if(args.ghGadget.controls.contexts){
    		control = AJS.$("<select id='select-context'/>").css({"width": "70%", "max-width":"300px"});
            AJS.$(args.ghGadget.controls.contexts.all).each(function(){
            	var option  = AJS.$("<option/>").addClass(this.className).attr({ value: this.value });
            	if(this.value == args.ghGadget.selections.contextName){
            		option.attr({selected: "selected"});
            	}
            	option.append(this.name);
           		control.append(option);
            });
            control.change(function(event){
                gadget.savePref("contextName", AJS.$(this).val());
                gadget.showView(true);
            });
		}
        return function ()
        {
            return control;
        };  
	}();
	var renderBarStats = function() {
		var div = '';
    	if(args.ghGadget.board.summary.progressStats){
    		div = AJS.$("<div/>").addClass("small top").css("white-space", "nowrap");
    		var counter = 0; var size = args.ghGadget.board.summary.progressStats.barItems.length;
            AJS.$(args.ghGadget.board.summary.progressStats.barItems).each(function(){
            	div.append(this.name + " ");
            	var a = (AJS.$("<a/>").text(this.value));
                if(this.location){
                	a.attr({href: gadget.getBaseUrl() + this.location});
            	}
            	div.append(a);
                if(++counter < size){
                	div.append(" / ");
                }
            });
    	}
		
        return function ()
        {
            return div;
        }; 
	}();
	var renderVersionHeader = function(){
		var table = AJS.$("<table/>").css("width", "100%");
		var tr = AJS.$("<tr/>");
		if(args.ghGadget.board.project){
    		var icon = AJS.$("<td/>").addClass("gh_icon");
    		icon.append( AJS.$("<img/>").attr({
        			title : gadget.getMsg("gh.gadget.unreleased"),
    				src : gadget.getBaseUrl() + "/images/icons/box_16.gif",
    				width : 16,
    				height : 16
    			})
    		);
    		tr.append(icon);
			tr.append(AJS.$("<td/>").append(versionSelect()));
			tr.append(AJS.$("<td/>").addClass(args.ghGadget.board.isOverDue ? "relDate overdue" : "relDate").text(args.ghGadget.board.releaseDate).attr({title: gadget.getMsg("gh.gadget.releasedate")}));
		}
		else{
			var icon = AJS.$("<td/>").addClass("gh_icon");
    		icon.append( AJS.$("<img/>").attr({
                    title : gadget.getMsg("gh.gadget.filter"),
    				src : gadget.getBaseUrl() + "/images/icons/ico_filters.png",
    				width : 16,
    				height : 16
    			})
    		);
    		tr.append(icon);
			tr.append(AJS.$("<td colspan='2'/>").append(filterSelect()));
		}
		tr.append(AJS.$("<td/>").addClass("progress").append(renderBar()));
		table.append(tr);
		
		var trStats = AJS.$("<tr/>");
		if(args.ghGadget.board.project){
    		var iconCtx = AJS.$("<td/>").addClass("gh_icon");
    		iconCtx.append(
    			AJS.$("<img/>").attr({
        			title : gadget.getMsg("gh.gadget.context"),
    				src : gadget.getBaseUrl() + "/images/icons/ico_filters.png",
    				width : 16,
    				height : 16
    			})
    		);
			trStats.append(iconCtx);
			trStats.append(AJS.$("<td/>").append(contextSelect()));
    	}
    	else{
			trStats.append(AJS.$("<td colspan='2'/>"));
    	}
		var tdStats = AJS.$("<td colspan='2'/>").attr({ valign: "top", align: "right" });
			tdStats.append(renderBarStats());
			trStats.append(tdStats);
		table.append(trStats);
        return function ()
        {
            return table;
        };                		
	}();
    var getMainContainer = function ()
    {
		var mainDiv = AJS.$("<div id='main-container'/>").addClass("main");
        mainDiv.append(renderProjectHeader());
        mainDiv.append(renderVersionHeader());
		gadget.getView().append(mainDiv);
		
        return function ()
        {
            return mainDiv;
        };
    }();
	var fieldSelect = function() {
		var control = '';
    	if(args.ghGadget.controls.watchedFields){
    		control = AJS.$("<select id='select-field'/>").css("width", "100%").css("clear", "both");
            AJS.$(args.ghGadget.controls.watchedFields.all).each(function(){
            	var option  = AJS.$("<option/>").addClass(this.className).attr({ value: this.value });
            	if(this.value == args.ghGadget.selections.fieldId){
            		option.attr({selected: "selected"});
            	}
            	option.append(this.name);
           		control.append(option);
            });
            control.change(function(event){                                                            
                gadget.savePref("fieldId", AJS.$(this).val());
                gadget.showView(true);
            });
        }

        return function ()
        {
            return control;
        };
	}();
	var displaySelect = function() {
		var div = '';
    	if(args.ghGadget.controls.displays){
    		div = AJS.$("<div/>").css("clear", "both");
    		var control = AJS.$("<select/>");
            AJS.$(args.ghGadget.controls.displays.all).each(function(){
            	var option  = AJS.$("<option/>").addClass(this.className).attr({ value: this.value });
            	if(this.value == ""+args.ghGadget.selections.forMaster){
            		option.attr({selected: "selected"});
            	}
            	option.append(this.name);
           		control.append(option);
            });
            control.change(function(event){
                gadget.savePref("forMaster", AJS.$(this).val());
                gadget.showView(true);
            });
            div.append(control);
        }
        else{
        	div = AJS.$("<div/>").css("height", "19px").css("clear", "both").text(" ");
        }
        return function ()
        {
            return div;
        };
	}();
	var buildMenuBar = function(){
		var div = AJS.$("<div/>").addClass("tabwrap");
    	if(args.ghGadget.controls.watchedFields){
    		div.append(AJS.$("<div/>").addClass("sortwrap").append(fieldSelect()));
   		}
		var ul = AJS.$("<ul/>").addClass("tabs horizontal");
       	AJS.$(args.ghGadget.controls.menuBar.items).each(function(){
				var li = AJS.$("<li/>").css("font-weight", "700").text(gadget.getMsg(this.name)).attr({id:"tab-" + gadget.getMsg(this.name) ,rel: this.value});
				if(this.value != args.ghGadget.selections.chartType){
   				li.click(function(event){
                        gadget.savePref("chartType", AJS.$(this).attr("rel"));
                        gadget.showView(true);
                    });
           		li.addClass("passive");
			}
			else{
           		li.addClass("active");
			}
           	ul.append(li);
		});
		
			div.append(ul);

        return function ()
        {
            return div;
        };
	}();
    var getStatsPanel = function ()
    {
        var counter = 0;
    	var panel = AJS.$("<div/>").addClass("gh-info");
    	var content = AJS.$("<div/>").addClass("bcontent");
    	if(args.ghGadget.board.masterAndDates){
			if(args.ghGadget.board.project){
   				content.append(GADGET.getInfoList(args.ghGadget.board.masterAndDates, "item separated", true, (counter++%2 == 0)));
   			}
   			else{
    			var datePickers = AJS.$("<ul/>").addClass("item separated gh-stripe");
    			datePickers.append(GADGET.getDatePicker(gadget, args.ghGadget.selections.startDate));
				var apply = AJS.$("<a/>").addClass("gh-apply-link").text("Save and Apply");
           		apply.click(function(event){
					gadget.savePref(args.ghGadget.selections.startDate.id, AJS.$('#startDate').val());
					gadget.savePref(args.ghGadget.selections.endDate.id, AJS.$('#endDate').val());
					gadget.showView(true);
				});
				datePickers.append(GADGET.getDatePicker(gadget, args.ghGadget.selections.endDate).append(apply));
				content.append(datePickers);
   				counter++;
   			}
   		}
		content.append(GADGET.getInfoList(args.ghGadget.board.summary.issueTypeStats.stats, "item ul-aa separated", true, (counter++%2 == 0)));
       	if(args.ghGadget.board.summary.progressStats){
       		content.append(GADGET.getInfoList(args.ghGadget.board.summary.progressStats.resolutionStats, "item ul-aa separated", false, (counter++%2 == 0)));
       	}
       	if(args.ghGadget.board.summary.timeTrackingStats){
       		content.append(GADGET.getInfoList(args.ghGadget.board.summary.timeTrackingStats.stats, "item ul-aa separated", true, (counter++%2 == 0)));
       	}
    	if(args.ghGadget.board.summary.watchedFieldStats){
       		content.append(GADGET.getInfoList(args.ghGadget.board.summary.watchedFieldStats.stats, "item separated", false, (counter++%2 == 0)));
       	}
       	panel.append(content);
       	
        return function ()
        {
            return panel;
        };
    }();
    var getChartContainer = function ()
    {
        var chartDiv = AJS.$("<div/>").addClass("chartPane").append(buildMenuBar()).append(displaySelect()).appendTo(getMainContainer());
        var chart = AJS.$("<div id='chart' />").appendTo(chartDiv);
        return function (){
            return chart;
        };
    }();
    var loadGadget = function ()
    {
		var chartImg;
		
		if (!args.ghGadget.selections.isStatsSelected) {
			var src = gadget.getBaseUrl() + "/charts?filename=" + args.ghGadget.board.chart.location;
			gadget.showLoading();
		
			AJS.$("#chart", gadget.getView()).get(0).innerHTML += "<img style='display:none' src='" + src +
                "' title='" + args.ghGadget.board.chart.title +                                                            
				"' alt='" + args.ghGadget.board.name +
				"' height='" + args.ghGadget.board.chart.height + 
				"' width='" + args.ghGadget.board.chart.width + "' />";
			chartImg = AJS.$("img", getChartContainer());
			AJS.$(chartImg, gadget.getView()).load(function () {
				AJS.$(this).show();
				gadget.hideLoading();
				gadget.resize();
			});
		}
		else{
			getStatsPanel().appendTo(getChartContainer());
			gadget.hideLoading();
			gadget.resize();
		}
		return function () {
			return chartImg;
		};
    }();

    if (gadgets.views.getCurrentView().getName() === "canvas"){
        loadGadget();
    }
};

/**
 * Ajax requests for the descriptor
 */
GADGET.descriptorArgs = function(baseUrl) {
	// none at the moment
	return [];
};

/**
 * Descriptor function
 */ 
GADGET.descriptor = function (gadget, args, baseUrl) {
    var searchParam;
    if (/^jql-/.test(gadget.getPref("selectedProjectId")) || gadget.getPref("isPopup") === "true"){
        searchParam =
            {
                userpref: "selectedProjectId",
                type: "hidden",
                value: gadgets.util.unescapeString(gadget.getPref("selectedProjectId"))
            };
    }
    else{
        searchParam = AJS.gadget.fields.projectOrFilterPicker(gadget,"selectedProjectId");
    }
    
    return  {
        action: baseUrl + "/rest/greenhopper/1.0/project-dashboard/validate",
        theme : function(){
            if (gadgets.window.getViewportDimensions().width < 450){
                    return "gdt top-label";
                }
                else{
                    return "gdt";
                }
        }(),
        fields: [
            searchParam,
            {userpref: "showLabels", description:gadget.getMsg("gh.gadget.show.labels.description"), type: "radio", label: gadget.getMsg("gh.gadget.show.labels"), selected: gadget.getPref("showLabels"), options:[{label: gadget.getMsg("gh.gadget.yes"), value:"true"}, {label:gadget.getMsg("gh.gadget.no"),value:"false"}]},
            {userpref: "showLegend", description:gadget.getMsg("gh.gadget.show.legend.description"), type: "radio", label: gadget.getMsg("gh.gadget.show.legend"), selected: gadget.getPref("showLegend"), options:[{label: gadget.getMsg("gh.gadget.yes"), value:"true"}, {label:gadget.getMsg("gh.gadget.no"),value:"false"}]},
            {userpref: "selectedBoardId", type: "hidden", value: gadget.getPref("selectedBoardId")},
            {userpref: "contextName", type: "hidden", value: gadget.getPref("contextName")},
            {userpref: "chartType", type: "hidden", value: gadget.getPref("chartType")},
            {userpref: "startDate", type: "hidden", value: gadget.getPref("startDate")},
            {userpref: "endDate", type: "hidden", value: gadget.getPref("endDate")},
            {userpref: "fieldId", type: "hidden", value: gadget.getPref("fieldId")},
            {userpref: "forMaster", type: "hidden", value: gadget.getPref("forMaster")},
            AJS.gadget.fields.nowConfigured()
        ]
    };
};


GADGET.getDatePicker = function(gadget, date, classes, outlined, even){
	var picker = AJS.$("<li/>");
	picker.append(AJS.$("<label for='" + date.id + "'/>").text(date.label + ": "));
	var datePicker = AJS.$("<input type='text' id='" + date.id + "'>").attr({ value: date.defaultValue, size: 8 });
	var cal = AJS.$("<span id='" + date.id + "_trigger_c'/>").addClass("btn");
	cal.css("background-image", "url('" + gadget.getBaseUrl() + "/images/icons/cal.gif')").append("&nbsp;&nbsp;");
	cal.hover(function(event){
		if(!AJS.$('#'+this.id).hasClass("js-cal")){
			AJS.$('#'+this.id).addClass("js-cal");
			Calendar.setup({firstDay:0,inputField:this.id.split("_")[0],button:this.id,align:"Tl",singleClick:true,ifFormat:AJS.$('#'+this.id.split("_")[0]+"_f").val(),positionStyle:'absolute'});
		}
	});
	picker.append(datePicker);
	picker.append(AJS.$("<input type='hidden' id='" + date.id + "_f'>").attr({ value: date.format }));
	picker.append(cal);
	return picker;
};
GADGET.getInfoList = function(items, classes, outlined, even){
	if(items){
		var list = AJS.$("<ul/>").addClass(classes);
		 if(even){
            list.addClass("gh-stripe");
        }
		for(var i = 0; i < items.length; i++){
			var li  = AJS.$("<li/>");
			if(outlined && i == items.length - 1){
				li.addClass("item-summary");
			}
			var group = AJS.$("<span/>").addClass("group");
			group.text(items[i].name + ":  ").append((AJS.$("<span/>")).addClass("value").text(items[i].value));
			list.append(li.append(group));
		}
		
		return list;
	}
	return "";
};
if (Calendar && !Calendar._TT["WEEKEND"]) Calendar._TT["WEEKEND"] = "0,6";
if (Calendar && !Calendar._TT["DAY_FIRST"]) Calendar._TT["DAY_FIRST"] = "Display %s first";
		    
		    