GADGET = {};

/**
 * Ajax requests for the template
 */
GADGET.templateArgs = function(baseUrl) {
	//Assign wallboard class here so errors also get this class in the body
	if (GH.gadget.utils.getViewName() == 'wallboard') AJS.$('body').addClass('wallboard');
	return [
	    {
	        key: "progress",
	        ajaxOptions: function()
	        {
	    		var gadget = this;
	    		return {
	    	        url: baseUrl + "/rest/greenhopper/1.0/progress-bar-stats/generate",
	    	        data: {
	    	            selectedProjectId: gadgets.util.unescapeString(gadget.getPref("selectedProjectId")),
	    	            selectedBoardId: gadget.getPref("selectedBoardId"),
	    	            contexts: gadget.getPref("contexts"),
	    	            width: Math.min(gadgets.window.getViewportDimensions().width, 800)
	    	        }
	    	    }
	        }
	    }
	];
}

/**
 * Gadget template function
 * 
 * <div class="gg-wrapper gg-qty-3"><!-- gg-qty-N where N is how many are shown -->				
 * 	<!-- REPEAT for each selected context --> 
 * 	<div class="gg-section"> 
 * 		<h3>Context Name</h3>
 * 		<ul class="gg-bargraph"> 
 * 			<!-- only show li if value more than 0 --> 
 * 			<li onclick="URL" class="gg-bar-todo gg-first" style="width: 3%;" title="To Do 20%"></li><!-- add gg-first to first visible list item --> 
 * 			<li onclick="URL" class="gg-bar-progress" style="width: 57%;" title="In Progress 16%"></li> 
 * 			<li onclick="URL" class="gg-bar-done gg-last" style="width: 40%;" title="Done 64%"></li><!-- add gg-last to last visible list item --> 
 * 		</ul> 
 * 		<ul class="gg-legend"> 
 * 			<li class="gg-first"><span class="gg-bar-todo"></span>To Do <a href="URL" target="_parent">7</a></li> 
 * 			<li><span class="gg-bar-progress"></span>In Progress <a href="URL" target="_parent">5</a></li> 
 * 			<li class="gg-last"><span class="gg-bar-done"></span>Done <a href="URL" target="_parent">22</a></li>					
 * 		</ul> 
 * 	</div> 
 * 	<!-- END repeat --> 
 * 	<ul class="gg-legend"> 
 * 		<li class="gg-first"><span class="gg-bar-todo"></span>To Do</li> 
 * 		<li><span class="gg-bar-progress"></span>In Progress</li> 
 * 		<li class="gg-last"><span class="gg-bar-done"></span>Done</li>					
 * 	</ul> 
 * </div> 
 */
GADGET.template = function (gadget, args, baseUrl) {

	/**
	 * Renders the stats legend, either with or without issue count links 
	 */
	var getProgressStatsLegend = function($parent, barItems, includeLinks) {
		var $ulElem = AJS.$('<ul class="gg-legend" />');
		
		var $barItems = AJS.$(barItems);
		var lastIndex = $barItems.size() - 1;
		AJS.$($barItems).each(function(index, barItem) {
			var liClass = '';
			if (index == 0) liClass += ' gg-first';
			if (index == lastIndex) liClass += 'gg-last';
			var spanClass = 'gg-bar-'+barItem.className;
			var url = gadget.getBaseUrl() + barItem.location;
			var issueCount =  includeLinks ? ' <a href="'+url+'" target="_parent">'+barItem.value+'</a>' : '';
			AJS.$('<li class="'+liClass+'"><span class="'+spanClass+'"></span>'+barItem.name+issueCount+'</li>').appendTo($ulElem); 
		});
		$parent.append($ulElem);
	}
	
	/**
	 * Renders a progress stats bar
	 */
	var getProgressStatsBar = function($parent, statsSummary) {
		var contextName = statsSummary.contextName;
		var sectionHtml = '' 
			+ '<div class="gg-section">'
			+ '  <h3>'+contextName+'</h3>'
			+ '</div>';
		var $ggSection = AJS.$(sectionHtml).appendTo($parent);

		// remove all that are empty
		var $barItems = AJS.$(statsSummary.progressStats.barItems);
		var filteredItems = [];
		$barItems.each(function(index, item) {
			if (item.value != "0") {
				filteredItems.push(item);
			} else {
				var a = 0;
			}
		});
		$barItems = AJS.$(filteredItems);
		
		// now create all links if we got bars
		if ($barItems.size() > 0) {
			var $ulElem = AJS.$('<ul class="gg-bargraph" />').appendTo($ggSection);
			var lastIndex = $barItems.size() - 1;
			$barItems.each(function(index, barItem) {
				var className = 'gg-bar-' + barItem.className;
				if (index == 0) className += ' gg-first';
				if (index == lastIndex) className += ' gg-last';
				var url = gadget.getBaseUrl() + barItem.location;
				AJS.$('<li onclick="javascript:GH.gadget.utils.redirectParent(\''+url+'\');" class="'+className+'" style="width: '+barItem.ratio+';" title="'+barItem.title+'"></li>').appendTo($ulElem);
			});
			
			// add the summary
			getProgressStatsLegend($ggSection, statsSummary.progressStats.barItems, true);
		} else {
			var message = gadget.getMsg("gh.gadget.noissues");
			AJS.$('<div class="gg-no-issues">'+message+'</div>').appendTo($ggSection);
		}
	};
	
	// empty content and set root classes
	gadget.getView().empty();
	var body = AJS.$('body');
	body.addClass('greenhopper-project-progress');
	
	// gg-wrapper element
	var $statsSummaries = AJS.$(args.progress.progressStatsList);
	var wrapperHtml = '<div class="gg-wrapper gg-qty-' + $statsSummaries.size() + '"></div>'
	var $ggWrapper = AJS.$(wrapperHtml).appendTo(gadget.getView());
	
	// add the title
	var $headerHtml = GH.gadget.utils.addProjectVersionHeader(gadget, args.progress, gadget.getView());
	
	// add all sections
	$statsSummaries.each(function(index, statsSummary) {
		getProgressStatsBar($ggWrapper, statsSummary);
    });
	
	// add the last legend
	if ($statsSummaries.size() > 0) {
		getProgressStatsLegend($ggWrapper, $statsSummaries.get(0).progressStats.barItems, false);
	}
};

/**
 * Ajax requests to be executed for the descriptor
 */
GADGET.descriptorArgs = function(baseUrl) {
    return [
        {
            key: "projectOptions",
            ajaxOptions:  baseUrl + "/rest/greenhopper/1.0/project-list"
        }
    ];
};

/**
 * Gadget descriptor
 */
GADGET.descriptor = function (gadget, args, baseUrl) {
	//[GHS-2677] 
	//Root cause is that the existing content is not cleared when an error is added as a result of a gadget refresh
	//To fix this we hide existing content when this class is appended into the body
	//Calling descriptor should remove class indicating invalidness
	AJS.$("body").removeClass('config-unavailable');
    return  {
        action: baseUrl + "/rest/greenhopper/1.0/progress-bar-stats/validate",
        theme : function()
        {
            if (gadgets.window.getViewportDimensions().width < 450)
            {
                return "gdt top-label";
            }
            else
            {
                return "gdt";
            }
        }(),
        fields: [
           GH.gadget.fields.projectPicker(gadget, "selectedProjectId", args.projectOptions.options),
           GH.gadget.fields.versionPicker(gadget, "selectedProjectId", "selectedBoardId", false, true),
           GH.gadget.fields.contextPicker(gadget, "selectedProjectId", "contexts", true),
//           GH.gadget.fields.sectionTitle(gadget, 'displayOptions', 'gh.gadget.common.displayoptions.label'),
           GH.gadget.fields.checkbox(gadget, "showProjectName", 'gh.gadget.common.options.projectname', gadget.getPref('showProjectName') == 'true'),
           GH.gadget.fields.checkbox(gadget, "showVersionName", 'gh.gadget.common.options.versionname', gadget.getPref('showVersionName') == 'true'),
           AJS.gadget.fields.nowConfigured()
        ]
    };
};

