GADGET = {};

/**
 * Ajax requests for the template
 */
GADGET.templateArgs = function(baseUrl) {
	//Assign wallboard class here so errors also get this class in the body
	if (GH.gadget.utils.getViewName() == 'wallboard') AJS.$('body').addClass('wallboard');
	return [
	    {
	        key: "task",
	        ajaxOptions: function()
	        {
	    		var gadget = this;
	    		return {
	    	        url: baseUrl + "/rest/greenhopper/1.0/task-board/generate.json",
	    	        data: {
	    	            selectedProjectId: gadgets.util.unescapeString(gadget.getPref("selectedProjectId")),
	    	            selectedBoardId: gadget.getPref("selectedBoardId"),
	    	            contexts: gadget.getPref("contexts"),
	    	            watchedField: gadget.getPref("watchedField"),
	    	            tbMappings: gadget.getPref("tbMappings"),
	    	            width: Math.min(gadgets.window.getViewportDimensions().width, 800)
	    	        }
	    	    }
	        }
	    }
	];
}

//This is the threshold for dots
GADGET.DOT_LIMIT = 8;

/**
 * Gadget template function
 * 
 */
GADGET.template = function (gadget, args, baseUrl) {
	
	//TEMPLATE for column stats/capacities
	var getColumnStats = function($parent, column){
		//Container for the stats
		var columnStatsHtml = '<div class="gg-column-stats"></div>';
		var $columnStatsWrapper = AJS.$(columnStatsHtml).appendTo($parent);
		
		//Min value if it exists
		if(column.displayMinCapacity != null){
			var minConstraintHtml = ''
			+ '<span class="gg-column-constraint gg-constraint-min">' + column.displayMinCapacity + '</span><span> / </span>';
			AJS.$(minConstraintHtml).appendTo($columnStatsWrapper);
		}
		
		// Count for the field
		var countHtml = '<span class="gg-column-count">' + column.displayValue + '</span>';
		AJS.$(countHtml).appendTo($columnStatsWrapper);
		
		//Max value if it exists
		if(column.displayMaxCapacity != null){
			var maxConstraintHtml = ''
			+ '<span> / </span><span class="gg-column-constraint gg-constraint-max">' + column.displayMaxCapacity + '</span>';
			AJS.$(maxConstraintHtml).appendTo($columnStatsWrapper);
		}
	};
	
	//TEMPLATE for column info
	var getColumnInfo = function($parent, column){
		var infoHtml = '<div class="gg-column-info"><h1>' + column.name + '</h1></div>';
		var $infoWrapper = AJS.$(infoHtml).appendTo($parent);
		// Get stats for the capacity now
		getColumnStats($infoWrapper, column);
	};
	
	//TEMPLATE for issues within a column
	var getIssue = function($parent, issue, column){
		var avatarUrl = '';
		//If an avatar exists then use the avatar, if not use default
		if(issue.hasAvatar){
			avatarUrl = gadget.getBaseUrl() + issue.avatarUrl;
		}else{
			avatarUrl = issue.avatarUrl;
		}
		var stayBusted = '';
		if(issue.daysInStatus > GADGET.DOT_LIMIT){
			stayBusted = ' gg-stay-long';
		}
		var isDone = '';
		if(column.isLast){
			isDone = ' gg-issue-done';
		}
		var isFlagged = '';
		if(issue.isFlagged){
			isFlagged = ' gg-flagged';
		}
		var issueHtml = ''
			+ '<div class="gg-issue gg-stay-' + issue.daysInStatus + stayBusted + isDone + isFlagged + ' gg-unassigned">'
			+ 	'<ul>'
			+ 		'<li class ="gg-issue-key">' + issue.key + '</li>'
			+ 		'<li class ="gg-issue-avatar"><img width="48" height="48" alt="projectavatar" src="' + avatarUrl + '"></li>'
			+ 		'<li class ="gg-issue-summary">' + issue.summary + '</li>'
			+ 		'<li class ="gg-issue-stay" style="background-color: ' + issue.cardColor + ';"><em></em></li>'
			+ 	'</ul>'
			+'</div>';
		AJS.$(issueHtml).appendTo($parent);
	};
	
	//TEMPLATE for column
	var getColumn = function($parent, column){
		//Wrapper for each column
		var wrapperHtml = '<div class="gg-column-wrapper"></div>';
		var $wrapper = AJS.$(wrapperHtml).appendTo($parent);
		//Add busted classes to busted columns
		if(column.minBusted){ $wrapper.addClass('gg-busted gg-busted-min');}
		if(column.maxBusted){ $wrapper.addClass('gg-busted gg-busted-max');}
		
		//More wrappers
		var columnHtml = '<div class="gg-column"></div>';
		var $columnWrapper = AJS.$(columnHtml).appendTo($wrapper);
		var innerColumnHtml = '<div class="gg-column-inner"></div>';
		var $innerColumnWrapper = AJS.$(innerColumnHtml).appendTo($columnWrapper);
		
		//Add header to the column
		getColumnInfo($innerColumnWrapper, column);
		
		//Create issues wrapper
		var issuesHtml = '<div class="gg-column-issues"></div>';
		var $issuesWrapper = AJS.$(issuesHtml).appendTo($innerColumnWrapper);		
		
		//go through each issue
		var $issues = AJS.$(column.issues);
		$issues.each(function(index, issue){
			getIssue($issuesWrapper, issue, column);
		});
	};
	
	//START//////////////////////////////////
	// empty content and set root classes
	gadget.getView().empty();
	var body = AJS.$('body');
	body.removeClass(function(index, className) {
		var regex = /\bgg-column-qty-\d+/;//TODO handle double digits??
		var matches = className.match(regex); 
		if( matches != null){
			return matches.toString().replace(',', ' ');//toString makes comma seperated, requires space seperated
		}
		return '';
    });
	body.addClass('greenhopper-task-board');
	body.addClass('gg-column-qty-' + args.task.columnCount);
    
    // create the items for the title
	var titleElements = GH.gadget.utils.getProjectVersionHeaderElements(gadget, args.task);
    var showContextName = gadget.getPref('showContextName') == 'true';
    if(showContextName){
    	titleElements.push(args.task.contextName);
	}
	GH.gadget.utils.insertHeaderElements(titleElements, gadget.getView());
	
	//Add all columns
	var $columns = AJS.$(args.task.columns);
	$columns.each(function(index, column){
		getColumn(gadget.getView(), column);
	});

    GADGET.scaleCards();
};

/**
 * Ajax requests to be executed for the descriptor
 */
GADGET.descriptorArgs = function(baseUrl) {
    return [
        {
            key: "projectOptions",
            ajaxOptions:  baseUrl + "/rest/greenhopper/1.0/project-list"
        }
    ];
};

/**
 * Gadget descriptor
 */
GADGET.descriptor = function (gadget, args, baseUrl) {
	//[GHS-2677] 
	//Root cause is that the existing content is not cleared when an error is added as a result of a gadget refresh
	//To fix this we hide existing content when this class is appended into the body
	//Calling descriptor should remove class indicating invalidness
	AJS.$("body").removeClass('config-unavailable');
    return  {
        action: baseUrl + "/rest/greenhopper/1.0/task-board/validate.json",
        theme : function()
        {
            if (gadgets.window.getViewportDimensions().width < 450)
            {
                return "gdt top-label";
            }
            else
            {
                return "gdt";
            }
        }(),
        fields: [
           GH.gadget.fields.projectPicker(gadget, "selectedProjectId", args.projectOptions.options),
           GH.gadget.fields.versionPicker(gadget, "selectedProjectId", "selectedBoardId", true, true),
           GH.gadget.fields.contextPicker(gadget, "selectedProjectId", "contexts", false),
           GH.gadget.fields.tbMappingPicker(gadget, "selectedProjectId", "tbMappings"),
           GH.gadget.fields.fieldPicker(gadget, "selectedProjectId", "watchedField", true),
           GH.gadget.fields.checkbox(gadget, "showProjectName", 'gh.gadget.common.options.projectname', gadget.getPref('showProjectName') == 'true'),
           GH.gadget.fields.checkbox(gadget, "showVersionName", 'gh.gadget.common.options.versionname', gadget.getPref('showVersionName') == 'true'),
           GH.gadget.fields.checkbox(gadget, "showContextName", 'gh.gadget.common.options.contextname', gadget.getPref('showContextName') == 'true'),
           AJS.gadget.fields.nowConfigured()
        ]
    };
};

// Dynamic Card Sizing
GADGET.scaleCards = function() {
    var $ = AJS.$;
    var cards = $('.gg-issue');
    var cardMinSize = 190;
    var cardMargin = parseInt(cards.css('margin-right'));

    var cardBorder = parseInt(cards.css('border-left-width')) + parseInt(cards.css('border-right-width'));
    if (isNaN(cardBorder)) cardBorder = 0;

    // we optimise the right margin of the right card in each column away later.
    // So for getting the computation right, we assume the column to be one margin larger.
    var columnWidth = $($('.gg-column-inner')[0]).width() - 1 + cardMargin;

    var realCardSize = cardMinSize + cardMargin + cardBorder;

    // the amount of space that's left in the column, assuming the minimum card size
    var remainder = columnWidth % realCardSize;
    var possibleNumberOfCards = parseInt(columnWidth / realCardSize);

    // we adjust the card size by the rounded-down share of the remaining space. Since we never
    // actually count the amount of cards really in the column, the space distribution will operate
    // on "virtual" cards, so even resize a single card. This is the intended behaviour.
    var adjustment = parseInt(remainder / possibleNumberOfCards);

    cards.css('width', cardMinSize + adjustment);

    // now we go and shove the right margin off every last card in each row (see columnWidth definition) 
    $('.gg-column-issues .gg-issue:nth-child(' + possibleNumberOfCards + 'n+' + possibleNumberOfCards + ')').css({'margin-right':0});
};
