/**
 * @constructor
 * @extends JIRA.Dialog
 */
JIRA.IssueActionsDialog = JIRA.Dialog.extend({

    _getDefaultOptions: function () {
        return AJS.$.extend(this._super(), {
            cached: false,
            id: "issue-actions-dialog",
            widthClass: "small"
        });
    },

    _setContent: function (content, decorate) {
        if (content) {
            this._super(content, decorate);
        } else {
            this._super(AJS.$([
                "<form id='issue-actions-dialog-form' class='aui'>",
                    "<div class='content-body'>",
                        "<div id='issueactions-suggestions' class='aui-list' />",
                        "<div class='description'>",
                        AJS.params.issueActionsHint,
                        "</div>",
                    "</div>",
                "</form>"
            ].join('')), true);
        }
        if (JIRA.Dialog.current === this) {
            var triggerConfig = new JIRA.setFocus.FocusConfiguration();
            triggerConfig.context = this.get$popup()[0];
            triggerConfig.parentElementSelectors = ['.content-body'];
            JIRA.setFocus.pushConfiguration(triggerConfig);
            JIRA.setFocus.triggerFocus();
        }
    },

    _formatActionsResponse: function (response) {

        function addSelected(issueId) {

             var url = window.location.href,
                newUrl = url;

            if (/selectedIssueId=[0-9]*/.test(url)) {
                newUrl = newUrl.replace(/selectedIssueId=[0-9]*/g, "selectedIssueId=" + issueId);
            } else {
                if (JIRA.IssueNavigator.isNavigator()) {
                    if (/\?/.test(url)) {
                        newUrl = newUrl + "&";
                    } else {
                        newUrl = newUrl + "?";          
                    }
                    newUrl = newUrl + "selectedIssueId=" + issueId;
                }
            }
            return encodeURIComponent(newUrl);
        }

        function addSelectedGH(issueKey) {
        	return GH.IssueActionsDialog.generateReturnUrl(issueKey);
        }
        
        function formatWorkflowResponse(workflowResponse) {

            var workflows = new AJS.GroupDescriptor({
                label: AJS.params.workflow
            });

            AJS.$(workflowResponse).each(function() {

            	// GreenHopper: make sure the issue id is not hardcoded, thus inserted by getAjaxOptions.
                workflows.addItem(new AJS.ItemDescriptor({
                    href: contextPath + "/secure/WorkflowUIDispatcher.jspa?id={0}&action=" + this.action + "&atl_token=" + response.atlToken + "&returnUrl=" + addSelectedGH(response.key),
                    label: this.name,
                    styleClass: "issueaction-workflow-transition"
                }));

            });

            return workflows;
        }

        function formatOperationResonse(operationsResponse) {

            var operations = new AJS.GroupDescriptor({
                label: AJS.params.actions
            });

            AJS.$(operationsResponse).each(function() {

                var _returnUrl;
/*
                if (this.name === "Clone") {
                    if (JIRA.IssueNavigator.isNavigator()) {
                          _returnUrl=addSelected(response.id);
                    } else {
                          _returnUrl="";
                    }
                } else {
*/                    _returnUrl=addSelectedGH(response.key);
/*                }
*/

                operations.addItem(new AJS.ItemDescriptor({
                    href: this.url + "&returnUrl=" + _returnUrl,
                    label: this.name,
                    styleClass: this.styleClass
                }));
            });
            return operations;
        }

        var res = [];

        if (response) {
            if (response.actions && response.actions.length != 0) {
                res.push(formatWorkflowResponse(response.actions));
            }

            if (response.operations && response.operations.length != 0) {
                res.push(formatOperationResonse(response.operations));
            }

        }

        return res;
    },

    decorateContent: function () {

        var instance = this,
            issueKey = JIRA.IssueNavigator.getSelectedIssueKey(),
            issueId = JIRA.Issue.getIssueId() || JIRA.IssueNavigator.getSelectedIssueId();

        if (issueKey) {
            this.addHeading(AJS.params.dotOperations + ": <span>" + issueKey + "</span>");
        } else {
            this.addHeading(AJS.params.dotOperations);
        }

        this.queryControl = new AJS.QueryableDropdownSelect({
            id: "issueactions",
            element: this.$content.find("#issueactions-suggestions"),
            ajaxOptions: {
                minQueryLength: 1,
                dataType: "json",
                url: AJS.format(contextPath + "/rest/api/1.0/issues/{0}/ActionsAndOperations?atl_token={1}", issueId, atl_token()),
                formatResponse: this._formatActionsResponse
            },
            showDropdownButton: true,
            loadOnInit: true
        });
        this.queryControl._handleServerError  = function(smartAjaxResult) {
            var errMsg = JIRA.SmartAjax.buildSimpleErrorContent(smartAjaxResult);
            var errorClass = smartAjaxResult.status === 401?'warn':'error';
            instance._setContent(AJS.$('<div class="ajaxerror"><div class="notify ' + errorClass+'"><p>' + errMsg + '</p></div></div>'), false);
            instance._addCloseLink();
        };


        this.timeoutId = undefined;
        this._addCloseLink();

    },

    _addCloseLink : function() {
        var instance=this,$closeLink, $buttonContainer, $buttons;
        
        $buttonContainer = AJS.$('<div class="buttons-container content-footer"></div>').appendTo(this.get$popupContent());
        $buttons = AJS.$('<div class="buttons"/>').appendTo($buttonContainer);
        $closeLink = AJS.$("<a href='#' class='cancel' id='aui-dialog-close'>" + AJS.params.closelink + "</a>");
        $closeLink.appendTo($buttons, this.get$popupContent()).click(function(e) {
            instance.hide();
            e.preventDefault();
        });
        this.get$popupContent().append($buttonContainer);
    },

    hide: function (undim) {

        if (this._super(undim) === false) {
            return false;
        }

        JIRA.setFocus.popConfiguration();        
    }
});

/**
 * Contains extensions for the issueActions dialog.
 */
GH.IssueActionsDialog = {};

/**
 * Old greenhopper code.
 */
if (window.Boards) {
	/**
	 * Define the return url to use
	 */
	GH.IssueActionsDialog.generateReturnUrl = function(issueKey) {
		// we want the user to be kicked back to the current board - getBoardForIssue2 will do that for us
		var boardId = Boards.mainBoard.id;
		var newUrl = '/secure/GetBoardForIssue2.jspa?pageType='+Boards.pageType+'&subType='+Boards.subType+'&selectedBoardId='+boardId+'&key='+issueKey+'&committed=false';
		return encodeURIComponent(newUrl);
	};
	
	/**
	 * Wires the close dialog link to the handleCancel function
	 * (which is by default only called when closing the dialog via escape).
	 */
	GH.IssueActionsDialog.wireCloseLinkToHandleCancel = function() {
		var $closeLink = this.get$popupContent().find('a.cancel');
		// remove original click handler
		$closeLink.unbind('click');
		var popup = this;
		// and bind a slightly adapted one
		$closeLink.click(function(e){
		    popup.handleCancel.call(popup);
		    e.preventDefault();
		});
	};
	
	/**
	 * Temporarily hide the search screen if displayed.
	 */
	GH.IssueActionsDialog.hideSearchDialog = function() {
		if(Boards.inSearchMode){
			Boards.hideSearchPopup();
		}
	};
	
	/** Register onContentReady handlers/ */
	AJS.$(function () {
		
		/**
		 * overwrite what we do when handleCancel is called
		 */
		JIRA.Dialogs.issueActions.handleCancel=function(){
			if (JIRA.Dialog.current === this) {
				this.hide();
				if(Boards.inSearchMode){
		        	Boards.returnToSearch();
				}
			}
		};
		
		// Attach both function to the popup, they get executed right before the dialog is displayed
		JIRA.Dialogs.issueActions.onContentReady(GH.IssueActionsDialog.hideSearchDialog);
	
		// make sure handleCancel is called when the close link is clicked
		JIRA.Dialogs.issueActions.onContentReady(GH.IssueActionsDialog.wireCloseLinkToHandleCancel);
		
		// also clear the workflow resolutions
		JIRA.Dialogs.issueActions.onContentReady(GH.clearAllowedWorkflowResolutions);
	});
}

/**
 * RapidBoard
 */
else {
	
	/**
	 * TODO: what return url should we use?
	 */
	GH.IssueActionsDialog.generateReturnUrl = function(issueKey) {
		return GH.RapidBoard.getCurrentPageReturnUrl();
	};
}
