/**
 * Contains IssueActions.Shortcuts.js adapted code.
 * 
 * This code deals with displaying success messages upon issue action completion.
 */

GH.Message = {};

/**
 * Adaption of IssueNavigator.Shortcuts.js -> flashIssueRow()
 */
GH.Message.showIssueMessage = function(issueId) {
	var sessionstorage = JIRA.SessionStorage;
    var selectedIssueMsg = null;
    var selectedIssueKey = null;
    if (! issueId)
    {
        if (! issueId)
        {
            // ok try session storage next
            issueId = sessionstorage.getItem('selectedIssueId');
        }
    }
    if (issueId)
    {
        selectedIssueKey = sessionstorage.getItem('selectedIssueKey');
        selectedIssueMsg = sessionstorage.getItem('selectedIssueMsg');
        GH.Message.showIssueMessageImpl(issueId, selectedIssueMsg, selectedIssueKey);
    }
    // always remove the state.  We want to be empty until some other action decides it wants to record this
    sessionstorage.removeItem('selectedIssueId');
    sessionstorage.removeItem('selectedIssueKey');
    sessionstorage.removeItem('selectedIssueMsg');
};



/**
 * Adaption of IssueNavigator.Shortcuts.js -> flashIssueRowWithId
 */
GH.Message.showIssueMessageImpl = function(issueId, selectedIssueMsg, selectedIssueKey) {
    // if we don't have an message, use a generic one if possible
    if (! selectedIssueMsg)
    {
        selectedIssueMsg = 'thanks_issue_updated';
    }

    var msgText = AJS.params[selectedIssueMsg];
    if (msgText && selectedIssueKey)
    {
        msgText = AJS.format(msgText, selectedIssueKey);
        GH.Message.showSuccess(msgText);
    }
};

GH.Message.prepareNotificationDiv=function() {
	// check whether we have to add the notification div (RapidBoard)
	var notificationDiv = AJS.$('#gh-notification');
	if (notificationDiv.length == 0) {
		notificationDiv = AJS.$('<div id="gh-notification"></div>');
		AJS.$('#gh').append(notificationDiv);
	}
    // remove any old content
    notificationDiv.text('');
    return notificationDiv;
};

GH.Message.showSuccess=function(message) {
	// fetch the div use use to display the message
	var notificationDiv = GH.Message.prepareNotificationDiv();
	
    // add the new message, then display with a timeout
	AJS.messages.success(notificationDiv, {body: message});
	notificationDiv.show();
	setTimeout('AJS.$("#gh-notification").hide()', 5000);
};

GH.Message.showWarning=function(message) {
	// fetch the div use use to display the message
	var notificationDiv = GH.Message.prepareNotificationDiv();
	
    // add the new message, then display with a timeout
	AJS.messages.warning(notificationDiv, {body: message, closeable: true});
	notificationDiv.show();
};

// Register a document ready handler to show any not yet displayed message
AJS.$(document).ready(function () {
	GH.Message.showIssueMessage();
});
