GH.Projector = {};

/**
 * Is projector mode currently enabled?
 */
GH.Projector.isEnabled = function() {
	if (GH.RapidBoard) {
		return GH.storage.get('gh.projector', true) ? true : false;
	} else {
		return GH.Cookie.getCookie('ghProjector') == '1';
	}
};

/**
 * Sets the projector mode to enabled or disabled.
 * Triggers a UI refresh
 */
GH.Projector.setEnabled = function(enabled) {
	if (GH.RapidBoard) {
		if (enabled) {
			GH.storage.put('gh.projector', true, true);
		} else {
			GH.storage.put('gh.projector', null, true);
		}
	} else {
		if (enabled) {
			GH.Cookie.setCookie('ghProjector','1');
		} else {
			GH.Cookie.deleteCookie('ghProjector');
		}
	}
	GH.Projector.updateUI();
};

/**
 * Toggle projector mode enabled state
 */
GH.Projector.toggleEnabled = function() {
	GH.Projector.setEnabled(! GH.Projector.isEnabled());
};

/**
 * Toggle Projector Mode
 */
GH.Projector.toggleUI = function(){
	GH.Projector.toggleEnabled();
};

/**
 * Updates the UI according to the projector state
 */
GH.Projector.updateUI = function(initialLoad){
	var enabled = GH.Projector.isEnabled();
	
	if (enabled) {
		// set the correct base class
        AJS.$('#gh, #jira').addClass('ghx-projector');
        
        // re-render swimlanes to fix heights of columns
        if (!initialLoad && GH.RapidBoard && GH.RapidBoard.SwimlaneView && GH.RapidBoard.SwimlaneView.renderSwimlanes) {
        	if(AJS.$('#gh').hasClass('ghx-board-rapid')){
        		GH.RapidBoard.ViewController.layoutFixedElements();
        	}
        }
	} else {
		// remove project mode class 
		if (! initialLoad) {
			AJS.$('#gh, #jira').removeClass('ghx-projector');
			if (GH.RapidBoard && GH.RapidBoard.ViewController) {
			    // re-position the fixed elements, like detailview, since the page layout changed
			    // from moving out of projector mode
			    GH.RapidBoard.ViewController.layoutFixedElements();
			}
		}
	}
};

/**
 * Initialise launch sequence...
 */
GH.Projector.init = function(){
	// simply update the ui
	GH.Projector.updateUI(true);
};

// Go go go!
AJS.$(GH.Projector.init);
