/**
 * Ajax Select input type support.
 */
GH.CardField.AjaxSelect = {
        
    /** Starts field editing. */
    editField : function(fieldInfo, editFieldData)
    {
        var field = fieldInfo.elem;
        var fieldWidth = AJS.$(field).width();
        var fieldWidthCorner = fieldWidth + 10;
        var fieldPos = AJS.$(field).position();
        var fieldPosTop = fieldPos.top;
        var fieldPosLeft = fieldPos.left + 1; // 1 is the magic number = border width

        // generate edit dom
        var editDom = AJS.$(GH.tpl.ajaxselect.renderUserPicker({
            name: "userpickerSelect",
            data: editFieldData
        }));
        var editDomField = editDom.find("select");

        fieldInfo.editDom = editDom;
        fieldInfo.editDomField = editDomField;
        fieldInfo.editFieldData = editFieldData;

        // update UI
        field.addClass('gh-edit-active').closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').addClass('gh-edit-active gh-edit-active-ajaxselect'); // Some styles change while edit is happening
        //field.children().css({'visibility':'hidden'}); // hide the normal content
        //field.closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').append(editDom);
        AJS.$(field).children().hide() // hide the normal content
        .end().append(editDom);

        if (AJS.$(field).hasClass('gh-issue-corner')){
            AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').addClass('gh-edit-corner');
        }
        // if the field is in a crowded issue, the left position is where the body starts, width based on issue width
        if (AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').find('.gh-issue').hasClass('gh-issue-crowded')){
            var fieldHead = AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').find('.gh-subhead');
            fieldPosLeft = AJS.$(fieldHead).width() + parseInt(AJS.$(fieldHead).css('margin-left')) + parseInt(AJS.$(fieldHead).css('border-left-width'));
            fieldWidth = AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent').width() * 0.7; // 70% of issue width
        }
        AJS.$(editDom).find('.gh-edit-contents')
                .css({
                   // 'top':fieldPosTop, 'left':fieldPosLeft,
                    'width':fieldWidth})
                .focus(); // focus and position the form field
        
        GH.SingleSelect = AJS.SingleSelect.extend({
        	submitForm: function () {
                if (!this.suggestionsVisible) {
                    this.handleFreeInput();
                    //AJS.$(this.$field[0].form).submit(); // submit on enter if field is empty
                    GH.CardField.AjaxSelect._finishEdit(fieldInfo);
                }
            },
            _handleEscape: function (e) {
                if (this.suggestionsVisible) {
                    e.stopPropagation();
                    if (e.type === "keyup") {
                        this.hideSuggestions();
                    }
                }
                if (e.type === "keyup") {
                	GH.CardField.AjaxSelect.cancelEdit(fieldInfo);
                }
            },
            _deactivate: function () {
            	this.acceptFocusedSuggestion();
                this.hideSuggestions();
                GH.CardField.AjaxSelect._finishEdit(fieldInfo);
            }
        });
        
        // setup the ajax select
        var selectObject = new GH.SingleSelect({
            element: editDomField,
            showDropdownButton: false,
            removeOnUnSelect: true,
            submitInputVal: true,
            // overlabel: AJS.I18n.getText("user.picker.ajax.short.desc"),
            errorMessage: AJS.I18n.getText("admin.errors.invalid.user"),
            ajaxOptions: {
                url: contextPath + editFieldData.restUrl,
                query: true, // keep going back to the sever for each keystroke
                minQueryLength: 0,
                data: {
                    fieldName: fieldInfo.fieldId,
                    issueKey: editFieldData.issueKey,
                    issueId: editFieldData.issueId,
                    showAvatar: true
                },
                formatResponse: function (response) {
                    var ret = [];
                    AJS.$(response).each(function(i, suggestions) {
                        var groupDescriptor = new AJS.GroupDescriptor({
                            weight: i, // order or groups in suggestions dropdown
                            id: "user-suggestions",
                            replace: true,
                            label: suggestions.footer // Heading of group
                        });

                        AJS.$(suggestions.users).each(function(){
                            groupDescriptor.addItem(new AJS.ItemDescriptor({
                                value: this.name, // value of item added to select
                                label: this.displayName, // title of lozenge
                                html: this.html,
                                allowDuplicate: false,
                                icon: this.avatarUrl
                            }));
                        });
                        ret.push(groupDescriptor);
                    });

                    return ret;
                }
            }
        });
        editDomField.data('selectObject', selectObject);
        fieldInfo.selectObject = selectObject;

        // select current value so that when user types the current value is cleared
        fieldInfo.editDomField.select();
        
        // submit when a new value is selected
        editDomField.bind("selected", function(event, data) {
            GH.CardField.AjaxSelect._finishEdit(fieldInfo);
        });
        
        // temp hack to test if truly a list not list-as-card
        if (AJS.$('#gh').hasClass('gh-list')){
            if (!AJS.$(editDom).parents().hasClass('gh-card-preview')){
                AJS.$(editDom).closest('.gh-issue-wrapper').addClass('gh-list-as-list');
            }
        else {
                AJS.$(editDom).closest('.gh-issue-wrapper').addClass('gh-list-as-card');
            }
        }
    },
    
    _finishEdit : function(fieldInfo) {
        // ensure we only run once through this path
        if (fieldInfo.finishSubmitted) return;
        fieldInfo.finishSubmitted = true;
        
        // fetch the value to submit
        var selectObject = fieldInfo.selectObject;
        var descriptor = selectObject.getSelectedDescriptor();
        var fieldValue = null;
        if (descriptor) {
        	fieldValue = descriptor.value();
        } else {
        	// we got no value currently. simply cancel
        	GH.CardField.AjaxSelect.cancelEdit(fieldInfo);
        	return;
        }

        // only actually submit if the values differ
        if (fieldValue !== fieldInfo.editFieldData.value) {
            GH.CardField.submitFieldValue(fieldInfo, fieldValue);
        } else {
            GH.CardField.AjaxSelect.cancelEdit(fieldInfo);
        }
    },
    
    /**
     * Cancels an edit operation. This is called in case update produced an error.
     */
    cancelEdit: function(fieldInfo) {
        var field = fieldInfo.elem;
        var wrapper = AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper');
        fieldInfo.editDomField
            .unbind('blur'); // unregister blur handler
        fieldInfo.editDom
            .remove();
        AJS.$(field)
            .removeClass('gh-edit-active')
            .children().show(); //css({'visibility':'visible'}); // show all previously hidden children
        
        // clean wrapper
        GH.CardField.AjaxSelect._cleanWrapper(wrapper, field);
    },
    
    _cleanWrapper: function(wrapper, field) {
    	// remove this field types class - todo: this is unclean! 
        wrapper.removeClass('gh-edit-active-ajaxselect');
        
        // only remove edit-active if we are the only edited field
        if (wrapper.find('.gh-edit-active').length < 1) {
        	wrapper.removeClass('gh-edit-active  gh-list-as-list');
        }
        
        // remove corner if we are the corner field
        if (AJS.$(field).hasClass('gh-issue-corner')) {
            wrapper.removeClass('gh-edit-corner');
        }
    },
    
    /**
     * Updates an existing field dom element with new values.
     * @param the field element to update
     * @param fieldData the data object to use
     */
    updateField : function(field, fieldData) {    	
        // render the new dom given the new field data
        var elem = AJS.$('<div class="gh-editable"></div>');
        elem.attr('title', fieldData.data.title);
        AJS.$(elem).text(fieldData.data.value);
        if (!fieldData.data.isCorner) {
            AJS.$('<em>').text(fieldData.data.label + ":").prependTo(elem);
        }
        
        // adapt field and wrapper...
        field = AJS.$(field);
        var wrapper = field.closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper');
        
        // remove current content
        field.children().remove();
        // add the new content
        AJS.$(field).append(elem);

        // mark field as not in edit mode anymore
        AJS.$(field).removeClass('gh-edit-active');
        
        // clean wrapper
        GH.CardField.AjaxSelect._cleanWrapper(wrapper, field);
    },

    /**
     * Autocomplete-Selects for the "create card" dialog. They work different from the inline-editing ones, since they don't have the open/close logic
     * and will be submitted with the form rather than on their onw.
     */
    editCreateField: function(fieldId, restUrl, minQueryLength, projectId) {
        var field = AJS.$('#newField_' + fieldId);

        GH.CreateSingleSelect = AJS.SingleSelect.extend({
            _deactivate: function () {
            	this.acceptFocusedSuggestion();
                this.hideSuggestions();
            }
        });
        
        var selectObject = new GH.CreateSingleSelect({
            element: field,
            showDropdownButton: false,
            removeOnUnSelect: true,
            submitInputVal: true,
            overlabel: AJS.I18n.getText("user.picker.ajax.short.desc"),
            errorMessage: AJS.I18n.getText("admin.errors.invalid.user"),
            ajaxOptions: {
                url: contextPath + restUrl,
                query: true, // keep going back to the sever for each keystroke
                minQueryLength: minQueryLength,
                data: {
                    fieldName: fieldId,
                    projectId: projectId,
                    showAvatar: true
                },
                formatResponse: function (response) {
                    var ret = [];
                    AJS.$(response).each(function(i, suggestions) {
                        var groupDescriptor = new AJS.GroupDescriptor({
                            weight: i, // order or groups in suggestions dropdown
                            id: "user-suggestions",
                            replace: true,
                            label: suggestions.footer // Heading of group
                        });

                        AJS.$(suggestions.users).each(function(){
                            groupDescriptor.addItem(new AJS.ItemDescriptor({
                                value: this.name, // value of item added to select
                                label: this.displayName, // title of lozenge
                                html: this.html,
                                allowDuplicate: false,
                                icon: this.avatarUrl
                            }));
                        });
                        ret.push(groupDescriptor);
                    });

                    return ret;
                }
            }
        });

        field.data('selectObject', selectObject);
        
        // The SingleSelect selects the current field upon initialization, on the create card dialog we want the summary to be selected instead...
        Boards.ffocus('newField_summary');
    }
};

// register controller
GH.CardField.controllers['ajaxselect'] = GH.CardField.AjaxSelect;
