/**
 * TextArea input type support.
 */
GH.CardField.TextArea = {
        
    /** Starts field editing. */
    editField : function(fieldInfo, editFieldData)
    {
        var field = fieldInfo.elem;
        
        // generate edit dom
        var editDom = AJS.$('<div class="gh-edit"><textarea rows="1" cols="40" class="js-inline-edit"></textarea></div>');
        var editDomField = AJS.$(editDom).find('textarea,input,select');
        AJS.$(editDomField).val(editFieldData.value);

        fieldInfo.editDom = editDom;
        fieldInfo.editDomField = editDomField;
        fieldInfo.editFieldData = editFieldData;
        
        // update UI
        AJS.$(field).addClass('gh-edit-active').closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').addClass('gh-edit-active gh-edit-active-textarea'); // Some styles change while edit is happening
        AJS.$(field).children().hide() // hide the normal content
            .end().append(editDom);  // insert the editable content in form field
        AJS.$(editDomField).focus(); // focus in the form field

        // autoResize of textarea only for Cards
        if (AJS.$('#gh, #gh-step-families, .gh-step-children, #search-mainboard').hasClass('gh-cards') || AJS.$(field).closest('.gh-card-preview').length > 0){
            // do not let the TB step-parent in card mode autoresize
            if (!AJS.$(field).parents('.gh-issue').parent().hasClass('gh-step-parent')){
            	var textarea = AJS.$(field).find('textarea');
            	textarea.autoResize();
            	
            	// issue a keydown event to make the textarea being replaced before the user starts typing
            	// otherwise we get problems on FF/Win7 where the cursor jumps to the end of the text on the first key press
            	textarea.keydown();
            }
        }

        // temp hack to test if truly a list not list-as-card
        if (AJS.$('#gh').hasClass('gh-list')){
            if (!AJS.$(editDomField).parents().hasClass('gh-card-preview')){
                AJS.$(editDomField).closest('.gh-issue-wrapper').addClass('gh-list-as-list');
            }
        else {
                AJS.$(editDomField).closest('.gh-issue-wrapper').addClass('gh-list-as-card');
            }
        }

        // Character Count - limit typing to field limit
        if (editFieldData.limit) {
            AJS.$(editDomField).attr('maxlength', editFieldData.limit);
            AJS.$(editDomField).bind('keyup',function(event){

                var contentLength = AJS.$(editDomField).val().length;
                var contentCut;

                // if the content is longer than limit remove the superfluous content
                if (contentLength > editFieldData.limit) {
                    var contentCut = AJS.$(editDomField).val().substring(0, editFieldData.limit);
                    AJS.$(editDomField).val(contentCut);
                }

            });
        }

        // On blur return the field with new value
        AJS.$(editDomField).blur(function(){
            GH.CardField.TextArea._finishEdit(fieldInfo);
        });
        
        // handle some keys specially
        AJS.$(editDomField).keydown(function(event) {
            if (event.keyCode == 27) { // escape key
                event.stopPropagation();
                GH.CardField.TextArea.cancelEdit(fieldInfo);
            }
            else if (event.keyCode == 13) { // return
                // check if shift is pressed: return = form submit, shift return = new line
                if (event.shiftKey != true) {
                    event.stopPropagation();
                    event.preventDefault();
                    GH.CardField.TextArea._finishEdit(fieldInfo);
                }
            }
        });
    },
    
    _finishEdit : function(fieldInfo) {
        fieldValue = AJS.$(fieldInfo.editDomField).val();
        GH.CardField.submitFieldValue(fieldInfo, fieldValue);
    },
    
    /**
     * Cancels an edit operation. This is called in case update produced an error.
     */
    cancelEdit: function(fieldInfo) {
        var field = fieldInfo.elem;
        AJS.$(field)
            .find('textarea')
            .unbind('blur') // unregister blur handler
            .end()
            .find('.gh-edit').remove() // remove edit element
            .end()
            .removeClass('gh-edit-active')
            .children().show(); // show all previously hidden children
        
        var wrapper = AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper');
        GH.CardField.TextArea._cleanWrapper(wrapper);
    },
    
    /**
     * Updates an existing field dom element with new values.
     * @param the field element to update
     * @param fieldData the data object to use
     */
    updateField : function(field, fieldData) {
        // render the new dom given the new field data
        var elem = AJS.$('<div class="gh-editable"></div>');
        elem.attr('title', fieldData.data.title);
        AJS.$(elem).append(fieldData.data.text);
        
        // remove all children
        AJS.$(field).children().remove();
        
        // add the new content
        AJS.$(field).append(elem);

        // remove the edit-in-progress class
        AJS.$(field).removeClass('gh-edit-active');
        var wrapper = AJS.$(field).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper');
        GH.CardField.TextArea._cleanWrapper(wrapper);
    },
    
    _cleanWrapper: function(wrapper) {
    	// remove this field types class - todo: this is unclean! 
        wrapper.removeClass('gh-edit-active-textarea');
        
        // only remove edit-active if we are the only edited field
        if (wrapper.find('.gh-edit-active').length < 1) {
        	wrapper.removeClass('gh-edit-active  gh-list-as-list');
        }
    }
};

// register controller
GH.CardField.controllers['textarea'] = GH.CardField.TextArea;
