if (typeof GH == 'undefined') { var GH = {}; }

/**
 * The idea of this library is to catch Javascript exceptions that came up when rendering the page,
 * either parse errors or runtime exceptions. The user should never be left with a blank screen, but
 * rather have information for support. 
 */
GH.Exception = {};

/**
 * This gets called through window.onError. That's a custom browser error handler, unfortunately
 * it lives outside the exception stack of the JS engines, so we don't have a stacktrace available here.
 * 
 * Turn the information we have into an AUI error message.
 */
GH.Exception.handleJsException = function(msg, file, line) {
    
    var stack = null;// TODO see if we can do that in a generic way: printStackTrace();
    
    // fetch exception details
    var exView = GH.Exception.buildExceptionView({
        title : 'An exception occurred',
        details : {
            'Exception' : msg,
            'Resource' : file,
            'Line' : line
        },
        stackTrace : stack
    });
    
    // add view
    var exContext = document.createElement('div'); 
    exContext.setAttribute('class', 'ghx-exception');
    exContext.appendChild(exView);
        
    // see if we have ghx-errors around, our preferred error anchor.
    // if that's not the case (because the exception happened before the DOM
    // was rendered), we fall back to body.
    var exTarget = document.getElementById('ghx-errors');
    if (!exTarget) {
        exTarget = document.createElement('div');
        exTarget.setAttribute('class', 'ghx-exception-wrapper');
        document.body.appendChild(exTarget);
    }
    exTarget.appendChild(exContext);

    // stops the browser from further handling the error
    return true;
};

/**
 * Construct the exception view. Since we're not sure at which point in the script execution the exception
 * happened, we must not rely on ANY library here. No jQuery please!
 */
GH.Exception.buildExceptionView = function(exceptionData) {
    
    // main container
    var msgView = document.createElement('div');
    msgView.setAttribute('class', 'aui-message error');
    
    // title
    var titleView = document.createElement('p');
    titleView.setAttribute('class', 'title');
    titleView.innerHTML = '<span class="aui-icon icon-error"></span>' + exceptionData.title;
    msgView.appendChild(titleView);

    // help text
    var helpText = document.createElement('p');
    helpText.innerHTML = 'Please try refreshing the page, or contact your administrator / <a href="http://support.atlassian.com">Atlassian Support</a> if the problem continues.';
    msgView.appendChild(helpText);
    
    // details
    var det = exceptionData.details; 
    if (det) {
        var detailViewTitle = document.createElement('h4');
        detailViewTitle.innerHTML = 'Details';
        msgView.appendChild(detailViewTitle);
        
        var detailView = document.createElement('ul');
        for (var key in det) {
            var detailItem = document.createElement('li');
            detailItem.innerHTML = key + ': ' + det[key];
            detailView.appendChild(detailItem);
        }
        msgView.appendChild(detailView);
    }
    
    // environment
    var envViewTitle = document.createElement('h4');
    envViewTitle.innerHTML = 'Environment';
    msgView.appendChild(envViewTitle);
    
    var envView = document.createElement('div');
    envView.innerHTML = navigator.userAgent;
    msgView.appendChild(envView);
    
    // stacktrace (currently not working, see above)
    var stack = exceptionData.stackTrace;
    if (stack) {
        var stackViewTitle = document.createElement('h3');
        stackViewTitle.innerHTML = 'Stack trace';
        msgView.appendChild(stackViewTitle);
        
        var stackView = document.createElement('ul');
        for (var i=0; i<stack.length; i++) {
            var stackItem = document.createElement('li');
            stackItem.innerHTML = stack[i];
            stackView.appendChild(stackItem);
        }
        msgView.appendChild(stackView);
    }
    
    return msgView;
};

window.onerror = GH.Exception.handleJsException;
