/*
 * Define common name spaces here
 */
/** Root for all GH functions, data and templates. */
if (typeof GH == 'undefined') { var GH = {}; }

/** Contains the data model. */
if (typeof GH.view == 'undefined') { GH.view = {}; }
if (typeof GH.view.model == 'undefined') { GH.view.model = {}; }

/** Contains global page specific data, such as the requestContext or the current user name. */
if (typeof GH.Global == 'undefined') { GH.Global = {}; }

/** Dialog stuff. */
if (typeof GH.Dialog == 'undefined') { GH.Dialog = {}; }

/** AJAX state management */
if (typeof GH.State == 'undefined') { GH.State = {}; }

/** Main XBoard namespace. */
if (typeof GH.XBoard == 'undefined') { GH.XBoard = {}; }

if (typeof GH.RapidBoard == 'undefined') { GH.RapidBoard = {}; }

/** Jql autocomplete logic. */
if (typeof GH.JQLAutoComplete == 'undefined') { GH.JQLAutoComplete = {}; }

/** Utility code */
if (typeof GH.RapidBoard.Util == 'undefined') { GH.RapidBoard.Util = {}; }

/** indexOf IE8 fix */
if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function(obj, start) {
         for (var i = (start || 0), j = this.length; i < j; i++) {
             if (this[i] === obj) { return i; }
         }
         return -1;
    }
}


/**
 * Replaces tokens in a string with arguments, similar to Java's MessageFormat.
 * Tokens are in the form {0}, {1}, {2}, etc.
 * @method format
 * @param message the message to replace tokens in
 * @param arg (optional) replacement value for token {0}, with subsequent arguments being {1}, etc.
 * @return {String} the message with the tokens replaced
 * @usage AJS.format("This is a {0} test", "simple");
 */
/*
AJS.format = function (message) {
     var token = /^((?:(?:[^']*'){2})*?[^']*?)\{(\d+)\}/, // founds numbers in curly braces that are not surrounded by apostrophes
         apos = /'(?!')/g; // founds "'", bot not "''"
     // we are caching RegExps, so will not spend time on recreating them on each call
     AJS.format = function (message) {
         var args = arguments,
             res = "",
             match = message.match(token);
         while (match) {
              message = message.substring(match[0].length);
              res += match[1].replace(apos, "") + (args.length > ++match[2] ? args[match[2]] : "");
              match = message.match(token);
         }
         return res += message.replace(apos, "");
     };
     return AJS.format.apply(AJS, arguments);
 };
*/

/**
 * The original AJS.format does not support choice or number values.
 * This version adds support for (integer number) choice values, proper number value support still pending
 */
AJS.format = function (message) {
    var apos = /'(?!')/g, // founds "'", but not "''"
        simpleFormat = /^\d+$/,
        numberFormat = /^(\d+),number$/, // TODO: incomplete, as doesn't support floating point numbers
        choiceFormat = /^(\d+)\,choice\,(.+)/,
        choicePart = /^(\d+)([#<])(.+)/; // TODO: does not work for floating point numbers!
        // we are caching RegExps, so will not spend time on recreating them on each call
    
    // formats a value, currently choice and simple replacement are implemented, proper 
    var getParamValue = function(format, args) {
        // simple substitute
        var res = '', match;
        if (match = format.match(simpleFormat)) { // TODO: heavy guns for checking whether format is a simple number...
            res = args.length > ++format ? args[format] : ''; // use the argument as is, or use '' if not found
        }
        
        // number format
        else if (match = format.match(numberFormat)) {
        	// TODO: doesn't actually format the number...
        	res = args.length > ++match[1] ? args[match[1]] : '';
        }
        
        // choice format
        else if (match = format.match(choiceFormat)) {
            // format: "0,choice,0#0 issues|1#1 issue|1<{0,number} issues"
            // match[0]: "0,choice,0#0 issues|1#1 issue|1<{0,number} issues"
            // match[1]: "0"
            // match[2]: "0#0 issues|1#1 issue|1<{0,number} issues"
            
            // get the argument value we base the choice on
            var value = (args.length > ++match[1] ? args[match[1]] : null);
            if (value !== null) {
                // go through all options, checking against the number, according to following formula,
                // if X < the first entry then the first entry is returned, if X > last entry, the last entry is returned
                //
                //    X matches j if and only if limit[j] <= X < limit[j+1]
                // 
                var options = match[2].split('|');
                
                var prevOptionValue = null; // holds last passed option
                for (var i=0; i < options.length; i++) {
                    // option: "0#0 issues"
                    // part[0]: "0#0 issues"
                    // part[1]: "0"
                    // part[2]: "#"
                    // part[3]" "0 issues";
                    var parts = options[i].match(choicePart);
                    
                    // if value is smaller, we take the previous value, or the current if no previous exists
                    var argValue = parseInt(parts[1], 10);
                    if (value < argValue) {
                        if (prevOptionValue) {
                            res = prevOptionValue;
                            break;
                        } else {
                            res = parts[3];
                            break;
                        }
                    }
                    // if value is equal the condition, and the match is equality match we accept it
                    if (value == argValue && parts[2] == '#') {
                        res = parts[3];
                        break;
                    }
                    else {
                        // value is greater the condition, fall through to next iteration
                    }
                    
                    // check whether we are the last option, in which case accept it even if the option does not match
                    if (i == options.length - 1) {
                        res = parts[3];
                    }
                    
                    // retain current option
                    prevOptionValue = parts[3];
                }
                
                // run result through format, as the parts might contain substitutes themselves
                var formatArgs = [res].concat(Array.prototype.slice.call(args, 1));
                res = AJS.format.apply(AJS, formatArgs);
            }
        }
        return res;
    };
    
    // drop in replacement for the token regex
    // splits the message to return the next accurance of a i18n placeholder.
    // Does not use regexps as we need to support nested placeholders
    // text between single ticks ' are ignored
    var _performTokenRegex = function(message) {
    	var tick=false, openIndex=-1, openCount=0;
    	for (var i=0; i < message.length; i++) {
    		// handle ticks
    		var c = message.charAt(i); 
    		if (c == "'") {
    			// toggle
    			tick = !tick;
    		}
    		// skip if we are between ticks
    		if (tick) continue;
    		// check open brackets
    		if (c == '{') {
    			if(openCount==0) openIndex = i;
    			openCount++;
    		}
    		else if (c == '}') {
    			if (openCount > 0) {
    				openCount--;
	    			if(openCount == 0) {
	    				// we found a bracket match - generate the result array (
	    				var match = [];
	    				match.push(message.substring(0, i+1)); // from begin to match
	    				match.push(message.substring(0, openIndex)); // everything until match start
	    				match.push(message.substring(openIndex+1, i)); // matched content
	    				return match;
	    			}
    			}
    		}
    	}
    	return null;
    };
    
    var _format = function (message) {
        var args = arguments,
            res = "",
            match = _performTokenRegex(message); //message.match(token);
        while (match) {
            // reduce message to string after match  
            message = message.substring(match[0].length);
             
            // add value before match to result
            res += match[1].replace(apos, "");
             
            // add formatted parameter
            res += getParamValue(match[2], args);
             
            // check for next match
            match = _performTokenRegex(message); //message.match(token);
        }
        // add remaining message to result
        return res += message.replace(apos, "");
    };
    return _format.apply(AJS, arguments);
};
