/**
 * Main RapidBoard functions
 */


/**
 * Opens the rapid board with the given rapid view id
 */
GH.RapidBoard.gotoRapidBoardPage = function(rapidViewId) {
	window.location.href = GH.Ajax.CONTEXT_PATH+'/secure/RapidBoard.jspa?rapidView=' + rapidViewId;
};

/** Get the current page url as used in the returnUrl JIRA request parameter.
 * Note: the contextPath is NOT part of the url in this case
 */
GH.RapidBoard.getCurrentPageReturnUrl = function() {
	var params = GH.RapidBoard.UrlState.getViewStateRequestParameterString();
	var url = '/secure/RapidBoard.jspa?'+params;
	return encodeURIComponent(url);
};

/**
 * Changes the rapid view to the passed view id.
 */
GH.RapidBoard.showRapidView = function(rapidViewId) {
	GH.RapidBoard.State.setRapidViewId(rapidViewId);
	GH.RapidBoard.State.pushState();
    GH.RapidBoard.reload();
};

/**
 * Reloads the model from the current view state, setting issueToSelect as the selected issue.
 */
GH.RapidBoard.reload = function() {
    GH.RapidBoard.loadBoardData();
};

/**
 * Calls the rapid board data rest service.
 */
GH.RapidBoard.loadBoardData = function() {
	// gather the parts of the screen we want to load in one go
	var params = {};
	if(GH.RapidBoard.State.getRapidViewId()) {
		params.rapidViewId = GH.RapidBoard.State.getRapidViewId();
		params.activeQuickFilters = GH.RapidBoard.State.getActiveQuickFilters();
	}
	else if (GH.RapidBoard.State.getMostRecentRapidViewId()) {
		params.lastRapidViewId = GH.RapidBoard.State.getMostRecentRapidViewId();
		params.activeQuickFilters = GH.RapidBoard.State.getActiveQuickFilters(params.lastRapidViewId);
	}
	params.loadRapidListData = true;
    
    /**
     * Work around for jQuery's messy handling of empty arrays,
     * see http://forum.jquery.com/topic/change-of-behavior-in-1-4-4-because-of-bug-6481
     * fyi: sends empty string instead of no value
     */
    if(params.activeQuickFilters && params.activeQuickFilters.length == 0) {
        delete params.activeQuickFilters;
    }

    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/xboard/compoundModel.json'),
        data: params,
        success: GH.RapidBoard.processData
    }, 'rapidBoardCompoundModel');
};

/**
 * Processes the data returned by the compoundModel rest service
 */
GH.RapidBoard.processData = function(data) {
	// set the model
	GH.view.model = data;
	
	// ensure we got the proper state set.
	// TODO: maybe everybody should ask the rapidviewselector for this info / a separate object take care of this state
	var selectedView = GH.RapidBoard.RapidViewSelector.getSelectedView();
	if (selectedView && (selectedView.id != GH.RapidBoard.State.getRapidViewId())) {
		GH.RapidBoard.State.setRapidViewId(selectedView.id);
		GH.RapidBoard.State.pushState();
	}
	
	// TODO: move out to the RapidListDataController
	GH.RapidBoard.RapidListDataController.initModel();

    AJS.$(document).trigger('modelChanged');
};

/**
 * Initialise event listeners of sub-components
 */
GH.RapidBoard.initListeners = function() {
    
    // view selector widget
    GH.RapidBoard.RapidViewSelector.init();

    // operations bar
    GH.RapidBoard.OperationsBar.init();
    
    // main view controller
    GH.RapidBoard.ViewController.init();
    
    // SwimlaneView
    GH.RapidBoard.SwimlaneView.init();
    
    // Stalker bar
    GH.RapidBoard.Stalker.init();
    
    // details view widget
    GH.RapidBoard.DetailsView.init();
    
    // keyboard shortcuts
    GH.RapidBoard.Navigation.init();
    
    // issue
    GH.RapidBoard.Issue.init();
    
    // drag and drop
    GH.RapidBoard.DragAndDrop.init();
    
    // release widget
    GH.RapidBoard.ReleaseView.init();
    
    // chart view
    GH.RapidBoard.ChartView.init();

    // quick filter
    GH.RapidBoard.QuickFilterView.init();

    // create dialog
    GH.RapidBoard.CreateDialog.init();
    
    // welcome mat result renderer
    GH.RapidBoard.WelcomeMessage.init();
};

GH.RapidBoard.init = function() {
	// render the base dom tree
	AJS.$('#gh').html(GH.tpl.board.x.renderRapidBoard());
	
    // start event handling environment
    GH.RapidBoard.initListeners();

    // initialize the state
	GH.RapidBoard.State.initalizeState();
    
    // reloads the complete model and then rerenders everything
    GH.RapidBoard.reload();
};


// RapidViewGadget

/**
 * Initializes the rapid view gadget.
 */
GH.RapidBoard.initRapidViewGadget = function(gadget) {
	// render the base html into the view
	gadget.getView().html('<div id="gh" class="ghx-board-rapid"></div>');
	AJS.$('#gh').html(GH.tpl.board.x.renderRapidBoard());

    // view selector widget
    GH.RapidBoard.RapidViewSelector.init();

    // main view controller
    GH.RapidBoard.ViewController.init();
    
    // SwimlaneView
    GH.RapidBoard.SwimlaneView.init();

    // Stalker bar
    GH.RapidBoard.Stalker.init();
};

/**
 * This function sets the correct configuration, then forces a reload
 */
GH.RapidBoard.renderRapidViewGadget = function(gadget) {
    // manually initialize the state
	// set the values
    var rapidViewId = gadget.getPref("rapidViewId");
	GH.RapidBoard.State.setDataValue('rapidViewId', rapidViewId); // we write that value in directly as we don't want to affect the session storage
    GH.RapidBoard.State.setActiveQuickFilters([]);
    
    // then reload
    GH.RapidBoard.reload();
};
