/**
 * Columns configuration tab.
 * 
 * column/status mapping and column constraint functionality
 */
GH.RapidBoard.Config.Columns = {};

/**
 * Model backing the rapid list configuration
 */
GH.RapidBoard.Config.Columns.model = undefined;

/**
 * Initializes the rapid list configuration tab
 */
GH.RapidBoard.Config.Columns.init = function(data) {
	// store the provided model
	GH.RapidBoard.Config.Columns.model = data.rapidListConfig.success; 
	
    // render the column mapping
    GH.RapidBoard.Config.Columns.renderColumnMapping();
    
	// bind to the tab link, we need to align the column heights. This can only be done once the dom is visible
	AJS.$('#ghx-columns-config-tab-link').bind('tabSelect', GH.RapidBoard.Config.Columns.alignColumnHeights);
    
	GH.RapidBoard.Config.Columns.EditField.registerInlineEdits();
};

GH.RapidBoard.Config.Columns.renderColumnMapping = function() {
	var model = GH.RapidBoard.Config.Columns.model;

    // render the column config
    AJS.$('#ghx-config-columns').html(GH.tpl.rapid.view.renderColumnMappingView({'model' : model}));

    // align the heights.
    GH.RapidBoard.Config.Columns.alignColumnHeights();
    
    // initialise everyone who cares about the DOM
    if (GH.RapidBoard.Config.Columns.model.canEdit) {
        GH.RapidBoard.Config.Columns.DnD.init();
        GH.RapidBoard.Config.Columns.attachListeners();
        GH.RapidBoard.Config.Columns.EditField.init();

        AJS.$('#ghx-mapping a[data-deleteindex]').click(GH.RapidBoard.Config.Columns.deleteColumn);
    }
};

/**
 * Sets the same column height for all columns. Otherwise shorter columns won't be droppable on the whole height 
 */
GH.RapidBoard.Config.Columns.alignColumnHeights = function() {
	// remove the set heights
	var viewTable = AJS.$('#ghx-mapping');
	var columns = viewTable.find('.ghx-config-status');
	columns.css({'height': ''});
	
    // make the columns the same height = height of parent + twice a status height
    var viewTableHeight = viewTable.height();
    var viewStatusHeight = viewTable.find('.ghx-status:first').outerHeight();
    columns.css({'height': viewTableHeight + (viewStatusHeight * 2)});
};

GH.RapidBoard.Config.Columns.attachListeners = function() {
	AJS.$('#ghx-config-addcolumn').click(GH.RapidBoard.Config.Columns.addColumn);
	
	AJS.$('#ghx-mapping').sortable({
        axis : 'x',
        handle:'.ghx-grabber-horizontal',
        helper: 'clone',
        items: '.ghx-mapped',
        forceHelperSize: true,
        forcePlaceholderSize: true,
        opacity: '0.9',
        placeholder: 'ghx-sortable-placeholder',
        revert: true,
        tolerance: 'pointer',
        update: function(event, ui) {
            GH.RapidBoard.Config.Columns.save();
        }
    });
    
    AJS.$('#ghx-statisticsfield-select').change(GH.RapidBoard.Config.Columns.save);
};

GH.RapidBoard.Config.Columns.addColumn = function() {
	// add a new column to the model
	var newColumn = {
			name : AJS.I18n.getText('gh.rapid.view.columnmapping.newname'),
			mappedStatuses : []
	};
	var mappedCols = GH.RapidBoard.Config.Columns.model.mappedColumns;
    var newPosition = mappedCols.length -1;

	// insert it second to the last (so one before "done")
	mappedCols.splice(mappedCols.length - 1, 0, newColumn);

	// now render, since we're using the DOM to save the current status,
	// this also readjusts the column widths
	GH.RapidBoard.Config.Columns.renderColumnMapping();
	
	// save the new view
	GH.RapidBoard.Config.Columns.save(true,function(){
        // focus on the name field for the new column
        AJS.$('#ghx-mapping .ghx-mapped:eq('+newPosition+') h2').click();
    });
};

GH.RapidBoard.Config.Columns.deleteColumn = function(event) {
	var index = AJS.$(this).attr('data-deleteindex');
	// remove the selected column from the model
	GH.RapidBoard.Config.Columns.model.mappedColumns.splice(index, 1);
	
	// now render, since we're using the DOM to save the current status
	// this also readjusts the column widths
	GH.RapidBoard.Config.Columns.renderColumnMapping();

	// save the new view
	GH.RapidBoard.Config.Columns.save();
};

/**
 * Take the current DOM and turn it back into the model. This saves us from having to synch model and DOM
 * during drag and drop, we just take what the user sees and persist it.
 * 
 * @param reload : should the view be reloaded after saving, defaults to 'true' 
 */
GH.RapidBoard.Config.Columns.save = function(reload, onReload) {
    if (typeof reload == 'undefined') {
        reload = true;
    }
    
    var model = {};
    
    // statistics field
    model.currentStatisticsField = {
    	id: AJS.$('#ghx-statisticsfield-select').val()
    };

    // columns
    var columnsData = [];
    model.rapidViewId = GH.RapidBoard.Config.Columns.model.rapidViewId; 
    model.mappedColumns = columnsData;
    AJS.$('#ghx-mapping .ghx-mapped').each(function(index, column) {
    	column = AJS.$(column);
    	var columnData = {};
        var mappedStatuses = [];
        
    	// status mapping for this column
    	column.find('.ghx-status').each(function(index, status) {
            mappedStatuses[index] = {
            	id : AJS.$(status).attr('data-statusid')
            };
        });
    	columnData.mappedStatuses = mappedStatuses;
        
    	// name, min and max constraint
        // ok, nearly what the user sees. Minus html encoding. WYSIWYM.
    	var columnId = column.attr('data-column-id');
    	if (columnId)
		{
    		columnData.id = parseInt(columnId);
		}
        var name = column.find('h2[data-fieldname="name"]').attr('data-fieldvalue');
        if (name) columnData.name = name;
        var min = column.find('span[data-fieldname="min"]').attr('data-fieldvalue');
        columnData.min = min;
        var max = column.find('span[data-fieldname="max"]').attr('data-fieldvalue');
        columnData.max = max;
        
        // add column
        columnsData.push(columnData);
	});

	GH.Ajax.put({
	    url : GH.Ajax.buildRestUrl('/rapidviewconfig/columns'),
	    data : {
	    	'model' : JSON.stringify(model)
	    },
	    success : function(result) {
	    	// assign the new model
	    	GH.RapidBoard.Config.Columns.model = result.success;
	    	
	    	if (reload) {
	    		GH.RapidBoard.Config.Columns.renderColumnMapping();
	    		
	    		if (onReload) {
	    			onReload();
	    		}
	    	}
	    }
	});
};

// ====== drag drop ======

GH.RapidBoard.Config.Columns.DnD = {};

GH.RapidBoard.Config.Columns.DnD.init = function() {
    GH.RapidBoard.Config.Columns.DnD.registerStatusDraggables();
    GH.RapidBoard.Config.Columns.DnD.registerDropZones();
};

GH.RapidBoard.Config.Columns.DnD.registerStatusDraggables = function() {
    AJS.$('#ghx-mapping .ghx-status').draggable({
        containment: '#ghx-mapping',
        revert: true,
        revertDuration: 0,
        zIndex: 3000
    });
};

GH.RapidBoard.Config.Columns.DnD.registerDropZones = function() {
    AJS.$('#ghx-mapping .ghx-config-status').droppable({
        accept: '.ghx-status',
        hoverClass: 'ghx-target-hover',
        over: function() { AJS.$('#ghx-mapping .ghx-config-status').eq(AJS.$(this).index()).addClass('ghx-target-hover'); },
        out: function() { AJS.$('#ghx-mapping .ghx-config-status').eq(AJS.$(this).index()).removeClass('ghx-target-hover'); },
        tolerance: 'pointer',
        drop: GH.RapidBoard.Config.Columns.DnD.statusDropped
    });
};

GH.RapidBoard.Config.Columns.DnD.statusDropped = function(event, ui) {
    AJS.$(this).append(AJS.$(ui.draggable));
    
    GH.RapidBoard.Config.Columns.save();
};


//====== inline edit ======

GH.RapidBoard.Config.Columns.EditField = {};

GH.RapidBoard.Config.Columns.EditField.MAX_ISSUE_COUNT_LIMIT = 1000;

GH.RapidBoard.Config.Columns.EditField.init = function() {};

/**
 * Data gathering function for the constraints fields.
 */ 
GH.RapidBoard.Config.Columns.EditField.constraintsData = function(viewElement, options) {
	data = GH.RapidBoard.Util.InlineEditable.defaultFieldData(viewElement, options);
	var columnIndex = viewElement.attr('data-columnindex');
	data.columnIndex = columnIndex;
	return data;
};

/**
 * Renders the edit for the constraint fields
 */
GH.RapidBoard.Config.Columns.EditField.renderConstraintInlineEdit = function(editData) {
	var elem = GH.RapidBoard.Util.InlineEditable.renderTextFieldInlineEdit(editData, GH.tpl.rapid.view.renderCapacityFieldEdit);
	return elem;
};

/** Called prior to starting the inline edit on the column configuration page. */
GH.RapidBoard.Config.Columns.EditField.preInlineEditCallback = function(editData) {
	// blur existing textfields - somehow focusing the new one doesn't trigger the blur on the old one
	AJS.$('#ghx-mapping .ghx-header input[type="text"]').blur();
	
	// hide bin
	editData.viewElement.siblings('.ghx-action').hide();
};

/**
 * Called after inline edit has finished.
 */
GH.RapidBoard.Config.Columns.EditField.postInlineEditCallback = function(editData) {	
	// show bin again
	editData.viewElement.siblings('.ghx-action').show();
};

/**
 * Update the column name
 */
GH.RapidBoard.Config.Columns.EditField.columnNameValidation = function(editData) {
    // new value
    var newValue = editData.editElement.val();
    
	// perform validation
    if (!GH.Validation.notBlank(editData.editElement, AJS.I18n.getText('gh.rapid.view.error.name.required'))) {
    	return false;
    }
    
    editData.newValue = newValue;
    return true;
};

/**
 * Save the changes without rerendering the page
 */
GH.RapidBoard.Config.Columns.EditField.constraintValidation = function(editData) {
    // new value
    var newValue = editData.editElement.val();
    
	// first trim the value
	newValue = AJS.$.trim(newValue);
    	
	var editView = editData.editElement;
	var fieldName = editData.fieldName;
	var columnIndex = editData.columnIndex;
	
	// if the new value is empty, we can skip all validation
	if (newValue !== '') {
	
    	// transform the value into a number
    	newValue = parseInt(newValue);
    	
    	// make sure it actually is a number
    	if (isNaN(newValue)) {
    		GH.Validation.showContextualErrors(editView, AJS.I18n.getText('gh.rapid.view.error.capacity.notnumber'));
            return false;
    	}
    	
    	// round the number down
    	newValue = Math.floor(newValue);
    	
    	// make sure the number is greater than 0
    	if (newValue < 1) {
    		GH.Validation.showContextualErrors(editView, AJS.I18n.getText('gh.rapid.view.error.capacity.toosmall', 0));
            return false;
    	}
    	
    	// make sure the number is smaller/equal to our limit
    	if (newValue >= GH.RapidBoard.Config.Columns.EditField.MAX_ISSUE_COUNT_LIMIT) {
    		GH.Validation.showContextualErrors(editView, AJS.I18n.getText('gh.rapid.view.error.capacity.toobig', GH.RapidBoard.Config.Columns.EditField.MAX_ISSUE_COUNT_LIMIT));
            return false;
    	}
    	
    	// now ensure that min value is smaller equal to max value 
        var oppositeField = fieldName == 'min' ? 'max' : 'min';
        var oppositeValue = AJS.$('#ghx-mapping .ghx-mapped .js-edit-'+oppositeField+'[data-columnindex='+columnIndex+']').attr('data-fieldvalue');

        // only check for min max numeric value constriants if
        // - newValue is not empty
        // - oppositeValue is not empty
        if (oppositeValue !== '') {
        	oppositeValue = parseInt(oppositeValue);
            if (fieldName == 'min') {
            	// ensure we actually compare numbers
                if (newValue > oppositeValue) {
                    GH.Validation.showContextualErrors(editView, AJS.I18n.getText('gh.rapid.view.error.mintoobig'));
                    return false;
                }
            } else {
                if (newValue < oppositeValue) {
                    GH.Validation.showContextualErrors(editView, AJS.I18n.getText('gh.rapid.view.error.maxtoosmall'));
                    return false;
                }
            }
        }
	}
    
	editData.newValue = newValue;
	return true;
};

/**
 * Save the column configuration upon successfuly inline edits
 */
GH.RapidBoard.Config.Columns.EditField.saveEditChanges = function(editData) {
	// switch to display mode for fast user feedback
	// also, and more importantly, the column model is saved as a whole, so the view needs to be available before actually saving
    GH.RapidBoard.Util.InlineEditable.updateView(editData);
    
    // save, but don't reload, since that would overwrite consecutive edit actions,
    // like when clicking the next field directly
    GH.RapidBoard.Config.Columns.save(false);
};

/**
 * Renders the column name view
 */
GH.RapidBoard.Config.Columns.EditField.renderColumnNameView = function(editData) {
	var displayView = AJS.$(GH.tpl.rapid.view.renderColumnNameField({
        fieldName : editData.fieldName,
        fieldValue : editData.newValue,
        displayValue : editData.newValue
    }));
	return displayView;
};

/**
 * Renders the min/max constraints
 */
GH.RapidBoard.Config.Columns.EditField.renderColumnConstraintView = function(editData) {
    if (editData.fieldName == 'min') {
        var displayValue = editData.newValue ? editData.newValue : AJS.I18n.getText('gh.rapid.config.capacity.setmin');
    } else {
        var displayValue = editData.newValue ? editData.newValue : AJS.I18n.getText('gh.rapid.config.capacity.setmax');
    }
    
    var displayView = AJS.$(GH.tpl.rapid.view.renderCapacityField({
        fieldName : editData.fieldName,
        fieldValue : editData.newValue,
        index : editData.columnIndex,
        displayValue : displayValue 
    }));
    return displayView;
};

/**
 * Register the inline edit handlers for the column inline edit fields.
 */
GH.RapidBoard.Config.Columns.EditField.registerInlineEdits = function() {
	// shorten the namespaces...
	var inlineEdit = GH.RapidBoard.Util.InlineEditable;
	var editField = GH.RapidBoard.Config.Columns.EditField;
	
	// title edit
	inlineEdit.register('#ghx-mapping .ghx-mapped .js-edit-name', {
	    preEdit    : editField.preInlineEditCallback,
	    validate   : editField.columnNameValidation,
	    save       : editField.saveEditChanges,
	    renderView : editField.renderColumnNameView,
	    postEdit   : editField.postInlineEditCallback
	});
	
	// min/max edit
	inlineEdit.register('#ghx-mapping .ghx-mapped .js-edit-min,.js-edit-max', {
	    getData    : editField.constraintsData,
	    preEdit    : editField.preInlineEditCallback,
	    renderEdit : editField.renderConstraintInlineEdit,
	    validate   : editField.constraintValidation,
	    save       : editField.saveEditChanges,
	    renderView : editField.renderColumnConstraintView,
	    postEdit   : editField.postInlineEditCallback
	});
};
