/**
 * Holds the code for the rapid view creation dialog
 */
GH.RapidBoard.CreateDialog = {};

GH.RapidBoard.CreateDialog.init = function() {
    AJS.$(document).delegate('.ghx-new-view', 'click', function(event){
    	GH.RapidBoard.CreateDialog.showCreateDialog();
    });
};

/**
 * Loads the create rapid view dialog
 */
GH.RapidBoard.CreateDialog.showCreateDialog = function() {
    // fetch the view model for the create dialog and pass it on to the handler
    GH.Ajax.get({
        url : GH.Ajax.buildRestUrl('/rapidview/createmodel'),
        dataType : 'json',
        success : GH.RapidBoard.CreateDialog.renderDialog
    });
};

/**
 * Renders the create rapidview dialog given the create model
 */
GH.RapidBoard.CreateDialog.renderDialog = function(model) {
    var dlg = new AJS.Dialog({
        width : 500,
        height : 300,
        id : 'ghx-rapidview-create'
    });
    AJS.$('#'+dlg.id).addClass('ghx-dialog');
    dlg.addHeader(AJS.I18n.getText('gh.rapid.view.create'));
    dlg.addPanel('', 'panel');
    dlg.getCurrentPanel().html(GH.tpl.rapid.view.renderCreateDialogForm({
        savedFilters : model.savedFilters
    }));
    dlg.addSubmit(
    	AJS.I18n.getText('gh.rapid.view.create.submit'),
    	function() { GH.RapidBoard.CreateDialog.submitDialog(dlg); }
    );
    dlg.addCancel(AJS.I18n.getText('gh.rapid.view.create.cancel'), function() {
        dlg.remove();
    });

    // bind the model to the select, so we can access it in the change event context
    var select = AJS.$('#ghx-create-filter-select');
    select.data('model', model);
    
    // register change handler on the filter selector to update the permission info
    select.change(GH.RapidBoard.CreateDialog.updatePermissionInfoHandler);
    
    // make sure the text fields don't submit on enter (we don't want regular form submits)
    AJS.$('#ghx-rapidview-create input[type="text"]').keydown(function(event) {
        if (event.keyCode == 13) {
            event.preventDefault();
        }
    });
    
    dlg.show();	
};

/**
 * Update the display of the JIRA filter permissions once the filter dropdown changes.
 */
GH.RapidBoard.CreateDialog.updatePermissionInfoHandler = function() {
    var select = this;
    var model = AJS.$(select).data().model;
    
    AJS.$('#ghx-create-permissioninfo-container').html(
        GH.tpl.rapid.view.renderPermissionInfo({
            'savedFilter' : model.savedFilters[select.selectedIndex]
        })
    );
};

/**
 * Submits the dialog data
 */
GH.RapidBoard.CreateDialog.submitDialog = function(dlg) {
	var nameField = AJS.$('#ghx-view-name');
	var filterField = AJS.$('#ghx-create-filter-select');
	if (!GH.Validation.notBlank(nameField, AJS.I18n.getText('gh.rapid.view.error.name.required'))) return;
	if (!GH.Validation.notBlank(filterField, AJS.I18n.getText('gh.rapid.view.error.filter'), '#ghx-filter-error')) return;
	
    GH.Ajax.post({
        url : GH.Ajax.buildRestUrl('/rapidview'),
        data : {
            'name' : nameField.val(),
            'filterId' : filterField.val()
        },
        success : function(result) {
            GH.RapidBoard.RapidViewSelector.gotoRapidViewConfiguration(result.success);
        },
        errorContextMap : {
            name : '#ghx-view-name',
            filterId :'#ghx-create-filter-select'
        }
    });
};
