/**
 * Quick Filters configuration tab
 */
GH.RapidBoard.Config.QuickFilter = {};

/**
 * Holds the model for the configuration.
 */
GH.RapidBoard.Config.QuickFilter.model = undefined;

/**
 * Initializes the Quick Filters config.
 */
GH.RapidBoard.Config.QuickFilter.init = function(data) {
	GH.RapidBoard.Config.QuickFilter.model = data.quickFilterConfig.success;

    GH.RapidBoard.Config.QuickFilter.initializeTable();
};

/**
 * Renders the edit form using the locally stored model
 */
GH.RapidBoard.Config.QuickFilter.initializeTable = function() {
    var model = GH.RapidBoard.Config.QuickFilter.model;
    var canEdit = model.canEdit;
    var rapidViewId = model.rapidViewId
    var quickFilters = model.quickFilters;

    AJS.$('#ghx-config-quickFilters').html(GH.tpl.rapid.view.renderQuickFilterConfigView({canEdit:canEdit}));

    // setup the restful table
    var restfulTable = new JIRA.RestfulTable({
        el: jQuery("#ghx-quick-filter-table"), // table to add entries to. Entries are appended to the tables <tbody> element
        url: GH.Ajax.buildRestUrl('/quickfilters/'+rapidViewId+'/'), //rest resource for collection
        editable: canEdit,
        reorderable: canEdit,
        entries: quickFilters,
        views: {
            editRow: GH.RapidBoard.Config.QuickFilter.EditRow,
            row: GH.RapidBoard.Config.QuickFilter.Row
        }
    });

    // Only make rows with js-sortable class sortable
    restfulTable.$tbody.sortable('option', 'items');

    // register jql auto complete
    if (model.canEdit) {
    	GH.JQLAutoComplete.initialize('ghx-quick-filter-jql-create', 'ghx-quick-filter-jql-errormsg-create');
	}
};


/**
 * Table view row object
 */
GH.RapidBoard.Config.QuickFilter.Row = JIRA.RestfulTable.Row.extend({

    initialize: function () {
        // call super
        JIRA.RestfulTable.Row.prototype.initialize.apply(this, arguments);

        // crap work around to handle backbone not extending events
        // (https://github.com/documentcloud/backbone/issues/244)
        this.events["click .js-quick-filter-delete"] = "_delete";
        this.delegateEvents();
    },

    _delete: function (e) {
        this._openDialog(GH.RapidBoard.Config.QuickFilter.DeleteForm,
                "ghx-quick-filter-" + this.model.get("id") + "-delete-dialog");

        e.preventDefault();
    },

    render: function () {
    	// set the correct class onto the row - either the row is sortable or it is the default row
    	if (! this.model.get('isDefault')) {
    		this.$el.addClass('js-sortable');
    	} else {
    		this.$el.addClass('ghx-default-quick-filter');
    	}
        this.$el.html(GH.tpl.rapid.view.renderQuickFilterRow(this.model.toJSON()));
        return this;
    }
});

/**
 * Table edit row object
 */
GH.RapidBoard.Config.QuickFilter.EditRow = JIRA.RestfulTable.EditRow.extend({

    render: function (renderData) {
        this.$el.html(GH.tpl.rapid.view.renderQuickFilterEditRow(renderData));

        return this;
    },


    // used to handle JQL errors - so form is not submitted
    submit: function () {
        var values = this.$el.serializeToObject();
        var oldJql = this.model.attributes.query;
        var newJql = values && values.query ? values.query : "";
        if(oldJql)
        {
            //If we have an old query, then we have an edit event
            AJS.$(AJS).trigger("gh.quickfilter.edit", {
                rapidViewId: GH.RapidBoard.Config.editedRapidViewId,
                oldJql: oldJql,
                newJql: newJql
            });
        } else {
            // else we have a create event
            AJS.$(AJS).trigger("gh.quickfilter.create", {
                rapidViewId: GH.RapidBoard.Config.editedRapidViewId,
                jql: newJql
            });
        }

        return JIRA.RestfulTable.EditRow.prototype.submit.apply(this, arguments);
    }
});

/**
 * QuickFilter delete form
 */
GH.RapidBoard.Config.QuickFilter.DeleteForm = Backbone.View.extend({

    /**
     * Destroys model on server
     *
     * @param {Object} values
     * @param complete
     * @return {JIRA.Admin.Version.DeleteForm}
     */
    submit: function (values, row, complete) {

        this.$(".throbber").addClass("loading");

        AJS.$(AJS).trigger("gh.quickfilter.delete", {
            rapidViewId: GH.RapidBoard.Config.editedRapidViewId,
            jql: row.model.attributes.query
        });

        this.model.destroy({
            data: values,
            success: function () {
                complete();
            },
            error: function () {
                complete();
            }
        });

        return this;
    },

    /**
     * Renders delete form. This differs from standard render methods, as it requires async request/s to the server.
     * As a result when this method is calle the first argument is a function that is called when the content has been
     * rendered.
     *
     * @param {function} ready - callback to declare content is ready
     * @return {JIRA.Admin.Version.DeleteForm}
     */
    render: function (ready) {

        this.el.innerHTML = GH.tpl.rapid.view.deleteForm({
            message: AJS.I18n.getText('gh.rapid.quickfilter.delete.confirm', soy.$$escapeHtml(this.model.get('name')))
        });
        AJS.$(this.el).addClass('ghx-dialog');
        ready.call(this, this.el);

        return this;
    }
});
