GH.RapidBoard.Config = {};

// map from queryParam tab value to id of tab elements in the dom
GH.RapidBoard.Config.tabs = {
    filter: "gh-tab-filter",
    columns: "gh-tab-columns",
    swimlanes: "gh-tab-swimlanes",
    quickFilters: "gh-tab-quickFilters"
};

// default is the left-most tab
GH.RapidBoard.Config.defaultTab = "filter";

/**
 * Initializes the RapidViewConfig page
 */
GH.RapidBoard.Config.init = function() {
	// load the model
	GH.RapidBoard.Config.loadEditModel();

    AJS.$(document).delegate('#js-rapidboard-link, .js-rapidboard-link','click', function(event) {
        var rapidViewId = GH.RapidBoard.Config.editedRapidViewId;
        if (rapidViewId) {
    	    GH.RapidBoard.gotoRapidBoardPage(rapidViewId);
	        event.preventDefault();
        }
    });

    // create dialog
    GH.RapidBoard.CreateDialog.init();
    // rapid view name edit
    GH.RapidBoard.Config.initRapidViewNameEdit();
};

/**
 * Calls the rapid board data rest service.
 */
GH.RapidBoard.Config.loadEditModel = function() {
	// find the right rapidViewId
	var state = GH.State.toViewState(window.location.href);
	var rapidViewId = state.rapidView;

    if(!rapidViewId) {
        window.location.href = GH.Ajax.CONTEXT_PATH + "/secure/ManageRapidViews.jspa";
        return;
    }

    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/rapidviewconfig/editmodel.json'),
        data: {
        	rapidViewId: rapidViewId
        },
        success: GH.RapidBoard.Config.renderEditScreen
    });
};

/** Holds the id of the currently edited rapidView */
GH.RapidBoard.Config.editedRapidViewId = undefined;

/**
 * Renders the complete edit screen 
 */
GH.RapidBoard.Config.renderEditScreen = function(model) {
	// render the content shell
	AJS.$('#ghx-content-main').html(GH.tpl.rapid.view.renderConfigurationTabs());
	AJS.tabs.setup(); // create the aui tabs

	// render the header
	GH.RapidBoard.Config.renderHeader(model);

	// render the name edit
	GH.RapidBoard.Config.renderViewNameForm(model);
	
	// render the filter part
	GH.RapidBoard.Config.Filter.init(model);
	
	// column configuration
	GH.RapidBoard.Config.Columns.init(model);
	
	// swimlane configuration
	GH.RapidBoard.Config.Swimlanes.init(model);

    // quick filter configuration
	GH.RapidBoard.Config.QuickFilter.init(model);

    // retain the view id
    GH.RapidBoard.Config.editedRapidViewId = model.id;

    // initialize the jql autocomplete
    AJS.$(document).bind("contentRefresh", function (e, context) {
        var textfield = AJS.$(e.target).find('.js-jql-autocomplete-field');
        var error = AJS.$(e.target).find('.js-jql-autocomplete-error');
        if (textfield.length > 0 && error.length > 0) {
            var textFieldId = textfield.attr('id');
            var errorId = error.attr('id');
            GH.JQLAutoComplete.initialize(textFieldId, errorId);
        }
    });

    // initialise which tab we're looking at
    GH.RapidBoard.Config.initState();

    // initialise the tab and state-change listeners
    GH.RapidBoard.Config.initListeners();
};

/**
 * Renders the general information
 */
GH.RapidBoard.Config.renderHeader = function(model) {
	
    // render the header html
	GH.RapidBoard.Config.renderTitle(model.name);
    
    // render the config header html
    var canCreateView = GH.Global.username ? true : false;
    var header = AJS.$('#ghx-config-header');
    header.html(GH.tpl.rapid.view.renderConfigHeader({canCreateView:canCreateView}));
    // render hero actions
    AJS.$('#ghx-header').append(GH.tpl.rapid.manageviews.renderHero({ canCreateView:canCreateView }));

    // render any error messaging
    header.append(GH.tpl.rapid.view.renderSwimlaneConfigPermissionsWarning({'model' : model}));

    // if user cannot edit we change some styles, below adds/removes the hook
    if(model.canEdit){
        AJS.$('#gh').addClass('ghx-editor').removeClass('ghx-reader');
    } else {
        AJS.$('#gh').addClass('ghx-reader').removeClass('ghx-editor');
    }
};

GH.RapidBoard.Config.renderTitle = function(viewName) {
	AJS.$('#ghx-header').html(GH.tpl.rapid.navigation.renderSimpleHeader({name: AJS.I18n.getText('gh.rapid.config.title')+': '+viewName}));
};

//
// Rapid view name edit
// 

/**
 * Validates the rapid view name
 */
GH.RapidBoard.Config.validateViewName = function(editData) {
	var newValue = editData.editElement.val();
	
	// perform validation
    if (!GH.Validation.notBlank(editData.editElement, AJS.I18n.getText('gh.rapid.view.error.name.required'))) {
    	return false;
    }
    
    editData.newValue = newValue;
    return true;
};

/**
 * Saves the rapid view name
 */
GH.RapidBoard.Config.saveViewName = function(editData) {
    // TODO we don't currently do something on success, all errors are global.
    GH.Ajax.put({
        url : GH.Ajax.buildRestUrl('/rapidviewconfig/name'),
        contentType : 'application/json',
        data : JSON.stringify({
        	'id'   : GH.RapidBoard.Config.editedRapidViewId,
            'name' : editData.newValue
        })
    });
    
    // give fast feedback
    GH.RapidBoard.Util.InlineEditable.updateView(editData);
};

/**
 * Updates the header according to the new name
 */
GH.RapidBoard.Config.updateTitle = function(editData) {
	if (editData.canceled) return;
	
	GH.RapidBoard.Config.renderTitle(editData.newValue);
};

/**
 * Registers the live handler for the rapid view name edit
 */
GH.RapidBoard.Config.initRapidViewNameEdit = function() {
	// edit view name
	GH.RapidBoard.Util.InlineEditable.register('.js-edit-rapidViewName', {
        validate : GH.RapidBoard.Config.validateViewName,
        save : GH.RapidBoard.Config.saveViewName,
        renderView : GH.RapidBoard.Config.renderViewNameField,
        postEdit : GH.RapidBoard.Config.updateTitle
    });
};

/**
 * Initializes the view name form
 */
GH.RapidBoard.Config.renderViewNameForm = function(model) {
	// render the markup
    var viewNameEdit = AJS.$('#ghx-view-name-edit');
    viewNameEdit.html(GH.tpl.rapid.view.renderViewNameEditForm({rapidViewName: model.name}));
};

/**
 * Renders the column name view
 */
GH.RapidBoard.Config.renderViewNameField = function(editData) {
	var displayView = AJS.$(GH.tpl.rapid.view.renderViewNameField({
        rapidViewName : editData.newValue
    }));
	return displayView;
};

/**
 * Sets the active config tab to the one representated by tabKey
 */
GH.RapidBoard.Config.setTab = function(tabKey) {
    //get the tab id based on the key we're given
    var tab = GH.RapidBoard.Config.tabs[tabKey];
    //If we're given a dud tab key, we want to revert to the default
    if(!tab) {
        tab = GH.RapidBoard.Config.tabs[GH.RapidBoard.Config.defaultTab];
        //If we're setting the default tab we may as well replace it in the history
        GH.State.replace({
            "rapidView" : GH.RapidBoard.Config.editedRapidViewId,
            "tab" : GH.RapidBoard.Config.defaultTab
        });
    }

    // and set the active tab
    AJS.tabs.change(AJS.$("a[href=#" + tab + "]"));
};

/**
 * Sets the initial state
 */
GH.RapidBoard.Config.initState = function() {
    //Get the state out of the uri
    var state = GH.State.getStateFromUri(document.location.href);

    //set the selected tab to the one specified in the url
    GH.RapidBoard.Config.setTab(state.tab);

};

GH.RapidBoard.Config.initListeners = function() {
    AJS.$(document).bind("tabSelect", function(event, data) {
        var tab = "";
        for(var key in GH.RapidBoard.Config.tabs) {
            if(data.pane.attr("id") === GH.RapidBoard.Config.tabs[key])
            {
                tab = key;
                break;
            }
        }
        
        // the tab has been selected, update the url if it doesn't reflect the new state 
        if(tab) {
        	var state = GH.State.toViewState(window.location.href);
        	if (tab != state.tab) {
	            GH.State.push({
	                "rapidView" : GH.RapidBoard.Config.editedRapidViewId,
	                "tab" : tab
	            });
        	}
        }
    });

    // upon history event, select the correct tab
    GH.State.registerStateChangeListener(function() {
        var state = GH.State.toViewState(window.location.href);
        GH.RapidBoard.Config.setTab(state.tab);
    });
};
