/**
 * RapidBoard drag and drop functionality
 */
GH.RapidBoard.DragAndDrop = {};

/** Element that's being dragged */
GH.RapidBoard.DragAndDrop.currentDragElement = undefined;

/**
 * Registers an issue as draggable.
 */
GH.RapidBoard.DragAndDrop.registerIssueDraggable = function(event) {

    var elem = AJS.$(this);
    var row = elem.closest('.ghx-columns');

    if (GH.RapidBoard.RapidListDataController.isRankable()) {
        // ranking is enabled for this RV, so we're using a jQuery sortable
        // on the column.
        var column = elem.closest('.ghx-column');

    	// remember which columns have been initialised
    	if (column.data("dragInit")) return;
    	column.data("dragInit", true);
    	
        column.sortable({
        	revert: 0,
            zIndex: 3000,
            //containment: row,
            start: GH.RapidBoard.DragAndDrop.issueDragStart,
            stop: GH.RapidBoard.DragAndDrop.issueDragStop,
            items: '> .ghx-issue, > .ghx-parent-group',
            // don't drag on the fake parent and the subtask group (activates in IE8 only)
            cancel: '.js-fake-parent, .ghx-parent-group > .ghx-subtask-group',
            update: GH.RapidBoard.DragAndDrop.rank 
        });
        
        column.find('.ghx-subtask-group').sortable({
    		revert: 0,
            zIndex: 3000,
            containment: row,
            start: function(event,ui){
                GH.RapidBoard.DragAndDrop.issueDragStart(event,ui);
                var parent = AJS.$(ui.item).parent();
                if(parent.children().length > 2){ // 2 includes the drag helper
                    parent.addClass('ghx-contain');
                }
            },
            stop: function(event,ui){
                GH.RapidBoard.DragAndDrop.issueDragStop(event,ui);
                AJS.$(ui.item).parent().removeClass('ghx-contain');
            },
            items: '> .ghx-issue',
            update: GH.RapidBoard.DragAndDrop.rank
        });
        
    } else {
        // no ranking for this view, so we use only draggable since sortable
        // can't properly be restricted to not show visual sort indications
        
        // remember which issues have been initialised
        if (elem.data("dragInit")) return;
        elem.data("dragInit", true);
        
        elem.draggable({
            revert: true,
            revertDuration: 0,
            zIndex: 3000,
            containment: row,
            start: GH.RapidBoard.DragAndDrop.issueDragStart,
            stop: GH.RapidBoard.DragAndDrop.issueDragStop
        });        
    }
};

/** 
 * Issue ranking.
 */
GH.RapidBoard.DragAndDrop.rank = function (event, ui) {

    var draggedIssue = AJS.$(ui.item);
    var issueKeys = GH.RapidBoard.Issue.getSelectedIssuesInOrder();
    var prevIssueKey = GH.RapidBoard.Issue.getIssueKey(draggedIssue.prev('.ghx-issue, .ghx-parent-group'));
    var nextIssueKey = GH.RapidBoard.Issue.getIssueKey(draggedIssue.next('.ghx-issue, .ghx-parent-group'));

    GH.RapidBoard.DragAndDrop.rankIssues(issueKeys, nextIssueKey, prevIssueKey);
};

/**
 * Issue ranking.
 */
GH.RapidBoard.DragAndDrop.rankIssues = function (issueKeys, nextIssueKey, prevIssueKey, successFn) {

    var successHandler = function (data) {
        if (successFn) {
            successFn(data);
        }
        GH.RapidBoard.reload();
    };
    GH.Ajax.put({
        url: GH.Ajax.buildRestUrl('/rank'),
        data: JSON.stringify({ 
            issueKeys : issueKeys, 
            rankBeforeKey : nextIssueKey,
            rankAfterKey : prevIssueKey
        }),
        contentType: 'application/json',
        success: successHandler,
        errorContextMap: {
        	permission: function(ctxError, res) {
        		// show error
        		GH.RapidBoard.Notification.showWarnings({ errors : ctxError.errors});

        		// force the pool to be rerendered, to get the issue back to its original place
        		AJS.$(document).trigger('updateSwimlaneView');
        	}
        }
    });
};

/**
 * Called when issue dragging starts.
 */
GH.RapidBoard.DragAndDrop.issueDragStart = function(event, ui) {
    // distinguish between sortable (ui.item) and draggable (this)
    var card = ui.item ? AJS.$(ui.item) : AJS.$(this);
    
    // check whether we are the wrong dragged element
    if (card.parent)
    
	// Keep reference to draggable
    GH.RapidBoard.DragAndDrop.currentDragElement = card;

	// ensure the issue is selected
	var issueId = GH.RapidBoard.Issue.getIssueId(card);
    var issueKey = GH.RapidBoard.Issue.getIssueKey(card);

    // if the dragged issue is not selected, we want to select it
    if(!GH.RapidBoard.Issue.isInSelection(issueKey)) {
        // let Issue.js handle the click if the issue is compatible with the current selection
        if(GH.RapidBoard.Issue.canAddToSelection(issueKey)) {
            GH.RapidBoard.Issue.handleIssueClick.call(card, event);
        } else {
            // otherwise force a new selection
            GH.RapidBoard.Issue.selectIssue(issueKey);
        }
    }

	// destroy all droppables, they may be around from previous drag ops.
	AJS.$('.ghx-column').droppable('destroy');

    // fetch selected issues,
    var selectedIssues = GH.RapidBoard.Issue.getSelectedIssues();

	// kick off possible transitions querying (if we only have a single selected issue)
    if(selectedIssues.length == 1) {
	    GH.RapidBoard.DragAndDrop.queryPossibleTransitions(issueId, issueKey);
    }

    // handle multidrag
    if(selectedIssues.length > 1) {
        if(card.hasClass("ghx-parent-group")) {
            card.children('.ghx-issue').addClass('ghx-move-main').find('.ghx-move-count > b').text(selectedIssues.length);
        } else {
            card.addClass('ghx-move-main').find('.ghx-move-count > b').text(selectedIssues.length);
        }
    }

};

/**
 * Called when an issue drag operation stopped.
 */
GH.RapidBoard.DragAndDrop.issueDragStop = function(event, ui) {
    // distinguish between sortable (ui.item) and draggable (this)
    var card = ui.item ? AJS.$(ui.item) : AJS.$(this);

    // remove multi-drag label
    if(card.hasClass('ghx-parent-group')) {
        card.children('.ghx-issue').removeClass('ghx-move-main')
    } else {
        card.removeClass('ghx-move-main');
    }

	GH.RapidBoard.DragAndDrop.clearTransitionTargets();
	GH.RapidBoard.DragAndDrop.currentDragElement = null;
};

/**
 * Called when an issue is dropped.
 */
GH.RapidBoard.DragAndDrop.issueDropped = function(event, ui) {
	var draggable = ui.draggable;
	var droppable = AJS.$(this);

	// fetch the transition id directly from the droppable
	var issueId = GH.RapidBoard.Issue.getIssueId(AJS.$(this));
	var transitionId = droppable.attr('target-transition-id');
	
	GH.RapidBoard.DragAndDrop.executeWorkflowTransition(issueId, transitionId);
};

/**
 * Queries the possible transition targets.
 */
GH.RapidBoard.DragAndDrop.queryPossibleTransitions = function(issueId, issueKey) {
	AJS.$.ajax({
        url: GH.Ajax.buildRestUrl('/xboard/issue/transitions.json'),
        method: 'GET',
        data: {
            'issueId' : issueId
        },
        dataType: 'json',
        success: function(data) {
        	GH.RapidBoard.DragAndDrop.initTransitionTargets(issueKey, data.success);
        }
    });
};

/**
 * Initializes the transition targets for a given issue
 */
GH.RapidBoard.DragAndDrop.initTransitionTargets = function(issueKey, data) {
	var dragElement = GH.RapidBoard.DragAndDrop.currentDragElement;
	
	// ensure the data is valid, thus is for the current draggable
	if (!dragElement ||
		GH.RapidBoard.Issue.getIssueKey(dragElement) != issueKey) {
		return;
	}
	
	// fetch the swimlane the issue is in, as only columns in this lane should be highlighted
	var swimlane = dragElement.closest('.ghx-swimlane');
	
	// setup the drop zones
	swimlane.children('.ghx-columns').children('.ghx-column').each(function(index, col) {
	    GH.RapidBoard.DragAndDrop.initTargetColumn(issueKey, data, col);
	});
	
	// initial positioning of drop zones
	GH.RapidBoard.DragAndDrop.positionDropZones();

	// enabled drop zones scroll handler
	AJS.$(window).bind('scroll resize',GH.RapidBoard.DragAndDrop.positionDropZones);

	// force the draggable to reload its droppables
	GH.RapidBoard.DragAndDrop.forceDroppablesReload();
};

/**
 * Initializes a target column
 * @param data the transitions data object
 */
GH.RapidBoard.DragAndDrop.initTargetColumn = function(issueKey, data, column) {
    // find the data that corresponds to this column
    var colId = GH.RapidBoard.RapidListDataController.getColumnId(column);
    if (!colId) return;
    var colData = GH.RapidBoard.RapidListDataController.getColumn(colId);
    if (!colData) return;

    // ensure the dragged issue is not in that column, skip in that case
    var issueColumnId = GH.RapidBoard.RapidListDataController.getColumnIndexByKey(issueKey);
    if (issueColumnId == colId) return;
    
	// for each column, find the transitions that match
	var matchingTransitions = [];
	for (var x=0; x < data.transitions.length; x++) {
		for (var i=0; i < colData.statusIds.length; i++) {
			if (colData.statusIds[i] == data.transitions[x].targetStatus) {
				matchingTransitions.push(data.transitions[x]);
			}
		}
	}
	if (matchingTransitions.length == 0) return;
	
	// fetch the column and its header
	column = AJS.$(column);
	var columnHeader = AJS.$('.ghx-column-headers').children('.ghx-column').eq(column.index());
	
	// this is a valid target option
	column.addClass('ghx-target-option');
	columnHeader.addClass('ghx-target-option');

	if (matchingTransitions.length > 1) {
		// create a drop zones overlay
		GH.RapidBoard.DragAndDrop.setupDropZones(column, columnHeader, matchingTransitions);
	} else {
		// enable full column as drop zone
		column.attr('target-transition-id', matchingTransitions[0].id);
		GH.RapidBoard.DragAndDrop.registerColumnDroppable(column, columnHeader);
	}
};

/**
 * Registers a column droppable.
 */
GH.RapidBoard.DragAndDrop.registerColumnDroppable = function(column, columnHeader) {
    var options = { 
    	accept: '.ghx-issue, .ghx-parent-group',
    	hoverClass: 'ghx-target-hover',
        over: function() {
        	columnHeader.addClass('ghx-target-hover');
        },
        out: function() {
        	columnHeader.removeClass('ghx-target-hover');
        },
    	tolerance: 'pointer',
    	drop: GH.RapidBoard.DragAndDrop.issueDropped
    };
    column.droppable(options);
};

/**
 * Sets up the drop zones for a given column.
 */
GH.RapidBoard.DragAndDrop.setupDropZones = function(column, columnHeader, transitions) {
	// setup the drop zone container
	var dropZones = AJS.$('<div class="ghx-drop-zones"></div>');
	for (var i=0; i<transitions.length; i++ ) {
		var transition = transitions[i];
		var zone = AJS.$(GH.tpl.board.x.renderDropZone({name: transition.name}));

		// setup the droppable
		var options = {
        	accept: '.ghx-issue, .ghx-parent-group',
        	hoverClass: 'ghx-target-hover',
            over: function() { 
            	column.addClass('ghx-target-hover');
            	columnHeader.addClass('ghx-target-hover');
            },
            out: function() {
            	// only clear out the class if no other drop zone has the hover state already
            	// for some reason when dragging between zones the over even of one zone is triggered before the out event of another
            	if (dropZones.find('.ghx-drop-zone.ghx-target-hover').length < 1) {
            		column.removeClass('ghx-target-hover');
                	columnHeader.removeClass('ghx-target-hover');
            	}
            },
        	tolerance: 'pointer',
        	drop: GH.RapidBoard.DragAndDrop.issueDropped
        };
        zone.droppable(options);
        zone.attr('target-transition-id', transition.id); // specifies where to transition the issue to if dropped
        
        // now add it to the dropzone
		AJS.$(dropZones).append(zone);
	}
    //Add them into the tree hidden, because their dimensions are currently unknown.
	AJS.$(column).append(dropZones.hide());
};

/**
 * Forces the draggable to reload the target droppables on the next mouse move.
 */
GH.RapidBoard.DragAndDrop.forceDroppablesReload = function() {
	// ensure the draggable refreshes the position of all droppables (as they were not around when the drag started)
    var ddmanager = AJS.$.ui.ddmanager;
	var draggable = ddmanager.current;
	if (draggable) {
	    // this sets the visibility on all dropzones, which otherwise prevents them from being checked on over.
	    ddmanager.prepareOffsets(draggable, null);
	    
	    // simulate a drag event in case we're already over a dropzone and the user manages to not move the mouse at all,
	    // he'll still get the dropzone activated.
	    ddmanager.drag(draggable, null);
	}
};

/**
 * Positions the drop zones accordingly
 */
GH.RapidBoard.DragAndDrop.positionDropZones = function() {
	// fetch the dimensions of the available area
	var dimension = GH.RapidBoard.ViewController.getContentAreaDimension();

	// TODO: height calculation should be the same for all columns, thus could be calculated once only
	AJS.$('.ghx-drop-zones').each(function() {
		var dropZones = AJS.$(this);
		var enclosingColumn = dropZones.closest('.ghx-column');
		
		// gather some data
		var colHeight = enclosingColumn.height();
		var colOffset = enclosingColumn.offset();
		var colLeft = colOffset.left;
	    var colWidth = enclosingColumn.width() + 8;
	    var windowTop = AJS.$(window).scrollTop();
	    var windowHeight = AJS.$(window).height();
        var stalkerBottom = AJS.$("#ghx-column-header-group").outerHeight();

	    // calculate the top of the column
	    var colTop = colOffset.top + parseInt(enclosingColumn.css('padding-top')); // adjustment for swimlane height, parseInt will convert 8px to 8
	    
	    // calculate the bottom of the column
	    var colBottom = colTop + colHeight + parseInt(enclosingColumn.css('padding-bottom'));
	    
		// calculate the top of the dropzone, which is top offset - vertical windows scroll. Ensure the value is at least 0
	    var top = colTop - windowTop;
	    top = top < stalkerBottom ? stalkerBottom : top;
	    
	    // calculate the bottom of the dropzone.
	    var height = windowHeight;
	    // 1. reduce the height by top
	    height -= top;
	    // 2. reduce the height by possible visible footer
	    var bottomDistance = (windowTop + windowHeight - colBottom);
	    height -=  bottomDistance > 0 ? bottomDistance : 0;
	    
	    // set the size of the drop zones div
	    //var zoneHeight = dimension.top < colTop ? dimensionTop : colTop; 
	    //dropZones.css({'position':'fixed', 'top': dimension.top, 'left': colLeft, 'height': dimension.height, 'width': colWidth});
	    dropZones.css({'position':'fixed', 'top': top, 'left': colLeft, 'height': height, 'width': colWidth});
	    
	    // then distribute the height equally between the individual zones
	    var dropZoneHeight = dropZones.height() / dropZones.children().length;
	    dropZones.children('.ghx-drop-zone').height(dropZoneHeight);

	}).show(); //Now that all the drop zones have valid layout/positioning, we want to display them.
};

/**
 * Clears effects added as part of the transition targets.
 */
GH.RapidBoard.DragAndDrop.clearTransitionTargets = function() {
	AJS.$('.ghx-column').removeClass('ghx-target-option').removeAttr('target-transition-id'); // make sure we clear out the target transition ids
	AJS.$('.ghx-column').removeClass('ghx-target-hover');
	AJS.$('.ghx-drop-zones').remove();
	
	// disable drop zones scroll handler
	AJS.$(window).unbind('scroll resize',GH.RapidBoard.DragAndDrop.positionDropZones);
};

/**
 * Executes a workflow transition.
 */
GH.RapidBoard.DragAndDrop.executeWorkflowTransition = function(issueId, transitionId) {
	var xsrfToken = AJS.$('#atlassian-token').attr('content');
	var url = GH.Ajax.CONTEXT_PATH+'/secure/WorkflowUIDispatcher.jspa?id={0}&action='
			+ transitionId
			+ '&atl_token='
			+ xsrfToken
			+ '&returnUrl='; // not used in our case
	var link = '<a href="'+url+'" class="issueaction-workflow-transition"></a>';
	var $link = AJS.$(link);
	$link.click();
};

/**
 * Called on page load
 */
GH.RapidBoard.DragAndDrop.init = function() {
	//registers issues as draggables
	// we register this on mouse over only
	AJS.$(document).delegate('.ghx-issue', 'mouseover', GH.RapidBoard.DragAndDrop.registerIssueDraggable);
};
