/**
 * Defines functions called by the keyboard-shortcut modules.
 */
GH.Shortcut = {
	focusBoardSearch: function() {
		
	},
	newIssue: function() {
		AJS.$('#create_link').click();
	},
	nextBox: function() {
        GH.RapidBoard.Navigation.selectNextColumn();
	},
	previousBox: function() {
        GH.RapidBoard.Navigation.selectPreviousColumn();
	},
	toggleBox: function() {
		
	},
	toggleIssue: function() {
        GH.RapidBoard.DetailsView.toggleDetailsView();
	},
	editIssuePriority: function() {
		
	},
	deleteIssue: function() {
		
	},
	nextIssue: function() {
        GH.RapidBoard.Navigation.selectNextIssue();
	},
	previousIssue: function() {
        GH.RapidBoard.Navigation.selectPreviousIssue();
	},
	viewIssue: function() {
        GH.RapidBoard.Shortcuts.viewSelectedIssue();
	},
	editIssue: function() {
        GH.RapidBoard.Shortcuts.editSelectedIssue();
	},
	assignIssue: function() {
        GH.RapidBoard.Shortcuts.assignSelectedIssue();
	},
	commentIssue: function() {
        GH.RapidBoard.Shortcuts.commentSelectedIssue();
	},
	editIssueLabels: function() {
        GH.RapidBoard.Shortcuts.editSelectedIssueLabels();
	},
	showOperationsDialog: function() {
        if(GH.RapidBoard.ViewController.isPoolDisplayed()) {
		    JIRA.Dialogs.issueActions.show();
        }
	},
	rightColumnToggle: function() {

	},
    projector: function() {
        GH.Projector.toggleUI();
	},
    sendToTop: function() {
        GH.RapidBoard.Shortcuts.sendToTop();
	},
    sendToBottom: function() {
        GH.RapidBoard.Shortcuts.sendToBottom();
	}
};


GH.RapidBoard.Shortcuts = {};

GH.RapidBoard.Shortcuts.viewSelectedIssue = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (issueKey) {
        var url = GH.Ajax.CONTEXT_PATH+'/browse/'+issueKey;
        window.location.href = url;
	}
};

GH.RapidBoard.Shortcuts.editSelectedIssue = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (issueKey) {
        var url = GH.Ajax.CONTEXT_PATH+'/secure/EditIssue!default.jspa?key='+issueKey + '&returnUrl=' + GH.RapidBoard.getCurrentPageReturnUrl();
        window.location.href = url;
	}
};

GH.RapidBoard.Shortcuts.assignSelectedIssue = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (issueKey) {
		AJS.$('a.issueaction-assign-issue').trigger('click');
	}
};

GH.RapidBoard.Shortcuts.commentSelectedIssue = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (issueKey) {
		AJS.$('a.issueaction-comment-issue').trigger('click');
	}
};

GH.RapidBoard.Shortcuts.editSelectedIssueLabels = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (issueKey) {
		AJS.$('a.issueaction-edit-labels').trigger('click');
	}
};

/**
 * Re-ranks the selected issue(s) so that they are at the top of their swimlane-column.
 */
GH.RapidBoard.Shortcuts.sendToTop = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    // we don't want to do anything if the current view is not rankable
    if (!GH.RapidBoard.RapidListDataController.isRankable()) {
        return;
    }

    // do analytics before we even know if they are successful
    AJS.$(AJS).trigger("gh.rapidboard.rankissues", {
        action: "sendToTop",
        label: "1" // the number of issues TODO once we have multi-select, use the number of issues selected
    });

    // get the currently selected issue
    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (!issueKey) {
        return;
    }

    // get the issue which we will be ranking before
    var firstVisibleIssueKey = GH.RapidBoard.RapidListDataController.getFirstRankableIssueKeyInColumn(issueKey);
    if (!firstVisibleIssueKey) {
        return;
    }

    // get the multi-selected issues, in currently sorted order.
    var selectedIssuesInOrder = GH.RapidBoard.Issue.getSelectedIssuesInOrder();

    // get a handle of the issue to select after we re-rank
    // this will be the issue that is going INTO our position - i.e. the previous issue
    var issueKeyToSelectAfterRanking = GH.RapidBoard.RapidListDataController.getBestFitSelectionAfterRank(selectedIssuesInOrder);
    var currentColumnIndex = GH.RapidBoard.RapidListDataController.getColumnIndexByKey(issueKey);

    // define a success handler
    var successHandler = function (data) {
        // select the issue if we got one
        if (issueKeyToSelectAfterRanking) {
            GH.RapidBoard.Issue.gotoIssue(issueKeyToSelectAfterRanking);
        }

        // show a success message
        GH.Message.showSuccess(AJS.I18n.getText("gh.keyboard.shortcut.send.to.top.success"));
    };

    // do the re-ranking
    GH.RapidBoard.DragAndDrop.rankIssues(selectedIssuesInOrder, firstVisibleIssueKey, null, successHandler);
};

/**
 * Re-ranks the selected issue(s) so that they are at the bottom of their swimlane-column.
 */
GH.RapidBoard.Shortcuts.sendToBottom = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    // we don't want to do anything if the current view is not rankable
    if (!GH.RapidBoard.RapidListDataController.isRankable()) {
        return;
    }

    // do analytics before we even know if they are successful
    AJS.$(AJS).trigger("gh.rapidboard.rankissues", {
        action: "sendToBottom",
        label: "1" // the number of issues TODO once we have multi-select, use the number of issues selected
    });

    // get the currently selected issue
    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (!issueKey) {
        return;
    }

    // get the issue which we will be ranking after
    var lastVisibleIssueKey = GH.RapidBoard.RapidListDataController.getLastRankableIssueKeyInColumn(issueKey);
    if (!lastVisibleIssueKey) {
        return;
    }

    // get the multi-selected issues, in currently sorted order.
    var selectedIssuesInOrder = GH.RapidBoard.Issue.getSelectedIssuesInOrder();

    // get a handle of the issue to select after we re-rank
    // this will be the issue that is going INTO our position - i.e. the next issue
    var issueKeyToSelectAfterRanking = GH.RapidBoard.RapidListDataController.getBestFitSelectionAfterRank(selectedIssuesInOrder);
    var currentColumnIndex = GH.RapidBoard.RapidListDataController.getColumnIndexByKey(issueKey);

    // define a success handler
    var successHandler = function (data) {
        // select the issue if we got one
        if (issueKeyToSelectAfterRanking) {
            GH.RapidBoard.Issue.gotoIssue(issueKeyToSelectAfterRanking);
        }

        // show a success message
        GH.Message.showSuccess(AJS.I18n.getText("gh.keyboard.shortcut.send.to.bottom.success"));
    };

    // do the re-ranking
    GH.RapidBoard.DragAndDrop.rankIssues(selectedIssuesInOrder, null, lastVisibleIssueKey, successHandler);
};

/**
 * Navigation keyboard shortcuts
 */
GH.RapidBoard.Navigation = {};

/**
 * Holds the currently selected index. Used for column traversing to not reset the
 * selected index every time we pass through a column containing less items.
 */
GH.RapidBoard.Navigation.magicIndex = 0,
/**
 * Holds whether a column change has been performed, in which case the magic index won't be updated
 */
GH.RapidBoard.Navigation.columnChange = false; 

/**
 * Initializes the magic index according to the currently selected issue.
 */
GH.RapidBoard.Navigation.initMagicIndex = function() {
	var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
	if (issueKey) {
		GH.RapidBoard.Navigation.setMagicIndex(issueKey);
	} else {
		GH.RapidBoard.Navigation.magicIndex = false;
	}
}

/**
 * Updates the selected issue index - if t. */
GH.RapidBoard.Navigation.adaptMagicIndex = function(event, issueKey) {
	// skip if this is a column change
	if (GH.RapidBoard.Navigation.columnChange) {
		GH.RapidBoard.Navigation.columnChange = false;
		return;
	}

    GH.RapidBoard.Navigation.setMagicIndex(issueKey);
};

/**
 * Sets the magic index to the index of the provided issue
 */
GH.RapidBoard.Navigation.setMagicIndex = function(issueKey) {
    if (issueKey) {
	    GH.RapidBoard.Navigation.magicIndex = GH.RapidBoard.RapidListDataController.getIssueVisibilityIndexByKey(issueKey);
    } else {
        GH.RapidBoard.Navigation.magicIndex = 0;
    }
};

/**
 * Get the current magic index value. returns 0 if not set
 */
GH.RapidBoard.Navigation.getMagicIndex = function() {
	return GH.RapidBoard.Navigation.magicIndex !== false ? GH.RapidBoard.Navigation.magicIndex : 0; 
};

/**
 * Select the next issue in a current column, or the first issue in the first column if none selected.
 */
GH.RapidBoard.Navigation.selectNextIssue = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (!issueKey) {
        // try selecting the first issue in the view
        var firstIssueKey = GH.RapidBoard.RapidListDataController.getFirstIssueKey();
        if (firstIssueKey) {
            GH.RapidBoard.Issue.selectIssue(firstIssueKey);
        }
    } else if (GH.RapidBoard.RapidListDataController.issueIsInCollapsedSwimlaneByKey(issueKey)) {
        GH.RapidBoard.SwimlaneView.setCollapsedById(
            GH.RapidBoard.RapidListDataController.getSwimlane(
                GH.RapidBoard.RapidListDataController.getSwimlaneIndexByKey(issueKey)
            ).id
        );
    } else {
        // only select next issue if there is one
        var nextIssueKey = GH.RapidBoard.RapidListDataController.getNextIssueKey(issueKey);
        if (nextIssueKey) {
            GH.RapidBoard.Issue.selectIssue(nextIssueKey);
        }
    }
},

/**
 * Select the previous issue in the current, or the last issue in the first column if none selected.
 */
GH.RapidBoard.Navigation.selectPreviousIssue = function() {
    if(!GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (!issueKey) {
        // try selecting the last issue in the view
        var lastIssueKey = GH.RapidBoard.RapidListDataController.getLastIssueKey();
        if (lastIssueKey) {
            GH.RapidBoard.Issue.selectIssue(lastIssueKey);
        }
    } else if (GH.RapidBoard.RapidListDataController.issueIsInCollapsedSwimlaneByKey(issueKey)) {
        GH.RapidBoard.SwimlaneView.setCollapsedById(
            GH.RapidBoard.RapidListDataController.getSwimlane(
                GH.RapidBoard.RapidListDataController.getSwimlaneIndexByKey(issueKey)
            ).id
        );
    } else {
        // only select next issue if there is one
        var prevIssueKey = GH.RapidBoard.RapidListDataController.getPreviousIssueKey(issueKey);
        if (prevIssueKey) {
            GH.RapidBoard.Issue.selectIssue(prevIssueKey);
        }
    }
};

/**
 * Select an issue in the next column or the first issue in the first column if none selected.
 * The function attempts to keep the selected index. 
 */
GH.RapidBoard.Navigation.selectNextColumn = function() {
    if (! GH.RapidBoard.ViewController.isPoolDisplayed()) return;

    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
	// simply select the first issue if we haven't got one
	if (!issueKey) {
		GH.RapidBoard.Navigation.selectNextIssue();
		return;
	}

	// fetch the data for the issue, which will give us the swimlane id
	var issueData = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	if (!issueData) {
		GH.RapidBoard.Navigation.selectNextIssue();
		return;
	}

    if (GH.RapidBoard.RapidListDataController.issueIsInCollapsedSwimlaneByKey(issueKey)) {
        GH.RapidBoard.SwimlaneView.setCollapsedById(issueData.swimlane.id);
        return;
    }

    var issueKeyToSelect = GH.RapidBoard.RapidListDataController.getIssueKeyInNextColumn(issueData, GH.RapidBoard.Navigation.getMagicIndex());
    if(issueKeyToSelect) {
        GH.RapidBoard.Navigation.columnChange = true;
        GH.RapidBoard.Issue.selectIssue(issueKeyToSelect);
    }

},

/**
 * Select an issue in the previous column or the first issue in the last column if none selected.
 * The function attempts to keep the selected index. 
 */
GH.RapidBoard.Navigation.selectPreviousColumn = function() {
    if (! GH.RapidBoard.ViewController.isPoolDisplayed()) return;

	var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
	// simply select the first issue if we haven't got one
	if (!issueKey) {
		GH.RapidBoard.Navigation.selectNextIssue();
        return;
	}
	
	// fetch the data for the issue, which will give us the swimlane id
	var issueData = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	if (!issueData) {
		GH.RapidBoard.Navigation.selectNextIssue();
		return;
	}

    if (GH.RapidBoard.RapidListDataController.issueIsInCollapsedSwimlaneByKey(issueKey)) {
        GH.RapidBoard.SwimlaneView.setCollapsedById(issueData.swimlane.id);
        return;
    }

    var issueKeyToSelect = GH.RapidBoard.RapidListDataController.getIssueKeyInPreviousColumn(issueData, GH.RapidBoard.Navigation.getMagicIndex());
    if(issueKeyToSelect) {
        GH.RapidBoard.Navigation.columnChange = true;
        GH.RapidBoard.Issue.selectIssue(issueKeyToSelect);
    }
};

/**
 * Called on page load.
 */
GH.RapidBoard.Navigation.init = function() {
	// initialize the magic inded on model change
	AJS.$(document).bind('modelChanged', GH.RapidBoard.Navigation.initMagicIndex);
	
	// update the magic index on issue selection
	AJS.$(document).bind('issueSelected', GH.RapidBoard.Navigation.adaptMagicIndex);
	
    /**
     * Handles the return key, which should act like the o key (view issue)/
     */
    AJS.$(document).keydown(function(e){
        if (e.keyCode == '13'){
            var target = e.target;
            // On different browser the originalTarget is different, but all of these are impossible for a user to trigger.
            if (target === undefined || target.nodeName === "HTML" || target.nodeName === "BODY" || target == document){
                GH.RapidBoard.Shortcuts.viewSelectedIssue();
            }
        }
    });
};
