/**
 * Note: this is a copy of 
 * https://bitbucket.org/shghs/aui/src/612a10e756ce/src/storage.js.
 * 
 * Slightly adapted to allow using either local or session storage, defaulting to local storage if not provided
 */

// require AUI
// provide AUI.storage

GH.storage = {};

/**
 * @param {string} key
 * @returns {Object|string|number|boolean}
 */
GH.storage.get = function(key, useSessionStorage) {
  if (useSessionStorage) {
    var value = sessionStorage.getItem(key);  
  } else {
    var value = localStorage.getItem(key);
  }
  return (typeof value === "string") ? JSON.parse(value) : value;
};

/**
 * @param {string} key
 * @param {Object|string|number|boolean} value
 * @returns {boolean}
 */
GH.storage.put = function(key, value, useSessionStorage) {
  if (value === null || value === undefined) {
	if (useSessionStorage) {
      sessionStorage.removeItem(key);
	} else {
      localStorage.removeItem(key);
	}
  } else {
    try {
      if (useSessionStorage) {
        sessionStorage.setItem(key, JSON.stringify(value));
      } else {
        localStorage.setItem(key, JSON.stringify(value));
      }
    } catch (QUOTA_EXCEEDED_ERR) {
      return false;
    }
  }
  return true;
};

if (!window.localStorage || !window.sessionStorage) {
  // When localStorage is unavailable, get and put methods always fail.
  GH.storage.get = function() {
    return null;
  };
  GH.storage.put = function() {
    return false;
  };
}
