/**
 * Details view component.
 */
GH.RapidBoard.DetailsView = {};

GH.RapidBoard.DetailsView.MORE_COMMENTS_COUNT = 5;

/**
 * Holds the view data object
 */
GH.RapidBoard.DetailsView.viewData = null;

/**
 * Holds the edit data object
 */
GH.RapidBoard.DetailsView.editData = null;

/**
 * Resize the column size.
 */
GH.RapidBoard.DetailsView.updateSize = function() {
	// currently we only size the issue details part, the rest is fixed
    var detailIssue = AJS.$('#ghx-detail-issue');
    if(detailIssue.length > 0){
    	var column = AJS.$('#ghx-detail');
//        column.width(AJS.$('#ghx-pool').width() - 9); // 50%
        column.width(AJS.$('#ghx-rabid').width() - AJS.$('#ghx-pool').width() - 2); // 30% test
        var dPos = detailIssue.position();
        var colHeight = column.height() - dPos.top;
        detailIssue.css({'height':colHeight});
    }
};

/**
 * Get the container the details view is rendered in
 */
GH.RapidBoard.DetailsView.getContainer = function() {
	return AJS.$('#ghx-detail');
};

/**
 * Returns the fixed element data (the fixed element size is managed by the ViewController)
 */
GH.RapidBoard.DetailsView.getFixedElement = function() {
	return {
		element: GH.RapidBoard.DetailsView.getContainer(),
		resizeCallback: GH.RapidBoard.DetailsView.updateSize
	};
};

GH.RapidBoard.DetailsView.currentKey = undefined;

/**
 * Renders the column body.
 * Always tries to update the details
 */
GH.RapidBoard.DetailsView.loadIssueDetails = function(event) {
	// save the work if we are not displayed
	if (!GH.RapidBoard.State.isDetailsViewOpened()) return;

	// fetch the key we want to display
	var issueKey = GH.RapidBoard.State.getSelectedIssueKey();
	if (! issueKey) {
		// issue display should not even be open. close
		GH.RapidBoard.DetailsView._closeDetailsView();
		return;
	}
	
	// update the current key we're trying to display
    // always reload the Detail View even if we're already looking at the issue
	GH.RapidBoard.DetailsView.currentKey = issueKey; // retain to compare once we get the result back
	
	// render the content
	var loadingContent = AJS.$(GH.tpl.board.x.renderIssueDetailsLoading());
	var contentDiv = AJS.$('#ghx-detail-contents');
	if (contentDiv.length > 0) {
		contentDiv.empty().append(loadingContent);
		
		// kick off loading of the content
		GH.RapidBoard.DetailsView.loadContent(issueKey);
		
	    // ensure we don't keep stale data around
	    GH.RapidBoard.DetailsView.viewData = null;
	    GH.RapidBoard.DetailsView.editData = null;
	    GH.RapidBoard.DetailFields.clearEditData();
	}
};

/**
 * Updates the issue details view.
 */
GH.RapidBoard.DetailsView.loadContent = function(issueKey) {
    // issue an ajax request
    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/xboard/issue/details.json'),
        data: {
            //'issueId' : issueId
        	issueKey: issueKey
        },
        success: function(data) {
        	// render the details
        	GH.RapidBoard.DetailsView.renderDetails(data)
        },
        onGlobalError: function(data) {
            GH.RapidBoard.DetailsView.loading = false;
            GH.RapidBoard.State.setSelectedIssueKey(undefined);
            GH.RapidBoard.DetailsView._closeDetailsView();
        }
    }, 'detailView');
};

/**
 * Renders the details view given the provided issue data.
 */
GH.RapidBoard.DetailsView.renderDetails = function(viewData) {
	// check whether the data belongs to what we want to render, otherwise directly ns
	if (viewData.key != GH.RapidBoard.DetailsView.currentKey) {
		// wrong key, discard and try again...
		GH.RapidBoard.DetailsView.loadIssueDetails();
		return;
	}
	
	// fixes xrfs token and sets correct return url
	GH.RapidBoard.DetailsView.applyOperationUrlPatches(viewData);
	
	// render the contents
    var detailContents = AJS.$(GH.tpl.board.x.renderIssueDetailsContents({issueDetails: viewData}));

    // render all fields
    var issueFields = detailContents.find('.ghx-issue-fields');
    var renderedFields = GH.RapidBoard.DetailFields.renderFields(viewData, issueFields);
    
    // replace the content
    var contentDiv = AJS.$('#ghx-detail-contents'); 
    contentDiv.empty().append(detailContents);

    // create dropdowns for the more operations
    if (AJS.$("#opsbar-opsbar-operations .aui-list").length > 0) {
	    var operationsDropdown = AJS.Dropdown.create({
	        trigger: AJS.$("#opsbar-opsbar-operations .more"),
	        content: AJS.$("#opsbar-opsbar-operations .aui-list"),
	        alignment: AJS.LEFT  	
	    });
    }
    if (AJS.$("#opsbar-opsbar-transitions .aui-list").length > 0) {
	    var transitionsDropdown = AJS.Dropdown.create({
	        trigger: AJS.$("#opsbar-opsbar-transitions .more"),
	        content: AJS.$("#opsbar-opsbar-transitions .aui-list"),
	        alignment: AJS.LEFT  	
	    });
    }
    
    // GHS-3059: don't display comment field if no comment set
    if (viewData.comments.length > 0) {
        // escape the comment's restriction role name
        GH.RapidBoard.DetailsView.escapeCommentsRestriction(viewData.comments);
        // comments rendering
        AJS.$('#ghx-detail-issue').append(AJS.$(GH.tpl.board.x.renderCommentsStructure({
            comments: viewData.comments,
            numComments: viewData.comments.length,
            totalComments: viewData.totalComments
        })));
    }
    
    // updating the size
    GH.RapidBoard.DetailsView.updateSize();
    
    // store the view data
    GH.RapidBoard.DetailsView.viewData = viewData;
};

/**
 * Replaces atl_token= by atl_token={tokenValue}.
 * The reason for the token not being there is that we generate the links through a rest call, but SimpleLinkFactory uses
 * the webwork Action context to figure out the current request in order to generate the token... Thanks but no thanks!
 */
GH.RapidBoard.DetailsView.applyOperationUrlPatches = function(viewData) {
	// fetch the return url to set
	var returnUrl = GH.RapidBoard.getCurrentPageReturnUrl();
	// fetch the xrfs token to add
	var token = AJS.$('#atlassian-token').attr('content');
	
	// go through all operations, which are either in primaryOperations, or inside each section of moreSections
	if (viewData.operations) {
		for (var i=0; i < viewData.operations.groups.length; i++) {
			var group = viewData.operations.groups[i];
			for (var j=0; j < group.primaryOperations.length; j++) {
				var operation = group.primaryOperations[j];
				operation.url = GH.RapidBoard.DetailsView.patchIssueOperationUrl(operation.url, returnUrl, token); 
			}
			for (var k=0; k < group.moreSections.length; k++) {
				var section = group.moreSections[k];
				for (var l=0; l < section.operations.length; l++) {
					var operation = section.operations[l];
					operation.url = GH.RapidBoard.DetailsView.patchIssueOperationUrl(operation.url, returnUrl, token);
				}
			}
		}
	}
};

GH.RapidBoard.DetailsView.issueIdReplacementRegex = /\?id=\d+/; 
GH.RapidBoard.DetailsView.patchIssueOperationUrl = function(url, returnUrl, token) {
	// set the returnUrl
	url += (url.indexOf('?') < 0) ? '?' : '&';
	url += 'returnUrl='+returnUrl;
	
	// fill in atl_token, but only if not already set
	var newToken = 'atl_token='+token;
	if (url.indexOf(newToken) < 0) {
		url = url.replace('atl_token=', newToken);
	}
	
	// remove the hardcoded issue id from all workflow transitions, replace by {0}
	// we have to do this because the transition dialog otherwise "remembers" the first executed issue id
	if (url.indexOf('WorkflowUIDispatcher.jspa') > -1) {
		url = url.replace(GH.RapidBoard.DetailsView.issueIdReplacementRegex, '?id={0}');
	}
	
	return url;
};

/**
 * Toggles the details view on and off.
 * This function is called by the keyboard shortcut!
 */
GH.RapidBoard.DetailsView.toggleDetailsView = function() {
	// only continue if the pool is currently displayed
	if (! GH.RapidBoard.ViewController.isPoolDisplayed()) return;
	
	if (GH.RapidBoard.DetailsView.isDetailsViewDisplayed()) {
		GH.RapidBoard.DetailsView._closeDetailsView();
	} else {
		var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
		GH.RapidBoard.DetailsView.showIssueDetails(issueKey);
	}
};

/** Handles the click on the issue key (force opening the details view) */
GH.RapidBoard.DetailsView.handleOpenIssueDetailsLink = function(event) {
	if (event) {
		event.preventDefault();
		
		// don't handle click if either is pressed
		var metaDown = event.metaKey;
	    var shiftDown = event.shiftKey;
		if (metaDown || shiftDown) return;
	}
	
	// fetch the issue id, then call show
	var issueKey = GH.RapidBoard.Issue.getIssueKey(AJS.$(this));
	if (!issueKey) return;
	GH.RapidBoard.DetailsView.showIssueDetails(issueKey);
};

GH.RapidBoard.DetailsView.showIssueDetails = function(issueKey) {
	if (!issueKey) return;
	
	// ensure we are selected
	GH.RapidBoard.Issue.selectIssue(issueKey, true);
	
	// either open the view or update the details
	if (!GH.RapidBoard.DetailsView.isDetailsViewDisplayed()) {	
		GH.RapidBoard.DetailsView._openDetailsView();
	} else {
		AJS.$(document).trigger('updateDetailsView');
	}
};

/**
 * Opens the details view.
 */
GH.RapidBoard.DetailsView._openDetailsView = function() {
	GH.RapidBoard.State.setDetailsViewOpened(true);
	GH.RapidBoard.State.pushState();
    AJS.$(document).trigger('showDetailsView');
};

/** Handles the details view close link. */
GH.RapidBoard.DetailsView.handleCloseDetailsViewLink = function(event) {
	if (event) {
		event.preventDefault();
	}
	GH.RapidBoard.DetailsView._closeDetailsView();
};

/**
 * Triggers the closing of the details view.
 */
GH.RapidBoard.DetailsView._closeDetailsView=function(event) {
	GH.RapidBoard.State.setDetailsViewOpened(false);
	GH.RapidBoard.State.pushState();
    AJS.$(document).trigger('hideDetailsView');
};

/**
 * Is the details view currently displayed?
 */
GH.RapidBoard.DetailsView.isDetailsViewDisplayed = function() {
	return AJS.$('#ghx-detail:visible').length > 0;
};

//
// Edit functionality
//

/**
 * Handles the click on a field.
 */
GH.RapidBoard.DetailsView.handleDetailsFieldClick = function(event) {
	// check whether we are an editable field, do nothing otherwise 
	var clickedElem = AJS.$(this);
	var fieldElem = clickedElem.closest('dd.js-editable-field');
	if (fieldElem.length < 1) return;
	
	// fetch required values
	var fieldId = fieldElem.attr('data-field-id');
	var issueId = fieldElem.closest('#ghx-detail-issue').attr('data-issueid');
	
	// kick off editing.
	GH.RapidBoard.DetailsView.editField(issueId, fieldId);
	
	// make sure the default for the event does not happen, otherwise we loose focus right as the click is fired not against the replaced edit
	event.preventDefault();
};

/**
 * Passes the edit to the sibling field, specified by delta.
 */
GH.RapidBoard.DetailsView.editRelativeField = function(focusedFieldId, delta) {
	var data = GH.RapidBoard.DetailsView.editData 
	for (var i=0; i < data.fields.length; i++) {
		if (data.fields[i].id == focusedFieldId) {
			if (data.fields[i+delta]) {
				GH.RapidBoard.DetailsView.editField(data.id, data.fields[i+delta].id);
				return true;
			} else {
				return false;
			}
		}
	}
	return false;
};

/**
 * Initiates field editing for a given field. Either forwards the work to the DetailFields or triggers a data load
 * if not yet loaded
 */
GH.RapidBoard.DetailsView.editField = function(issueId, fieldId) {
	// check whether we are already editing, just focus in this case
	if (GH.RapidBoard.DetailFields.isEditing(fieldId)) {
		GH.RapidBoard.DetailFields.focusEditedField(fieldId);
		return;
	}
	
	// check whether we can reuse the current edit data or have to reload it
	// currently we reload unless we are currently already editing another field
	if (GH.RapidBoard.DetailsView.editData &&
		GH.RapidBoard.DetailsView.editData.id == issueId &&
		GH.RapidBoard.DetailFields.getEditingCount() > 0)
	{
		GH.RapidBoard.DetailsView.startEdit(GH.RapidBoard.DetailsView.editData, fieldId);
		return;
	}
	
	// load the data
	GH.RapidBoard.DetailsView.loadEditData(issueId, fieldId);
};

/**
 * Loads the edit data and kicks off editing once loaded.
 */
GH.RapidBoard.DetailsView.loadEditData = function(issueId, fieldToEditUponCompletion) {
	// Double click multiple loads prevention
	if (GH.RapidBoard.DetailsView.viewData && GH.RapidBoard.DetailsView.viewData.editIsLoading) return;
	GH.RapidBoard.DetailsView.viewData.editIsLoading = true;
	
    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/xboard/issue/edit-data.json'),
        data: {
            'issueId' : issueId
        },
        success: function(data) {
        	GH.RapidBoard.DetailsView.viewData.editIsLoading = false;
        	GH.RapidBoard.DetailsView.editData = data;
        	GH.RapidBoard.DetailsView.startEdit(data, fieldToEditUponCompletion);
        }
    });
};

/**
 * Opens the editor for a given field
 */
GH.RapidBoard.DetailsView.startEdit = function(editData, fieldId) {
	// find the field info inside the editData
	var fieldData;
	for (var i=0; i < editData.fields.length; i++) {
		if (editData.fields[i].id == fieldId) {
			fieldData = editData.fields[i];
			break;
		}
	}
	if (!fieldData) return;
	
	// mark the field as being edited and hand over to details field for editing
	GH.RapidBoard.DetailFields.editField(editData, fieldData);
};

/**
 * Called on a successful field update
 */
GH.RapidBoard.DetailsView.handleFieldUpdateSuccess = function(fieldData) {
	// make sure we are still on the same detail view
	if (GH.RapidBoard.DetailsView.viewData && GH.RapidBoard.DetailsView.viewData.id == fieldData.issueId) {
		// update the view data for the given field
		var viewData = GH.RapidBoard.DetailsView.viewData;
		for (var i=0; i < viewData.fields.length; i++) {
			if (viewData.fields[i].id == fieldData.view.id) {
				viewData.fields[i] = fieldData.view;
				break;
			}
		}
		
		// 	update the ui
		GH.RapidBoard.DetailFields.updateField(fieldData.view);
		
		// update the edit data for the given field
		var editData = GH.RapidBoard.DetailsView.editData;
		if (editData) {
			for (var i=0; i < editData.fields.length; i++) {
				if (editData.fields[i].id == fieldData.edit.id) {
					editData.fields[i] = fieldData.edit;
					break;
				}
			}
		}
	}
	else {
		// show success message
		var message = GH.tpl.board.x.fieldUpdateSuccess();
		GH.Message.showSuccess(message);
	}
	
	// then fire an event to update the rest of the ui
	AJS.$(document).trigger('issueUpdated', fieldData.issueId);
};

/**
 * Called on a failed field update
 */
GH.RapidBoard.DetailsView.handleFieldUpdateFailure = function(issueId, fieldId, errorCollection) {
	// make sure we are still on the same view
	if (GH.RapidBoard.DetailsView.viewData &&
		GH.RapidBoard.DetailsView.viewData.id == issueId &&
		GH.RapidBoard.DetailFields.isEditing(fieldId)
	) {
		// show errors
	    if (errorCollection.contextualErrors) {
	        for (var i=0; i < errorCollection.contextualErrors.length; i++) {
	            var fieldError = errorCollection.contextualErrors[i];
	            GH.RapidBoard.DetailFields.showFieldErrors(fieldError.contextId, fieldError.errors);
	        }
	    }
	}
	else {
		// display warning message
		var message = GH.tpl.board.x.fieldUpdateFailure();
		GH.Message.showWarning(message);
	}
};

//
// Comment functionality
//

/**
 * Handles 'Show more comments' button
 */
GH.RapidBoard.DetailsView.handleCommentsMoreClick = function(event) {
	// issue an ajax request
    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/xboard/comment/more.json'),
        data: {
        	issueKey: GH.RapidBoard.State.getSelectedIssueKey(),
            numComments: GH.RapidBoard.DetailsView.getNumDisplayedComments() + GH.RapidBoard.DetailsView.MORE_COMMENTS_COUNT
        },
        success: function(data) {
        	// render the comments
        	GH.RapidBoard.DetailsView.renderComments(data)
        }
    });
};

/**
 * Handles 'Show all comments' button
 */
GH.RapidBoard.DetailsView.handleCommentsAllClick = function(event) {
	// issue an ajax request
    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/xboard/comment/all.json'),
        data: {
        	issueKey: GH.RapidBoard.State.getSelectedIssueKey()
        },
        success: function(data) {
        	// render the comments
        	GH.RapidBoard.DetailsView.renderComments(data)
        }
    });
};

GH.RapidBoard.DetailsView.getNumDisplayedComments = function() {
    return AJS.$('#ghx-detail-comments .ghx-comment').length;
}

/**
 * Helper function to escape the comments restrictions roles
 */
GH.RapidBoard.DetailsView.escapeCommentsRestriction = function(comments) {
    for (var i = 0; i < comments.length; i++) {
        var role = comments[i].restrictedTo;
        // role is null if there is no restriction
        if (role) {
            comments[i].restrictedTo = soy.$$escapeHtml(role);
        }
    }
};

/**
 * Renders the comments
 */
GH.RapidBoard.DetailsView.renderComments = function(viewData) {

    // current number of comments
    var numComments = GH.RapidBoard.DetailsView.getNumDisplayedComments();

    // escape the comment's restriction role name
    GH.RapidBoard.DetailsView.escapeCommentsRestriction(viewData.comments);
    AJS.$('#ghx-detail-comments .ghx-comment:last').append(AJS.$(GH.tpl.board.x.renderComments({
        // only render comments that are currently not displayed
        comments: viewData.comments.slice(numComments)
    })));

    // update the number of comments now displayed
    numComments = GH.RapidBoard.DetailsView.getNumDisplayedComments();

    // update the underbar
    AJS.$('.ghx-underbar').html(AJS.$(GH.tpl.board.x.renderCommentsUnderbar({
        numComments: numComments,
        totalComments: GH.RapidBoard.DetailsView.viewData.totalComments
    })));
};

/**
 * initializes the detail view stuff
 */
GH.RapidBoard.DetailsView.init = function() {
    
    // showing issue details
    AJS.$('#ghx-pool').delegate('.js-detailview','click', GH.RapidBoard.DetailsView.handleOpenIssueDetailsLink);

    // hiding issue details
    AJS.$(document).delegate('#ghx-detail .js-detailclose','click', GH.RapidBoard.DetailsView.handleCloseDetailsViewLink);
    
    // editing of fields
    AJS.$(document).delegate('#ghx-detail .js-editable-field', 'mousedown', GH.RapidBoard.DetailsView.handleDetailsFieldClick);

    // load more comments
    AJS.$(document).delegate('#js-commentsMore','click', GH.RapidBoard.DetailsView.handleCommentsMoreClick);

    // load all comments
    AJS.$(document).delegate('#js-commentsAll','click', GH.RapidBoard.DetailsView.handleCommentsAllClick);
    
    // updating of the details view
    AJS.$(document).bind('issueSelected updateDetailsView', GH.RapidBoard.DetailsView.loadIssueDetails);
};
