/**
 * Component taking care of the "release" feature
 */
GH.RapidBoard.ReleaseView = {};

/** the currently active release dialog */
GH.RapidBoard.ReleaseView.dialog = null;

/**
 * Add the control to the page that triggers the release of the current done column
 */
GH.RapidBoard.ReleaseView.renderControl = function() {
    AJS.$('.ghx-column-headers .ghx-column:last').append(GH.tpl.rapid.release.renderReleaseControl());
    AJS.Dropdown.create({
        trigger: AJS.$(".ghx-column-headers .aui-dd-link"),
        content: AJS.$(".ghx-column-headers .aui-list"),
        alignment: AJS.RIGHT
    });
    AJS.$('.ghx-column-headers').find('.js-release').click(GH.RapidBoard.ReleaseView.showDialog);
};

GH.RapidBoard.ReleaseView.showDialog = function() {
    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/release/model.json'),
        dataType: 'json',
        data : { rapidViewId : GH.RapidBoard.State.getRapidViewId() },
        success: GH.RapidBoard.ReleaseView.renderDialog
    });
};

/**
 * Renders the "release" dialog where the details of the new version can be specified
 * 
 * @param model : see ReleaseViewModel
 */
GH.RapidBoard.ReleaseView.renderDialog = function(model) {
    _.each(model.projectIssueDetails, function(project) {
        project.name = soy.$$escapeHtml(project.name);
    });
    model.rapidViewId = GH.RapidBoard.State.getRapidViewId();
    
    var dlg = GH.RapidBoard.ReleaseView.dialog = new AJS.Dialog({
        width : 575,
        height : 350,
        id : 'ghx-dialog-release'
    });
    AJS.$('#'+dlg.id).addClass('ghx-dialog');
    dlg.addHeader(AJS.I18n.getText('gh.rapid.release.dialog.title'));
    dlg.addPanel('', 'panel');
    dlg.getCurrentPanel().html(GH.tpl.rapid.release.renderReleaseDialog({ model : model }));
    dlg.addSubmit(AJS.I18n.getText('gh.rapid.release.dialog.ok'), function() {
        GH.RapidBoard.ReleaseView.release();
    });        
    dlg.addCancel(AJS.I18n.getText('common.forms.cancel'), function() {
        // analytics tracking for clicking cancel
        AJS.$(AJS).trigger("gh.release.cancel", {
            count: model.projectIssueDetails.length
        });
        dlg.remove();
    });
    
    AJS.$('.aui-dialog').addClass('gh-dialog');
    
    // initialise DatePicker
    Calendar.setup({
        firstDay : 0,
        inputField : 'ghx-release-date',
        button : 'ghx-release-date_trigger_c',
        align : 'Tl',
        singleClick : true,
        positionStyle : 'fixed',
        ifFormat : model.datePickerFormat
    });

    // analytics tracking for clicking release
    AJS.$(AJS).trigger("gh.release.start");
    
    // hook up issue nav links
//    AJS.$('#ghx-release-navigator a').click(GH.RapidBoard.ReleaseView.prepareIssueNavigator);
    
    dlg.show();
};

/**
 * Release! (create versions in all projects, add issues, release versions, perform miracle).
 */
GH.RapidBoard.ReleaseView.release = function() {

    // analytics tracking for release
    AJS.$(AJS).trigger("gh.release.complete");

    var rapidViewId = GH.RapidBoard.State.getRapidViewId();

    GH.Ajax.post({
        url: GH.Ajax.buildRestUrl('/release'),
        data: {
            versionName : AJS.$('#ghx-release-name').val(),
            releaseDate : AJS.$('#ghx-release-date').val(),
            description : AJS.$('#ghx-release-desc').val(),
            rapidViewId : rapidViewId
        },
        success : function(result) {
            // no errors! first, remove dialog...
            GH.RapidBoard.ReleaseView.dialog.remove();
            
            // .. then show success message
            GH.Message.showSuccess(result.success.message);
            
            // and reload the whole thing
            GH.RapidBoard.reload();
        },
        errorContextMap : {
            name : '#ghx-release-name',
            releaseDate : '#ghx-release-date'
        },
        // hide the dialog if there were global errors, because that means
        // the whole thing is not in the right state and someone's got to get some
        // issues into done first
        onGlobalError : function() {
            if (GH.RapidBoard.ReleaseView.dialog) {
                GH.RapidBoard.ReleaseView.dialog.remove();
            }
        }
    });
};

//GH.RapidBoard.ReleaseView.prepareIssueNavigator = function(event) {
//    var element = AJS.$(event.target);
//    var projectId = element.attr('data-projectid');
//    
//    GH.Ajax.get({
//        url: GH.Ajax.buildRestUrl('/release/navigator'),
//        async : false,  // we want this to happen before issue nav is called, because JIRA expects things to be in the session.
//        data: {
//            projectId : projectId,
//            rapidViewId : GH.RapidBoard.State.getRapidViewId()
//        },
//        onGlobalError: function() {
//            event.preventDefault();
//        }
//    });
//};

/**
 * Add the "render" control when the model changes
 */
GH.RapidBoard.ReleaseView.init = function() {
    AJS.$(document).bind('modelChanged hideDetailsView hideChartView', GH.RapidBoard.ReleaseView.renderControl);
};
