/**
 * This library is a wrapper for flot (see http://people.iola.dk/olau/flot/API.txt) for API doc. 
 * It allows us to centralise utility functions and add default options to all charts. 
 */
GH.Chart = {};

/**
 * Default chart options. Specific options will override these.
 */
GH.Chart.defaultOptions = {
    colors: ["#9eb6d4", "#afd8f8", "#cb4b4b", "#4da74d", "#9440ed"],
    grid: {
        backgroundColor:"#fff",
        borderColor:"#bbb",
        borderWidth: 1,
        internalLineWidth: 0,
        labelMargin: 2,
        minBorderMargin: 1
    },
    legend: {
        container: "#ghx-chart-legend",
        noColumns: 1 // number of columns in legend table
    },
    shadowSize: 0,
    xaxis: {
        tickLength: 4 // sets the little marks next to the units on the x axis, a value here also stops horizontal lines across chart
    },
    yaxis: {
        tickLength: 4 // sets the little marks next to the units on the y axis, a value here also stops horizontal lines across chart
    }

};

/**
 * Draw a chart using flot
 * 
 * @param container : jQuery object or selector of the element where the chart will be drawn
 * @param series : flot data series
 * @param options : chart options (optional). Override default options.
 * 
 * @returns the created flot chart object, for future reference 
 */
GH.Chart.draw = function(container, series, options) {
    container = AJS.$(container);
    
    // clone the global defaults into a local variable (we might have more than one chart in a page)
    var effectiveOptions = AJS.$.extend(true, {}, GH.Chart.defaultOptions);
    if (options) {
        // override the defaults with the given options
        AJS.$.extend(true, effectiveOptions, options);
    }

    return AJS.$.plot(container, series, effectiveOptions);
};
