GH.ChartTimeFrames = {};

/**
 * # of milliseconds in a day
 */
GH.ChartTimeFrames.day = 1000 * 60 * 60 * 24;

// TODO: require date format
GH.ChartTimeFrames.dateFormat = "%d.%m.%Y";

/**
 * Stores the current values
 */
GH.ChartTimeFrames.from = null;
GH.ChartTimeFrames.to = null;
GH.ChartTimeFrames.days = 7; // 7 days by default
GH.ChartTimeFrames.now = new Date().getTime();
GH.ChartTimeFrames.isRelative = true;
GH.ChartTimeFrames.minXValue = null;

/**
 * Convert milliseconds to days
 */
GH.ChartTimeFrames.millisToDays = function(duration) {
    if (duration) {
        return duration / GH.ChartTimeFrames.day;
    } else {
        return duration;
    }
};

/**
 * Convert days to milliseconds
 */
GH.ChartTimeFrames.daysToMillis = function(days) {
    if (days) {
        return days * GH.ChartTimeFrames.day;
    } else {
        return days;
    }
};

/**
 * Predefined time frames. All have today defined as end date
 */
GH.ChartTimeFrames.timeFrames = [
    {
    	disabled: true,
    	relative: true,
        days: 7,
        name: AJS.I18n.getText('gh.rapid.timeframes.1w')
    },
    {
    	disabled: true,
    	relative: true,
        days: 14,
        name: AJS.I18n.getText('gh.rapid.timeframes.2w')
    },
    {
    	disabled: true,
    	relative: true,
        days: 30,
        name: AJS.I18n.getText('gh.rapid.timeframes.1m')
    },
    {
    	disabled: true,
    	relative: true,
        days: 90,
        name: AJS.I18n.getText('gh.rapid.timeframes.3m')
    },
    {
    	disabled: true,
    	relative: true,
        days: 180,
        name: AJS.I18n.getText('gh.rapid.timeframes.6m')
    },
    {
    	disabled: false,
    	relative: true,
        days: 0,
        name: AJS.I18n.getText('gh.rapid.timeframes.inf')
    },
    {
    	disabled: false,
    	relative: false,
    	days: -1,
        name: AJS.I18n.getText('gh.rapid.timeframes.custom')
    }
];

GH.ChartTimeFrames.getSelectedTimeFrame = function() {
	for(var i=0; i < GH.ChartTimeFrames.timeFrames.length; i++) {
		if (!GH.ChartTimeFrames.isRelative && !GH.ChartTimeFrames.timeFrames[i].relative) {
			return GH.ChartTimeFrames.timeFrames[i];
		}
		else if (GH.ChartTimeFrames.timeFrames[i].days == GH.ChartTimeFrames.days) {
			return GH.ChartTimeFrames.timeFrames[i];
		} 
	}
	return false;
};

/**
 * Get the start date of the chart
 */
GH.ChartTimeFrames.getChartStartDate = function() {
	var day = GH.ChartTimeFrames.day;
	if(GH.ChartTimeFrames.isRelative) {
		// infinite
		if (GH.ChartTimeFrames.days == 0) {
			var min = GH.ChartTimeFrames.minXValue;
			var nowLessOne = GH.ChartTimeFrames.now - 1*day; // backtrack by 1 day
			if (!min || min > nowLessOne) min = nowLessOne;
			return min;
			// ensure we show at least 1 day
			//return GH.ChartTimeFrames.minXValue ? GH.ChartTimeFrames.minXValue : GH.ChartTimeFrames.now - 7*day; // return minXValue or 7 days  
		}
		// # of days
		else {
			// return now - number of days
			return GH.ChartTimeFrames.now - GH.ChartTimeFrames.days*day; 
		}   
	} else {
		// return start date
		return GH.ChartTimeFrames.from;
	}
};

/**
 * Get the end date of the chart
 */
GH.ChartTimeFrames.getChartEndDate = function() {
	var day = GH.ChartTimeFrames.day;
	if(GH.ChartTimeFrames.isRelative) {
		return GH.ChartTimeFrames.now;
	} else {
		return GH.ChartTimeFrames.to;
	}
};

/**
 * Initializes the chart time frames component
 */
GH.ChartTimeFrames.init = function() {
	// hacks to handle closing of the layer. We need to attach a close evaluation function that does NOT close if we either click on the dialog
	// or on one of the calendars. 
	AJS.$(document).bind("showLayer", function (e, type, inlineLayer) {
	    if (type === "inlineDialog" && AJS.InlineDialog.current.id === "chartTimeFramesInlineDialog") {
	    	// handle InlineLayer out of zone click handler
	    	inlineLayer._validateClickToClose = function(e) {
	    		if(AJS.$(e.target).parents('.calendar').length > 0) return false;
	    		if(AJS.$(e.target).parents('#inline-dialog-chartTimeFramesInlineDialog').length > 0) return false;
	    		return true;
	    	};
	    	// handle InlineDialog out of zone click handler - inline layer is good enough plus this guy doesn't allow us any overwriting.
	    	// --> Kill
	    	AJS.$('body').unbind('click.chartTimeFramesInlineDialog.inline-dialog-check');
            // Fix Wrapped Button Group
            setTimeout(function(){
                GH.RapidBoard.ChartView.fixButtons('#inline-dialog-'+AJS.InlineDialog.current.id);
            },0);
	    }
	});
};

/**
 * Called before the ajax request to load a chart
 */
GH.ChartTimeFrames.update = function(chartId) {
	// load the configuration from the state
	var days = GH.RapidBoard.State.getBoardSetting(null, 'chart.timeframe.days', 7); // fall back to 7 days if not defined
	var from = new Date().getTime();
	var from = GH.RapidBoard.State.getBoardSetting(null, 'chart.timeframe.from', from);
	var to = new Date().getTime() - 7*GH.ChartTimeFrames.day;
	var to = GH.RapidBoard.State.getBoardSetting(null, 'chart.timeframe.to', to);
	
	GH.ChartTimeFrames.setNewChartLimits(days, from, to, false);
};

/**
 * Called once the chart data has been loaded
 */
GH.ChartTimeFrames.setBoundries = function(minXValue, currentTime) {
	// set the internal values
	GH.ChartTimeFrames.now = currentTime;
	GH.ChartTimeFrames.minXValue = minXValue || currentTime-7*GH.ChartTimeFrames.day; // use currentTime if we got no minXValue
	
	// TODO: load persisted values
	
	// update the filters disabled state
	GH.ChartTimeFrames.disableOutOfRangeButtons();
	
	// update the display
	GH.ChartTimeFrames.updateDatesInfo();
};

/**
 * Disables buttons that are out of range, e.g. 1 year if we only got data for 1 month.
 * All filters that are inside the date range are enabled, PLUS the first one that is outside the range.
 * Thus, if we got 3 weeks worth of data, also the 1 month button will be enabled.
 */
GH.ChartTimeFrames.disableOutOfRangeButtons = function() {
	// enable all
	_.each(GH.ChartTimeFrames.timeFrames, function(element, index) {
		element.disabled = false;
	});
	/*
	// calculate the maximum number of days
	var maxDays = GH.ChartTimeFrames.millisToDays(GH.ChartTimeFrames.now - GH.ChartTimeFrames.minXValue);
	
	// enable the first Greater button, but only if we have data. Otherwise simply show the "all" button"
	// only enable buttons that are inside the complete data span
	var firstGreater = false; //maxDays > 0;
	
	// enable/disable according to maxDays and firstGreater
	_.each(GH.ChartTimeFrames.timeFrames, function(element, index) {
		if (element.relative) {
			if (element.days > maxDays) {
				// outside the range, still enable if this is the first one outside
				if (firstGreater) {
					element.disabled = false;
					firstGreater = false;
				} else {
					element.disabled = true;
				}
			}
			else {
				element.disabled = false;
			}
		}
	});
	*/
};

/**
 * Updates the from and to dates depending on the current selection
 */
GH.ChartTimeFrames.updateDialogFromAndToDates = function(contents) {
	// figure out which days is currently selected
	var activeButton = contents.find('.js-timeframe-button.active');
	var activeDays = activeButton.attr('data-timeframe-id');
	
	// nothing to do if the custom option is selected
	if (activeDays == -1) return;
	
	// calculate from and to dates
	var to = GH.ChartTimeFrames.now;
	if (activeDays > 0) {
		var from = to - activeDays*GH.ChartTimeFrames.day; 
	} else {
		// infinitive, use min value
		var from = GH.ChartTimeFrames.minXValue;
	}
	
	// render the strings
	var fromDateString = new Date(from).print(GH.ChartTimeFrames.dateFormat);
	var toDateString = new Date(to).print(GH.ChartTimeFrames.dateFormat);
	
	// update the fields
	contents.find('#ghx-field-from').val(fromDateString);
	contents.find('#ghx-field-to').val(toDateString);
};

/**
 * Updates the displayed dates information 
 */
GH.ChartTimeFrames.updateDatesInfo = function() {
	// update from and to according to the days if we are relative. otherwise from and to are already set
	var selectedTimeFrame = GH.ChartTimeFrames.getSelectedTimeFrame();
	if (selectedTimeFrame.relative) {
		GH.ChartTimeFrames.to = GH.ChartTimeFrames.now;
		if (selectedTimeFrame.days > 0) {
			GH.ChartTimeFrames.from = GH.ChartTimeFrames.to - selectedTimeFrame.days * GH.ChartTimeFrames.day; 
		} else {
			// infinitive, use min value
			GH.ChartTimeFrames.from = GH.ChartTimeFrames.minXValue;
		}
	}
	
	// format the dates
	var fromDateString = new Date(GH.ChartTimeFrames.from).print(GH.ChartTimeFrames.dateFormat);
	var toDateString = new Date(GH.ChartTimeFrames.to).print(GH.ChartTimeFrames.dateFormat);
		
	var display = AJS.I18n.getText('gh.rapid.timeframes.label', fromDateString, toDateString, selectedTimeFrame.name);
	AJS.$('.js-timeframe-info').text(display);
};

/**
 * Set the new chart limits and updates the chart
 */
GH.ChartTimeFrames.setNewChartLimits = function(activeDays, from, to, refreshChart) {
	if (activeDays < 0) { // absolute		
		GH.ChartTimeFrames.from = from;
		GH.ChartTimeFrames.to = to;
		GH.ChartTimeFrames.isRelative = false;
		GH.ChartTimeFrames.days = -1;
	} else { // relative
		GH.ChartTimeFrames.isRelative = true;
		GH.ChartTimeFrames.days = activeDays;
	}
	
	// update the state variables
	GH.RapidBoard.State.setBoardSetting('chart.timeframe.days', GH.ChartTimeFrames.days);
	GH.RapidBoard.State.setBoardSetting('chart.timeframe.from', from);
	GH.RapidBoard.State.setBoardSetting('chart.timeframe.to', to);
	
	// update the date info
	GH.ChartTimeFrames.updateDatesInfo();
	
	// refresh the chart, no need to reload data
	if (refreshChart) {
		GH.RapidBoard.ChartView.refreshChart();
	}
};

/**
 * Renders the reporting timeframes dialog
 */
GH.ChartTimeFrames.renderChartTimeFramesDialog = function(contents, trigger, showPopup) {
	// render the from and to dates. These will be empty if a relative time is selected
	var fromDateString = GH.ChartTimeFrames.from ? new Date(GH.ChartTimeFrames.from).print(GH.ChartTimeFrames.dateFormat): '';
	var toDateString = GH.ChartTimeFrames.to ? new Date(GH.ChartTimeFrames.to).print(GH.ChartTimeFrames.dateFormat) : '';
	
	// render the content. this is rerendered every time the dialog opens so we don't have to worry about updating the html
	contents.html(GH.tpl.rapid.chart.renderChartTimeFrames({
		'timeFrames': GH.ChartTimeFrames.timeFrames,
		'relative': GH.ChartTimeFrames.isRelative,
		'days': GH.ChartTimeFrames.days,
		'from': fromDateString,
		'to': toDateString
	}));
	
    // mark this for GH styling	
	contents.closest('.aui-inline-dialog').addClass('ghx-inline-dialog');
	
	// update the date fields, necessary for relative options
	GH.ChartTimeFrames.updateDialogFromAndToDates(contents);
	
	// handler called to select one of the buttons
	var selectButton = function(button) {
		// skip if button is disabled
		if(button.hasClass('disabled')) return;
		
		// select/unselect others
		contents.find('.js-timeframe-button').removeClass('active');
		button.addClass('active');
		
		// update the from and to dates
		GH.ChartTimeFrames.updateDialogFromAndToDates(contents);
	};
	
	// handle clicks on the different buttons
	contents.find('.js-timeframe-button').click(function(event) {
		selectButton(AJS.$(this));
	});
	
    Calendar.setup({
        firstDay : 0,
        inputField : 'ghx-field-from',
        button : 'ghx-field-from-button',
        align : 'Tl',
        singleClick : true,
        cache: true,
        positionStyle : 'fixed',
        ifFormat : GH.ChartTimeFrames.dateFormat,
        onUpdate: function() {
        	selectButton(contents.find('.js-timeframe-button[data-timeframe-id="-1"]'));
        }
    });
    Calendar.setup({
        firstDay : 0,
        inputField : 'ghx-field-to',
        button : 'ghx-field-to-button',
        align : 'Tl',
        singleClick : true,
        cache: true,
        positionStyle : 'fixed',
        ifFormat : GH.ChartTimeFrames.dateFormat,
        onUpdate: function() {
        	selectButton(contents.find('.js-timeframe-button[data-timeframe-id="-1"]'));
        }
    });
	
	// handle click on update and cancel
	contents.find('.js-save-button').click(function(e) {
		e.preventDefault();
		GH.ChartTimeFrames.handleDialogSave(contents);
	});
	contents.find('.js-cancel-button').click(function(e) {
		AJS.InlineDialog.current.hide();
	});
	
	
	
	// InlineDialog hack 
	/*
	contents.click(function(event) {
		// makes sure the dialog is not closed when the user clicks away
		event.stopPropagation();
	});
	*/
	
	// now display the dialog
	showPopup();
};

/**
 * Called to save the dialog data
 */
GH.ChartTimeFrames.handleDialogSave = function(contents) {
	// update the columns
	var activeButton = contents.find('.js-timeframe-button.active');
	var activeDays = parseInt(activeButton.attr('data-timeframe-id'), 10);
	var from = null, to=null;
	
	// remove all errors
	contents.find('.ghx-error').remove();
	
	// fetch from and to in case of custom dates
	if (activeDays < 0) {
		// set from and to
		// load the from and to dates
		var format = Calendar._TT["DEF_DATE_FORMAT"];
		var fromDate = contents.find('#ghx-field-from').val();
		if (fromDate == '') {
            contents.find('#ghx-field-from').closest('.field-group').append('<div class="ghx-error">'+AJS.I18n.getText('gh.rapid.timeframes.error.from')+'</div>');
			return;
		}
		fromDate = Date.parseDate(fromDate, GH.ChartTimeFrames.dateFormat);
		var toDate = contents.find('#ghx-field-to').val();
		if (toDate == '') {
            contents.find('#ghx-field-to').closest('.field-group').append('<div class="ghx-error">'+AJS.I18n.getText('gh.rapid.timeframes.error.to')+'</div>');
			return;
		}
		toDate = Date.parseDate(toDate, GH.ChartTimeFrames.dateFormat); 
		if (fromDate.getTime() > toDate.getTime()) {
            contents.find('#ghx-field-to').closest('.field-group').append('<div class="ghx-error">'+AJS.I18n.getText('gh.rapid.timeframes.error.reverse')+'</div>');
			return;
		}
		from = fromDate.getTime();
		to = toDate.getTime();
	}
	
	// set the new limits
	GH.ChartTimeFrames.setNewChartLimits(activeDays, from, to, true);
	GH.RapidBoard.State.pushState();
	
	// hide dialog
	if (AJS.InlineDialog.current) AJS.InlineDialog.current.hide();
};

/**
 * Initializes the time frames.
 * @param the minimum values in the data set
 * @param currentTime the time defined as "now" 
 */
/*
GH.ChartTimeFrames.bla = function(xMin, currentTime) {
	
    var selectedIsDisabled = false;
    AJS.$(".ghx-timeframe-button").removeClass("disabled").each(function(){
        var $this = AJS.$(this);
        if(($this.attr("data-timeframe-id") * GH.ChartTimeFrames.msPerDay) + absMin > currentTime) {
            $this.addClass("disabled");
            if($this.hasClass("active")){
                $this.removeClass("active");
                selectedIsDisabled = true;
            }

        }
    });
    if(selectedIsDisabled) {
        AJS.$(".ghx-timeframe-button[data-timeframe-id=0]").addClass("active");
    }
    AJS.$(document).delegate('.ghx-timeframe-button', 'click', GH.ChartTimeFrames.setTimeFrame);
};


GH.ChartTimeFrames.getXAxisLimits = function (days, numDays) {
    var l = days.length;
    var xaxis = {};

    // NOTE: I removed these offsets as they didn't seem nessecary.
    // Old comment left for posterity:
    // offset the max and min a little to prevent the tick labels from going over the edge. Magic number by trial and error.

    // max should either be highest value in days, or null.
    xaxis.max = l > 0 ? days[l - 1]: null;

    // if we have a numDays, then min should be numDays < max
    if (numDays != 0) {
        xaxis.min = xaxis.max != null ? xaxis.max - (numDays * GH.ChartTimeFrames.msPerDay) : null;
    }

    // min should fail over to the first entry in days if we have null or something less
    // than the first entry (and if such an entry exists)
    if(l > 0 && (xaxis.min == null || xaxis.min < days[0])) {
        xaxis.min = days[0];
    }

    return xaxis;
};

GH.ChartTimeFrames.setTimeFrame = function () {
    var $this = AJS.$(this), timeFrameId = parseInt($this.attr("data-timeframe-id"));
    if($this.hasClass("disabled") || $this.hasClass("active")) {
        return;
    }

    //Make this the active button
    AJS.$(".ghx-timeframe-button").removeClass("active");
    $this.addClass("active");

    GH.RapidBoard.State.setTimeFrame(timeFrameId);

    AJS.$(GH.RapidBoard.ChartView).trigger("timeFrameChanged");

    GH.RapidBoard.State.pushState();
};
*/