/**
 * Chart view component.
 *
 * Loads chart data, then passes contorl to chart implementation for actual rendering
 */
GH.RapidBoard.ChartView = {};

GH.RapidBoard.ChartView.charts = [
                                  {
                                      id: 'controlChart',
                                      name: AJS.I18n.getText('gh.rapid.charts.control.name'),
                                      description: AJS.I18n.getText('gh.rapid.charts.control.description'),
                                      preview: 'control-chart'
                                  },
                                  {
                                      id: 'cumulativeFlowDiagram',
                                      name: AJS.I18n.getText('gh.rapid.charts.cfd.name'),
                                      description: AJS.I18n.getText('gh.rapid.charts.cfd.description'),
                                      preview: 'cumulative-flow-diagram'
                                  }
                              ];

/**
 * Get the available charts
 */
GH.RapidBoard.ChartView.getCharts = function() {
	return GH.RapidBoard.ChartView.charts;
};

/**
 * Get the chart configuration for a given chart id.
 */
/*
GH.RapidBoard.ChartView.getChartConfig = function(chartId) {
	var charts = GH.RapidBoard.ChartView.charts;
	for(var i=0; i < charts.length; i++) {
		if(charts[i].id == chartId) return charts[i];
	}
	return false;
};
*/

/**
 * Get the container the chart view is rendered in
 */
GH.RapidBoard.ChartView.getContainer = function() {
	return AJS.$('#ghx-display');
};

GH.RapidBoard.ChartView.getChartStatus = function() {
	return AJS.$('#ghx-chart-status');
};

GH.RapidBoard.ChartView.getChartGroup = function() {
	return AJS.$('#ghx-chart-group');
};


/**
 * Initializes the chart view
 */
GH.RapidBoard.ChartView.init = function() {
	AJS.$(document).bind('loadChart', GH.RapidBoard.ChartView.loadChart);
	
	// initialize helpers
	GH.ChartTimeFrames.init();
    GH.ChartFilters.init();
};

/**
 * Get the currently selected chart type
 */
GH.RapidBoard.ChartView.getSelectedChart = function() {
	return GH.RapidBoard.State.getChartType();
};

/**
 * Is the chart view currently visible
 */
GH.RapidBoard.ChartView.isChartViewVisible = function() {
	return AJS.$('#ghx-chart-nav:visible').length > 0; // TODO: "know the information without checking the dom
};

/**
 * Loads the chart. This function is invoked by the 'loadChart' event
 */
GH.RapidBoard.ChartView.loadChart = function() {
	// check whether we should show the intro
    if(GH.ChartIntro.shouldShowIntro()) {
    	GH.ChartIntro.openIntro();
        return;
    }

    // render the chart view
    GH.RapidBoard.ChartView.renderChartView();
};

/**
 * Loads the chart data for a given chart type
 */
GH.RapidBoard.ChartView.renderChartView = function() {
	// chart structure
    AJS.$('#ghx-display').addClass('ghx-chart').html(GH.tpl.rapid.chart.renderChart());

    // chart menu
    AJS.$('#ghx-chart-nav').html(AJS.$(GH.tpl.rapid.chart.renderChartMenu({
        activeChart: GH.RapidBoard.ChartView.getSelectedChart(),
        availableCharts: GH.RapidBoard.ChartView.getCharts()
    })));
    
    // time frames
    AJS.$('#ghx-chart-timeframe').append(GH.tpl.rapid.chart.renderChartTimeFrame());
    
    // chart filters
    AJS.$('#ghx-chart-filter').append(GH.tpl.rapid.chart.renderChartFilter());

    // initialize all elements
    GH.RapidBoard.ChartView.initializeElements();

    // then load the chart
    GH.RapidBoard.ChartView.loadChartData();
};

/**
 * Renders the dropdown menu for the Chart Menu items & attach events
 */
GH.RapidBoard.ChartView.initializeElements = function() {
	// kill any previous inline dialogs
	AJS.$('#inline-dialog-chartTimeFramesInlineDialog,#inline-dialog-chartFiltersInlineDialog').remove();
	
    // initialize inline dialogs
    var options = {
    	width: 500,
    	cacheContent : false, // don't cache the dialog content
    	hideDelay : 60000 // set longer timeout (default is 10 seconds)
    };
    var optionsTimeFrames = {
    	width: 700,
    	cacheContent : false, // don't cache the dialog content
    	hideDelay : 60000 // set longer timeout (default is 10 seconds)
    };
    AJS.InlineDialog("#js-chart-filter", "chartFiltersInlineDialog", GH.ChartFilters.renderChartFiltersDialog, options);
    AJS.InlineDialog("#js-chart-timeframe", "chartTimeFramesInlineDialog", GH.ChartTimeFrames.renderChartTimeFramesDialog, optionsTimeFrames);
    
    // initialize the charts dropdown
    var chartDropdowns = AJS.Dropdown.create({
        trigger: AJS.$("#ghx-chart-nav .ghx-chart-menu .aui-dd-link"),
        content: AJS.$("#ghx-chart-nav .ghx-chart-options.aui-list"),
        alignment: AJS.LEFT
    });

    // bind a change event to the select
    AJS.$('.ghx-chart-options .aui-list-item').bind('click', function(event) {
    	GH.RapidBoard.ChartView.handleChartDropdownSelection(event, AJS.$(this), chartDropdowns);
    });
};

/**
 * Fixes Button Groups Which Wrap to Multiple Lines
 */
GH.RapidBoard.ChartView.fixButtons = function(el) {

    var bars = AJS.$(el).find('ul.ops');
    var buts;
    var barPos;
    var barX;
    var butPos;
    var butX;
    var butEl;

    bars.each(function(){

        barPos = AJS.$(this).position();
        barX = barPos.left;

        if(AJS.$(this).hasClass('ghx-slider')){
            // slider buttons
            buts = AJS.$(this).find('li');
            buts.each(function(){

                butPos = AJS.$(this).position();
                butX = butPos.left;

                if(butX === barX){
                    AJS.$(this).addClass('first').find('a').addClass('first');
                    AJS.$(this).prev('li').addClass('last').find('a').addClass('last');

                }
            });

        } else {
            // normal ops buttons
            buts = AJS.$(this).find('.button');
            buts.each(function(){

                butPos = AJS.$(this).position();
                butX = butPos.left;

                if(butX === barX){
                    AJS.$(this).addClass('first');
                    AJS.$(this).prev('.button').addClass('last');
                }
            });
        }
    });
};

/**
 * Handles the chart type dropdown selection 
 */
GH.RapidBoard.ChartView.handleChartDropdownSelection = function(event, item, chartDropdowns) {
    // check which chart got selected
    var dataChart = item.find('a').attr('data-chart');

    // if we've hit the intro button, then go to the intro...
    if(dataChart == "chart-intro") {
    	GH.ChartIntro.openIntro();
        return;
    }

    // hide the dropdown
    AJS.$(chartDropdowns).each(function(index, dropdown) {
        dropdown.hide();
    });
    
    // change the chart. TODO: this should directly set the chart and invoke loadChart
    GH.RapidBoard.ChartView.openChartView(dataChart);
    GH.RapidBoard.State.pushState();
};



/**
 * Loads the chart.
 * This function updates the tools, initializes the spinner, then sends off a request to load the data
 */
GH.RapidBoard.ChartView.loadChartData = function() {
	// fetch the current chart type
	var chartType = GH.RapidBoard.ChartView.getSelectedChart();
	
	// update the tools
	GH.RapidBoard.ChartView.updateTools(chartType);	
	
	// show the spinner
	GH.RapidBoard.ChartView.getChartGroup().hide();
	GH.RapidBoard.ChartView.getChartStatus().empty().append('<span class="gh-spinner">').show();
	
    // Put together the ajax request, including the configuration from the filters
	var data = {
		'rapidViewId' : GH.RapidBoard.State.getRapidViewId()
	};
	GH.ChartFilters.addRequestParams(data);
	
	// then issue the request
    GH.Ajax.get({
        url: GH.Ajax.buildRestUrl('/rapid/charts/' + chartType.toLowerCase() + '.json'),
        data: data,
        dataType: 'json',
        success: function(data) {
        	// ignore if we are not visible anymore
        	if (! GH.RapidBoard.ChartView.isChartViewVisible()) return;
        	// process and display the data
        	GH.RapidBoard.ChartView.processChartData(chartType, data);
        }
    }, 'rapidChartData');
};

/**
 * Refreshes the tools according to the selected chart type.
 * The tools are the ones that provided ajax request parameters, it is therefore important 
 * that their state is properly set before loading a chart
 */
GH.RapidBoard.ChartView.updateTools = function(chartType) {
	// update the display of the time frame
    GH.ChartTimeFrames.update(chartType);
    
    // initialize the chart filters
    GH.ChartFilters.update(chartType);
    
    // push the state
    GH.RapidBoard.State.pushState();
};

/**
 * Processes the returned chart data.
 */
GH.RapidBoard.ChartView.processChartData = function(chartType, data) {
	// ensure we are still on the given chart, discard the data otherwise
	var currentChartType = GH.RapidBoard.ChartView.getSelectedChart();
	if (currentChartType != chartType) return;

	// render the data
    switch(chartType) {
    case 'controlChart':
        // render the details
        GH.RapidBoard.ChartView.ControlChart.showChart(data);
        break;
    case 'cumulativeFlowDiagram':
        GH.RapidBoard.ChartView.CFD.showChart(data);
        break;
    }
};

/**
 * Refreshes the current chart to pick up any configuration changes that don't require
 * data reloading.
 * 
 * This is currently used to update the chart in case of time frame changes or changed columns
 */
GH.RapidBoard.ChartView.refreshChart = function() {
	var chartType = GH.RapidBoard.ChartView.getSelectedChart();
	// render the data
    switch(chartType) {
    case 'controlChart':
    	// render the chart
    	GH.RapidBoard.ChartView.ControlChart.calculateDataAndRenderChart();
        break;
    case 'cumulativeFlowDiagram':
    	GH.RapidBoard.ChartView.CFD.handleTimeFrameChanged();
        break;
    }
};

/**
 * Set the chart axis labels and properly align the y axis
 */
GH.RapidBoard.ChartView.setAndAlignAxisLabels = function(xAxisLabel, yAxisLabel) {
	// set the proper x and y axis labels
	var xAxis = AJS.$('#ghx-axis-x').text(xAxisLabel);
	var yAxis = AJS.$('#ghx-axis-y').text(yAxisLabel);
	
    // Placement for y-axis label
    var chartAxisPos = (yAxis.width() / 2 + 8);
    yAxis.css({
        left: -chartAxisPos
    });
};

/**
 * Opens the chart view.
 */
GH.RapidBoard.ChartView.openChartView = function(chartType) {
	// fall back to state provided chart if not defined
    chartType = chartType || GH.RapidBoard.State.getChartType();
	GH.RapidBoard.State.setChartType(chartType);
	GH.RapidBoard.State.setChartViewOpened(true);
	GH.RapidBoard.State.pushState();

    AJS.$(document).trigger('showChartView');
};

/**
 * Called to close the chart view.
 */
GH.RapidBoard.ChartView.closeChartView=function(event, dontPushState) {
	if (event) {
		event.preventDefault();
	}
	GH.RapidBoard.State.setChartViewOpened(false);
    if(!dontPushState) {
	    GH.RapidBoard.State.pushState();
    }
    AJS.$(document).trigger('hideChartView');
};
