AJS.test.require("com.pyxis.greenhopper.jira:gh-rapid");

var day = 24*60*60*1000

// no data values

test("Test min/max/mean with empty data", function() {
	var data = [];
	var resultObj = {};
	GH.ControlChart.Data.calculateMinMaxMean(data, resultObj);
	ok(resultObj.mean == 0, "Expected 0 as mean" );
	ok(resultObj.min == 0, "Expected 0 as min");
	ok(resultObj.max == 0, "Expected 0 as max");
});

test("Test devitation with empty data", function() {
	var data = [];
	var resultObj = {};
	GH.ControlChart.Data.calculateDeviation(data, 0, resultObj);
	ok(resultObj.deviation == 0, "Expected 0 as deviation");
});

test("Test aggregated means with empty data", function() {
	var data = [];
	var resultObj = {};
	GH.ControlChart.Data.calculateAggregatedMean(data, 0, 3*day, 1*day, 1*day, resultObj);
	ok(resultObj.aggregatedMean.length == 5, "Expected 5 aggregation points, received " + resultObj.aggregatedMean.length); // min, max, one for each day
	ok(resultObj.aggregatedMean[0][1] == 0, "Expected 0 as first aggregated mean");
});

// single data value

test("Test min/max/mean with single data value", function() {
	var data = [[0, 20]];
	var resultObj = {};
	GH.ControlChart.Data.calculateMinMaxMean(data, resultObj);
	ok(resultObj.mean == 20, "Expected 0 as mean" );
	ok(resultObj.min == 20, "Expected 0 as min");
	ok(resultObj.max == 20, "Expected 0 as max");
});

test("Test devitation with single data value", function() {
	var data = [[0, 20]];
	var resultObj = {};
	GH.ControlChart.Data.calculateDeviation(data, 20, resultObj);
	ok(resultObj.deviation == 0, "Expected 0 as deviation");
});

test("Test aggregated means with single data value", function() {
	var data = [[0, 20]];
	var resultObj = {};
	GH.ControlChart.Data.calculateAggregatedMean(data, 0, 3*day, 1*day, 1*day, resultObj);
	ok(resultObj.aggregatedMean.length == 5, "Expected 5 aggregation points");
	ok(resultObj.aggregatedMean[0][1] == 20, "Expected 20 as first aggregated mean");
	ok(resultObj.aggregatedMean[1][1] == 20, "Expected 20 as second aggregated mean");
	ok(resultObj.aggregatedMean[2][1] == 0, "Expected 0 as third aggregated mean, received " + resultObj.aggregatedMean[2][1]);
});

test("Test aggregated means with single data value and 2 for shift window", function() {
	var data = [[0, 20]];
	var resultObj = {};
	GH.ControlChart.Data.calculateAggregatedMean(data, 0, 3*day, 2*day, 1*day, resultObj); // 2 day window
	ok(resultObj.aggregatedMean.length == 4, "Expected 4 aggregation points, received " + resultObj.aggregatedMean.length);
	ok(resultObj.aggregatedMean[0][1] == 20, "Expected 20 as first aggregated mean");
	ok(resultObj.aggregatedMean[1][1] == 20, "Expected 20 as second aggregated mean");
	ok(resultObj.aggregatedMean[2][1] == 0, "Expected 0 as third aggregated mean, received " + resultObj.aggregatedMean[2][1]);
});

// 5 values

test("Test min/max/mean with 5 values", function() {
	var data = [[0, 20], [1*day, 10], [1.5*day, 5], [2*day, 30], [3.5*day, 20]];
	var resultObj = {};
	GH.ControlChart.Data.calculateMinMaxMean(data, resultObj);
	var mean = (20+10+5+30+20)/5;
	ok(resultObj.mean == mean, "Expected "+mean+" as mean, received " + resultObj.mean);
	ok(resultObj.min == 5, "Expected 5 as min, received " + resultObj.min);
	ok(resultObj.max == 30, "Expected 30 as max, received " + resultObj.max);
});

test("Test devitation with 5 values", function() {
	var data = [[0, 20], [1*day, 10], [1.5*day, 5], [2*day, 30], [3.5*day, 20]];
	var resultObj = {};
	
	// first fetch the mean (should have been successfully tested by previous test
	GH.ControlChart.Data.calculateMinMaxMean(data, resultObj);
	var mean = resultObj.mean;
	
	// then calculate deviation
	GH.ControlChart.Data.calculateDeviation(data, mean, resultObj);
	var deviation = Math.sqrt((Math.pow(mean-20,2)+Math.pow(mean-10,2)+Math.pow(mean-5,2)+Math.pow(mean-30,2)+Math.pow(mean-20,2))/4);
	ok(resultObj.deviation == deviation, "Expected "+deviation+" as deviation, received "+resultObj.deviation);
});

test("Test aggregated means 5 values", function() {
	var data = [[0, 20], [1*day, 10], [1.5*day, 5], [2*day, 30], [3.5*day, 20]];
	var resultObj = {};
	GH.ControlChart.Data.calculateAggregatedMean(data, 0, 7*day, 1*day, 1*day, resultObj);
	
	ok(resultObj.aggregatedMean.length == 9, "9 aggregated points expected, received " + resultObj.aggregatedMean.length);
	ok(resultObj.aggregatedMean[0][0] == 0, "x point of 0 mean expected at "+0+", received "+resultObj.aggregatedMean[0][0]);
	ok(resultObj.aggregatedMean[1][0] == 0.5*day, "x point of 1 mean expected at "+0.5*day+", received "+resultObj.aggregatedMean[1][0]);
	ok(resultObj.aggregatedMean[2][0] == 1.5*day, "x point of 2 mean expected at "+1.5*day+", received "+resultObj.aggregatedMean[2][0]);
	ok(resultObj.aggregatedMean[3][0] == 2.5*day, "x point of 3 mean expected at "+2.5*day+", received "+resultObj.aggregatedMean[3][0]);
	ok(resultObj.aggregatedMean[4][0] == 3.5*day, "x point of 4 mean expected at "+3.5*day+", received "+resultObj.aggregatedMean[4][0]);
	ok(resultObj.aggregatedMean[5][0] == 4.5*day, "x point of 5 mean expected at "+4.5*day+", received "+resultObj.aggregatedMean[5][0]);
	ok(resultObj.aggregatedMean[6][0] == 5.5*day, "x point of 6 mean expected at "+5.5*day+", received "+resultObj.aggregatedMean[6][0]);
	ok(resultObj.aggregatedMean[7][0] == 6.5*day, "x point of 7 mean expected at "+6.5*day+", received "+resultObj.aggregatedMean[7][0]);
	ok(resultObj.aggregatedMean[8][0] == 7*day, "x point of 8 mean expected at "+7*day+", received "+resultObj.aggregatedMean[8][0]);
	
	ok(resultObj.aggregatedMean[0][1] == 20, "y point of 0 mean expected at 20, received " + resultObj.aggregatedMean[0][1]);
	ok(resultObj.aggregatedMean[1][1] == 20, "y point of 1 mean expected at 20, received " + resultObj.aggregatedMean[1][1]);
	ok(resultObj.aggregatedMean[2][1] == 7.5, "y point of 2 mean expected at 7.5, received " + resultObj.aggregatedMean[2][1]);
	ok(resultObj.aggregatedMean[3][1] == 30, "y point of 3 mean expected at 30, received " + resultObj.aggregatedMean[3][1]);
	ok(resultObj.aggregatedMean[4][1] == 20, "y point of 4 mean expected at 20, received " + resultObj.aggregatedMean[4][1]);
	ok(resultObj.aggregatedMean[5][1] == 0, "y point of 5 mean expected at 0, received " + resultObj.aggregatedMean[5][1]);
	ok(resultObj.aggregatedMean[6][1] == 0, "y point of 6 mean expected at 0, received " + resultObj.aggregatedMean[6][1]);
	ok(resultObj.aggregatedMean[7][1] == 0, "y point of 7 mean expected at 0, received " + resultObj.aggregatedMean[7][1]);
	ok(resultObj.aggregatedMean[8][1] == 0, "y point of 8 mean expected at 0, received " + resultObj.aggregatedMean[8][1]);
});

test("Test aggregated mean with 2 day window and 1 day delta", function() {
	var data = [[0, 20], [1*day, 10], [1.5*day, 5], [2*day, 30], [3.5*day, 20]];
	var resultObj = {};
	
	// calculate over 7 days, but use 2 as window length
	GH.ControlChart.Data.calculateAggregatedMean(data, 0, 7*day, 2*day, 1*day, resultObj);
	
	ok(resultObj.aggregatedMean.length == 8, "8 aggregated points expected, received " + resultObj.aggregatedMean.length);

	ok(resultObj.aggregatedMean[0][0] == 0*day, "x point of 0 mean expected at "+0*day+", received "+resultObj.aggregatedMean[0][0]);
	ok(resultObj.aggregatedMean[1][0] == 1*day, "x point of 1 mean expected at "+1*day+", received "+resultObj.aggregatedMean[1][0]);
	ok(resultObj.aggregatedMean[2][0] == 2*day, "x point of 2 mean expected at "+2*day+", received "+resultObj.aggregatedMean[2][0]);
	ok(resultObj.aggregatedMean[3][0] == 3*day, "x point of 3 mean expected at "+3*day+", received "+resultObj.aggregatedMean[3][0]);
	ok(resultObj.aggregatedMean[4][0] == 4*day, "x point of 4 mean expected at "+4*day+", received "+resultObj.aggregatedMean[4][0]);
	ok(resultObj.aggregatedMean[5][0] == 5*day, "x point of 5 mean expected at "+5*day+", received "+resultObj.aggregatedMean[5][0]);
	ok(resultObj.aggregatedMean[6][0] == 6*day, "x point of 6 mean expected at "+6*day+", received "+resultObj.aggregatedMean[6][0]);
	ok(resultObj.aggregatedMean[7][0] == 7*day, "x point of 7 mean expected at "+7*day+", received "+resultObj.aggregatedMean[7][0]);
	
	ok(resultObj.aggregatedMean[0][1] == 20, "y point of 0 mean expected at " + 20 + ", received " + resultObj.aggregatedMean[0][1]); // 20
	ok(resultObj.aggregatedMean[1][1] == 35/3, "y point of 1 mean expected at " + 35/3 + ", received " + resultObj.aggregatedMean[1][1]); // 20, 10, 5
	ok(resultObj.aggregatedMean[2][1] == 45/3, "y point of 2 mean expected at " + 45/3 + ", received " + resultObj.aggregatedMean[2][1]); // 10, 5 and 30
	ok(resultObj.aggregatedMean[3][1] == 25, "y point of 3 mean expected at 25, received " + resultObj.aggregatedMean[3][1]); // 30 + 20
	ok(resultObj.aggregatedMean[4][1] == 20, "y point of 4 mean expected at 20, received " + resultObj.aggregatedMean[4][1]); // 20
	ok(resultObj.aggregatedMean[5][1] == 0, "y point of 5 mean expected at 0, received " + resultObj.aggregatedMean[5][1]);
	ok(resultObj.aggregatedMean[6][1] == 0, "y point of 6 mean expected at 0, received " + resultObj.aggregatedMean[6][1]);
	ok(resultObj.aggregatedMean[7][1] == 0, "y point of 6 mean expected at 0, received " + resultObj.aggregatedMean[7][1]);
});
