GH.RapidBoard.ChartView.CFD = {};

/** the colours for the columns. index in the array relates to column position (fist colour, first column). */
GH.RapidBoard.ChartView.COLUMN_COLORS = ["#FF800D","#23819C","#9669FE","#59955C","#B9264F","#B05F3C","#2966B8","#9A03FE","#FF2626","#FF5353","#C8B400","#79FC4E","#8C8CFF","#C48484","#4FBDDD","#FFBE28","#99C7FF","#AE70ED","#FFA8A8","#DAAF85"];

/** the context holding the data necessary to draw the current chart */
GH.RapidBoard.ChartView.CFD.context = null;

GH.RapidBoard.ChartView.CFD.showChart = function(model) {
	// build the series
    var series = GH.RapidBoard.ChartView.CFD.buildChartSeries(model);

    // the series are drawn last to first, since that's required for flot-stack to work. So we need to fetch the column colours
    // for the series and turn them around as well.
    var colors = GH.RapidBoard.ChartView.COLUMN_COLORS.slice(0, series.length).reverse();

    // set the chart boundries
    var now = new Date().getTime();
    var minXValue = model.days.length > 0 ? model.days[0] : null;
    GH.ChartTimeFrames.setBoundries(minXValue, now);

    // fetch the current limits for the chart
    var xMin = GH.ChartTimeFrames.getChartStartDate();
    var xMax = GH.ChartTimeFrames.getChartEndDate();

    var options = {
        colors: colors,
        xaxis: {
            mode : 'time',
            minTickSize : [1, 'day'],    // make sure we don't scale lower than a day, but will reduce the amount of ticks for long timespans
            min : xMin,
            max : xMax
        },
        yaxis: {
            minTickSize : 1,     // no half issues
            min: 0
        },
        series: {
            stack : true,
            lines : {
                fill : 1
            }
        },
        selection: { mode: "x" },
        legend: {
        	container : null,
        	backgroundOpacity : 0.5,
        	position : "nw"
        }
    };

    var overviewOptions = AJS.$.extend(true, {}, options);
    overviewOptions.legend = { show : false };
    overviewOptions.xaxis = { show : false };
    overviewOptions.yaxis = { show : false };
    
    GH.RapidBoard.ChartView.getChartStatus().hide();
    GH.RapidBoard.ChartView.getChartGroup().show();

    // set the chart labels
    GH.RapidBoard.ChartView.setAndAlignAxisLabels(AJS.I18n.getText('gh.rapid.chart.controlchart.days'), AJS.I18n.getText('gh.chart.issuecount'));

    // we have two views, the main (big) and overview (small, for selection)
    var chartContainer = AJS.$('#ghx-chart-view');
    var overviewGroup = AJS.$('#ghx-chart-overview-group');
    var overviewContainer = AJS.$('#ghx-chart-overview');
    var overviewHeader = AJS.$(overviewGroup).find('h4');
    var overviewDesc = AJS.$(overviewGroup).find('.ghx-description');

    // set the header and desc text
    AJS.$(overviewHeader).text(AJS.I18n.getText('gh.rapid.charts.cfd.overview.name'));
    AJS.$(overviewDesc).text(AJS.I18n.getText('gh.rapid.charts.cfd.overview.description'));

    // Show the required elements
    overviewHeader.show();
    overviewDesc.show();
    chartContainer.show();
    overviewGroup.show();

    // draw the main chart
    var ctx = GH.RapidBoard.ChartView.CFD.context = {
        chartContainer : chartContainer,
        series : series,
        options : options,
        model: model,
        absMin: model.days.length > 0 ? model.days[0] : null
    };
    chartContainer.bind("plotselected", GH.RapidBoard.ChartView.CFD.handleSelection);
    ctx.mainChart = GH.Chart.draw(chartContainer, series, options);

    // We reversed the data, so the legend now looks reversed too. Re-Reverse it again once rendered
    GH.RapidBoard.ChartView.CFD._reverseLegendTable();

    overviewContainer.bind("plotselected", GH.RapidBoard.ChartView.CFD.handleSelection);
    overviewContainer.bind("dblclick", GH.RapidBoard.ChartView.CFD.resetSelection);

    // draw the overview chart
    ctx.overviewChart = GH.Chart.draw(overviewContainer, series, overviewOptions);
    
    // TODO copied from ControlChart.js, there should be a better way of doing this
    // check whether we have to show the busted message
    var spill = false;
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    for (var i=0; i < swimlanes.length; i++) {
        spill = spill || swimlanes[i].maxIssuesExceeded;
    }
    if (spill) {
        var warningDiv = AJS.$(GH.tpl.rapid.chart.renderDataExceeded({}));
        AJS.$('#ghx-chart-data').append(warningDiv);
    }
};

GH.RapidBoard.ChartView.CFD._reverseLegendTable = function() {
    var legendTable = AJS.$('#ghx-chart-view').find('.legend').find('tbody');
    legendTable.children().each(function() {
    	legendTable.prepend(AJS.$(this));
    });
};

/**
 * Updates the charts with the new timeframe information.
 * Sets the preview label on the small chart and the absolute sc
 */
GH.RapidBoard.ChartView.CFD.updateTimeFrameSelection = function() {
	var xMin = GH.ChartTimeFrames.getChartStartDate();
    var xMax = GH.ChartTimeFrames.getChartEndDate(); 
	
    var ctx = GH.RapidBoard.ChartView.CFD.context;
    ctx.options.xaxis.min = xMin;
    ctx.options.xaxis.max = xMax;
    
    // select the overview chart, but only if it is not all time
    var selectedTimeFrame = GH.ChartTimeFrames.getSelectedTimeFrame();
    if (selectedTimeFrame.days != 0) {
	    ctx.overviewChart.setSelection({
	        "xaxis": {
	            "from": xMin,
	            "to": xMax
	        }
	    }, true);
    }

    // draw the main chart again
    GH.Chart.draw(ctx.chartContainer, ctx.series, ctx.options);
    
    // reverse the legend table	
    GH.RapidBoard.ChartView.CFD._reverseLegendTable();
};

/**
 * Called when the timeframe selection has changed
 */
GH.RapidBoard.ChartView.CFD.handleTimeFrameChanged = function(event) {
	// update the chart
	GH.RapidBoard.ChartView.CFD.updateTimeFrameSelection();
};

/**
 * When the selection on either the main or the overview chart changes, redraw the main chart with the new boundaries.
 */
GH.RapidBoard.ChartView.CFD.handleSelection = function(event, ranges) {
    //AJS.$(".ghx-timeframe-button").removeClass("active");
    //GH.RapidBoard.ChartView.CFD.updateTimeFrameSelection(ranges.xaxis.from, ranges.xaxis.to);
	GH.ChartTimeFrames.setNewChartLimits(-1, ranges.xaxis.from, ranges.xaxis.to, true); // set to custom and use from and to
	
	// update the chart
	//GH.RapidBoard.ChartView.CFD.updateTimeFrameSelection();
};

GH.RapidBoard.ChartView.CFD.resetSelection = function(event) {
    //AJS.$(".ghx-timeframe-button").removeClass("active");
    //GH.RapidBoard.ChartView.CFD.updateTimeFrameSelection(null, null);
	GH.ChartTimeFrames.setNewChartLimits(0, 0, 0, true); // set to All Time
};

/**
 * Turn the server-side model into chartable series. What we want in the end is the number of issues in a certain column
 * on a certain day. So the target structure looks like this:
 * 
 * series: [
 *      todo: [ [ day1, issueCount ], [ day2, issueCount ], ... ],
 *      in progress: [ [ day1, issueCount ], [ day2, issueCount ], ... ],
 *      ...
 * ]
 * 
 * @param model : see CumulativeFlowChartModel for details. Basically, the server gives us the info when an issue changed
 *                from one column into another, and the range of the chart. This saves a lot of unnecessary data (when
 *                issues haven't actually changed).
 */
GH.RapidBoard.ChartView.CFD.buildChartSeries = function(model) {
    var series = [];
    var days = model.days;
    var changes = model.dayColumnChanges;
    var columns = model.columns;
    
    // go over columns backwards, because that's the order the stacking expects it
    for (var colIdx = columns.length-1; colIdx >= 0; colIdx--) {
        var column = columns[colIdx];
        var columnData = [];
        
        // we start with the initial issue count for the current column (on the first
        // day of the chart scope).
        var issueCount = column.initialIssueCount;

        // now go through the days range of the chart
        for (var dayIdx = 0; dayIdx < days.length; dayIdx++) {
            var day = days[dayIdx];
            
            // this is a list of issues that were changed that day, with info on what changed 
            var issues = changes[day];
            
            if (issues) {
                // we have status changes this day, but we don't know yet if our current
                // column is affected.
                for (var issueIdx = 0; issueIdx < issues.length; issueIdx++) {
                    var issue = issues[issueIdx];
                    
                    // JIRA allows transitions to the same status, so we might have things coming and going to the
                    // same column. Both have to be counted.
                    
                    if (issue.columnFrom == colIdx) {
                        // the issue moved out from this column 
                        issueCount--;
                        
                    } 
                    
                    if (issue.columnTo == colIdx) {
                        // the issue moved into this column
                        issueCount++;
                    }
                }
            }
            
            // add the issue count for the given day. If there were no changes, we just
            // continue with yesterday's issue count.
            columnData.push([day, issueCount]);
        }
        
        // now add the column data to the overall series array
        series.push({
            data : columnData,
            label : soy.$$escapeHtml(column.name)
        });
    }
    
    return series;
};
