/**
 * Notification system. Responsible for showing things like error messages.
 * Possible future use for push notifications (triggered Bamboo build)
 */
GH.RapidBoard.Notification = {};

/**
 * Display a list of global (not field related) errors on the page. The errors
 * are expected to have been translated on the server side.
 * 
 * @param errors : see ResultEntry.error.errors for model
 */
GH.RapidBoard.Notification.showErrors = function(errors) {
    var notificationDiv = GH.Message.prepareNotificationDiv();

    var messageList = GH.tpl.rapid.notification.renderErrorMessages(errors)

    AJS.messages.error(notificationDiv, {
        title : AJS.I18n.getText('gh.error.errortitle'),
        body : messageList,
        closeable : true
    });
    notificationDiv.show();
};

/**
 * Display a list of errors as warnings
 */
GH.RapidBoard.Notification.showWarnings = function(errors) {
    var notificationDiv = GH.Message.prepareNotificationDiv();

    var messageList = GH.tpl.rapid.notification.renderErrorMessages(errors)

    AJS.messages.warning(notificationDiv, {
        title : AJS.I18n.getText('gh.error.warningtitle'),
        body : messageList,
        closeable : true
    });
    notificationDiv.show();
    
    // auto hide
    setTimeout(function() { notificationDiv.hide(); }, 5000);
};
