/**
 * Welcome Mat view controller
 */
GH.Welcome = {};

/**
 * Show the start page of the welcome mat
 */
GH.Welcome.showWelcomePage = function() {
    // render the base frame markup
    AJS.$('#gh').html(GH.tpl.rapid.welcome.renderBase());
    
    var uri = AJS.parseUri(window.location.href);

    AJS.$('#ghx-status').html(GH.tpl.rapid.welcome.renderIntro());
    AJS.$('#ghx-options').html(GH.tpl.rapid.welcome.renderFirstStep({
        loggedIn : GH.Welcome.State.data.model.currentUserName != null,
        welcomeMatType : GH.Welcome.State.data.welcomeMatType,
        canUserSearchForRapidViews : GH.Welcome.State.data.canUserSearchForRapidViews,
        returnPath : '?os_destination=' + encodeURIComponent('/secure/' + uri.file)
    }));
    AJS.$('#ghx-video').html(GH.tpl.rapid.welcome.renderVideo({ protocol : uri.protocol }));

    GH.Welcome.bindOpenDialog('#ghx-welcome-project-create', GH.Welcome.openAgileProjectDialog);
    GH.Welcome.bindOpenDialog('#ghx-welcome-view-create', GH.Welcome.openCreateNewViewDialog);
    GH.Welcome.bindOpenDialog('#ghx-welcome-view-find', GH.Welcome.openFindExistingViewDialog);

    // bind on the Import Project link to "complete" the Welcome Mat before the user goes off to another part of JIRA
    AJS.$('#ghx-welcome-project-import').click(function(event) {
        GH.Welcome.completeWelcomeMat(GH.Welcome.redirectToImport);
    });
};

/**
 * Opens the dialog to create a new project, filter and rapid view
 */
GH.Welcome.openCreateNewViewDialog = function() {
    // due to a bug in AJS.Dialog, we need to explicitly remove any lingering DOM elements associated with this dialog
    AJS.$("#ghx-welcome-dialog-view-create").remove();

    var dialog = new AJS.Dialog({
        width : 600,
        height : 400,
        id : 'ghx-welcome-dialog-view-create'
    });

    // analytics for create view dialog
    dialog.eventCategory = "gh.welcome.create.view";
    AJS.$(AJS).trigger("gh.welcome.dialog.change", {
        action: "start",
        category: dialog.eventCategory,
        label: ""
    });

    // add two steps to the dialog
    GH.Welcome.showCreateViewStep(dialog);
    GH.Welcome.showProjectTypeStep(AJS.I18n.getText('gh.rapid.welcome.view.create.button'), GH.Welcome.createView, dialog);

    // return to the first page/panel
    dialog.gotoPage(0);
    dialog.gotoPanel(0);
    dialog.show();

    // bind an esc handler
    GH.Welcome.bindHideHandler(dialog);

    // this causes the user picker frother single select to be created
    AJS.$(dialog.getCurrentPanel().body).trigger("contentRefresh");
};

/**
 * Create A New View
 */
GH.Welcome.showCreateViewStep = function(dialog) {
    var dialogEl = AJS.$('#'+dialog.id);
    dialog.addHeader(AJS.I18n.getText('gh.rapid.welcome.view.create'));
    dialogEl.addClass('ghx-dialog');

    // render panel
    var page = AJS.$(GH.tpl.rapid.welcome.renderCreateNewView());
    dialog.addPanel(AJS.I18n.getText('gh.rapid.welcome.view.create'), page, 'ghx-welcome-page');

    var viewName = dialogEl.find('#ghx-welcome-create-viewname');
    var projectIdsSelect = dialogEl.find("#ghx-welcome-viewprojects");
    // create the project MultiSelect control
    var projectIdsMultiSelect = new AJS.MultiSelect({
        element: projectIdsSelect,
        width:300,
        itemAttrDisplayed: "label",
        ajaxOptions: {
            url: GH.Ajax.buildRestUrl('/project.json'),
            formatResponse: function (response) {
                var ret = [];
                AJS.$(response.success).each(function(){
                    ret.push(new AJS.ItemDescriptor({
                        // value of item added to select
                        value: this.id.toString(),
                        // title of lozenge -- This is already HTML-escaped by MultiSelect, so no need to do it here
                        label: this.displayName,
                        // html used in suggestion -- This is NOT HTML-escaped by MultiSelect, so we must do so explicitly
                        html: soy.$$escapeHtml(this.displayName)
                    }));
                });
                return ret;
            }
        }
    });

    // function to check if we have valid values in all form controls
    var shouldNextBeDisabled = function(){
        return viewName.val() == '' || !projectIdsSelect.val() || projectIdsMultiSelect.$container.parent().find(".error").length > 0;
    };

    // bind a validation function to the Next button
    var formValidateFn = function(dialog) {
        if (shouldNextBeDisabled()) {
            return false;
        }
        return GH.Welcome.validateViewCreate(dialog);
    };
    dialog.addButton(AJS.I18n.getText('gh.rapid.common.next'), formValidateFn, 'ghx-next');
    GH.Welcome.hideVideoAndBindCancel(dialog);

    // disable the Next button initially
    dialogEl.find('.ghx-next').attr('disabled', 'disabled');

    // create an event handler which will toggle the disabled state of the next button based on errors in the form
    var nextDisableEventHandler = function(event){
        var next = AJS.$('#'+dialog.id).find('.ghx-next');
        if (shouldNextBeDisabled()) {
            next.attr('disabled', 'disabled');
        } else {
            next.attr('disabled', '');
        }
    };

    // bind event handler to viewName
    viewName.keyup(nextDisableEventHandler);

    // bind event handler to project ID select
    projectIdsSelect.bind('selected', nextDisableEventHandler);
    // TODO binding to 'unselect' doesn't work properly - this is a workaround
    AJS.$(projectIdsMultiSelect.model).bind("unselect", function() {
        if (this.getSelectedDescriptors().length == 0) {
            nextDisableEventHandler(event);
        }
    });
};

/**
 * Opens the dialog to create a new project, filter and rapid view
 */
GH.Welcome.openFindExistingViewDialog = function() {
    // due to a bug in AJS.Dialog, we need to explicitly remove any lingering DOM elements associated with this dialog
    AJS.$("#ghx-welcome-dialog-view-find").remove();

    var dialog = new AJS.Dialog({
        width : 600,
        height : 200,
        id : 'ghx-welcome-dialog-view-find'
    });

    // analytics for find project dialog
    dialog.eventCategory = "gh.welcome.find.view";
    AJS.$(AJS).trigger("gh.welcome.dialog.change", {
        action: "start",
        category: dialog.eventCategory,
        label: ""
    });

    // add one step to the dialog
    GH.Welcome.showFindViewStep(dialog);

    // return to the first page/panel
    dialog.gotoPage(0);
    dialog.gotoPanel(0);
    dialog.show();

    // bind an esc handler
    GH.Welcome.bindHideHandler(dialog);

    // this causes the user picker frother single select to be created
    AJS.$(dialog.getCurrentPanel().body).trigger("contentRefresh");
};

/**
 * Find an Existing View
 */
GH.Welcome.showFindViewStep = function(dialog) {
    dialog.addHeader(AJS.I18n.getText('gh.rapid.welcome.view.find'));
    AJS.$('#'+dialog.id).addClass('ghx-dialog');

    var page = AJS.$(GH.tpl.rapid.welcome.renderFindExistingView());

    dialog.addPanel(AJS.I18n.getText('gh.rapid.welcome.view.find'), page, 'ghx-welcome-page');
    dialog.addButton(AJS.I18n.getText('gh.rapid.operations.go'), GH.Welcome.validateViewFind);
    GH.Welcome.hideVideoAndBindCancel(dialog);

    new AJS.SingleSelect({
        element: AJS.$("#ghx-welcome-find-viewname"),
        width:300,
        itemAttrDisplayed: "label",
        errorMessage: '',
        ajaxOptions: {
            url: GH.Ajax.buildRestUrl('/rapidviews/list.json'),
            formatResponse: function (response) {
                var ret = [];
                AJS.$(response.success.views).each(function() {
                    ret.push(new AJS.ItemDescriptor({
                        // value of item added to select
                        value: this.id.toString(),
                        // title of lozenge -- This is already HTML-escaped by SingleSelect, so no need to do it here
                        label: this.name,
                        // html used in suggestion -- This is NOT HTML-escaped by SingleSelect, so we must do so explicitly
                        html: soy.$$escapeHtml(this.name)
                    }));
                });
                return ret;
            }
        }
    });
};

/**
 * Opens the dialog to create a new project, filter and rapid view
 */
GH.Welcome.openAgileProjectDialog = function() {
    // due to a bug in AJS.Dialog, we need to explicitly remove any lingering DOM elements associated with this dialog
    AJS.$("#ghx-welcome-dialog").remove();

    var dialog = new AJS.Dialog({
        width : 600,
        height : 400,
        id : 'ghx-welcome-dialog'
    });

    // analytics for create project dialog
    dialog.eventCategory = "gh.welcome.create.project";
    AJS.$(AJS).trigger("gh.welcome.dialog.change", {
        action: "start",
        category: dialog.eventCategory,
        label: ""
    });

    // add two steps to the dialog
    GH.Welcome.showCreateProjectStep(dialog);
    GH.Welcome.showProjectTypeStep(AJS.I18n.getText('gh.rapid.welcome.project.create'), GH.Welcome.createProject, dialog);

    // return to the first page/panel
    dialog.gotoPage(0);
    dialog.gotoPanel(0);
    dialog.show();

    // bind an esc handler
    GH.Welcome.bindHideHandler(dialog);

    // this causes the user picker frother single select to be created
    AJS.$(dialog.getCurrentPanel().body).trigger("contentRefresh");
};

/**
 * Create A New Project
 */
GH.Welcome.showCreateProjectStep = function(dialog) {
    dialog.addHeader(AJS.I18n.getText('gh.rapid.welcome.project.create'));
    AJS.$('#'+dialog.id).addClass('ghx-dialog');

    var page = AJS.$(GH.tpl.rapid.welcome.renderCreateProject({ model : GH.Welcome.State.data.model }));

    // auto-suggest a project key based on the name
    page.find('#ghx-welcome-projectname').keyup(GH.Welcome.suggestProjectKey);
    
    dialog.addPanel(AJS.I18n.getText('gh.rapid.welcome.project.create'), page, 'ghx-welcome-page');
    dialog.addButton(AJS.I18n.getText('gh.rapid.common.next'), GH.Welcome.validateProject);
    GH.Welcome.hideVideoAndBindCancel(dialog);
};

/**
 * Select Project Type (currently only kanban supported). This is the step that currently
 * fires the data creation.
 */
GH.Welcome.showProjectTypeStep = function(createText, createFunction, dialog) {
    dialog.addPage();
    dialog.addHeader(AJS.I18n.getText('gh.rapid.welcome.template.select'));
    AJS.$('#'+dialog.id).addClass('ghx-welcome-dialog');

    var page = AJS.$(GH.tpl.rapid.welcome.renderProjectTypeStep());
    dialog.addPanel(AJS.I18n.getText('gh.rapid.welcome.template.select'), page, 'ghx-welcome-page');
    
    dialog.addButton(AJS.I18n.getText('gh.rapid.common.back'), function(dialog) {
        dialog.prevPage();
    });

    dialog.addButton(createText, function(){
        // analytics for completed welcome mat
        AJS.$(AJS).trigger("gh.welcome.dialog.change", {
            action: "complete",
            category: dialog.eventCategory,
            // TODO with RB for scrum: parameterise this, currently hardcoded as all logic is also hardcoded to kanban
            label: "kanban"
        });
        createFunction();
    }, 'ghx-create');
    
    GH.Welcome.hideVideoAndBindCancel(dialog);

    var buttonCreate = AJS.$('#'+dialog.id).find('.ghx-create');
    var optionKanban = AJS.$('#'+dialog.id).find('.ghx-kanban');
    var optionScrum = AJS.$('#'+dialog.id).find('.ghx-scrum');
    var presets = AJS.$('#ghx-presets');

    // disable create initially
    buttonCreate.attr('disabled','disabled');

    var activeBox = null;
    var activeText = null;

    // toggle on option click - Kanban only so far
    optionKanban.click(function(){
        console.log('clicked: ' + this);
        if (activeBox == this) {
            AJS.$(this).removeClass('ghx-active');
            buttonCreate.attr('disabled','disabled');
            activeBox = null;
            activeText = null;
        } else {
            activeBox = this;
            activeText = AJS.$(presets).html();
            AJS.$(this).addClass('ghx-active');
            buttonCreate.attr('disabled','');
        }
    });

    // Kanban: Hover & Show Presets
    optionKanban.hover(
        function(){
            console.log('mouseOver: ' + this);
            AJS.$(presets).html(GH.tpl.rapid.welcome.renderKanbanPresets());
        },
        function(){
            console.log('mouseOut: ' + this);
            AJS.$(presets).html(activeText || '');
        }

    );
    // Scrum: Hover & Show Presets
    optionScrum.hover(
        function(){
            console.log('mouseOver: ' + this);
            AJS.$(presets).html(GH.tpl.rapid.welcome.renderScrumPresets());
        },
        function(){
            console.log('mouseOut: ' + this);
            AJS.$(presets).html(activeText || '');
        }
    );

};

/**
 * Load some data from the server and show the welcome page
 */
GH.Welcome.loadModel = function() {
    // load the model and kick off to the first step view
    GH.Ajax.get({
        url : GH.Ajax.buildRestUrl('welcome'),
        success : function(result) {
            // initialise the internal state with data from the model
            GH.Welcome.State.setWelcomeMatType(result.welcomeMatType);
            GH.Welcome.State.setCanUserSearchForRapidViews(result.canUserSearchForRapidViews);
            GH.Welcome.State.setModel(result);
            GH.Welcome.State.setProject({
                name : '',
                key : '',
                leadUserName : result.currentUserName
            });

            GH.Welcome.showWelcomePage();
        }
    });
};

/**
 * Do some validation on the project data, so the user can't leave the page with invalid data selected.
 */
GH.Welcome.validateProject = function(dialog) {
    AJS.$(dialog.getCurrentPanel().body).find('.ghx-error').remove();
    var project = {
            name : AJS.$('#ghx-welcome-projectname').val(),
            key : AJS.$('#ghx-welcome-projectkey').val(),
            lead : AJS.$('#ghx-welcome-projectlead option:selected').val()
    };
    GH.Ajax.get({
        url : GH.Ajax.buildRestUrl('welcome/validateProject'),
        contentType : 'application/json',
        data : project,
        errorContextMap : {
            'projectName' : '#ghx-welcome-projectname',
            'projectKey' : '#ghx-welcome-projectkey',
            'projectLead' : '#ghx-welcome-projectlead'
        },
        success : function(data) {
            GH.Welcome.State.setProject(project);

            dialog.nextPage();
        }
    });
};

GH.Welcome.validateViewCreate = function(dialog) {
    var view = {
        viewName : AJS.$('#ghx-welcome-create-viewname').val(),
        projectIds : AJS.$('#ghx-welcome-viewprojects').val()
    };
    var url = GH.Ajax.buildRestUrl('welcome/validateViewCreate');
    var errorContextMap = {
        'viewName' : '#ghx-welcome-create-viewname',
        'projectIds' : '#ghx-welcome-viewprojects'
    };
    var successFn = function(data) {
        GH.Welcome.State.setView(view);
        dialog.nextPage();
    };
    GH.Welcome._validateView(dialog, view, url, errorContextMap, successFn);
};

/**
 * Do some validation on the view selected
 */
GH.Welcome.validateViewFind = function(dialog) {
    var view = {
        id : AJS.$('#ghx-welcome-find-viewname').val()
    };
    var url = GH.Ajax.buildRestUrl('welcome/validateViewFind');
    var errorContextMap = {
        'viewName' : '#ghx-welcome-find-viewname'
    };
    var successFn = function(data) {
        // analytics for complete find view dialog
        AJS.$(AJS).trigger("gh.welcome.dialog.change", {
            action: "complete",
            category: dialog.eventCategory,
            label: ""
        });

        // before redirecting, "complete" the welcome mat so that it is not shown for future visits
        GH.Welcome.completeWelcomeMat(function(){
            // forward user to view
            GH.Welcome.redirectToRapidView(view.id);
        });
    };
    GH.Welcome._validateView(dialog, view, url, errorContextMap, successFn);
};

/**
 * Do some validation on the view selected
 */
GH.Welcome._validateView = function(dialog, view, url, errorContextMap, successFn) {
    // clear any existing errors
    AJS.$(dialog.getCurrentPanel().body).find('.ghx-error').remove();
    GH.Ajax.get({
        url : url,
        contentType : 'application/json',
        data : view,
        errorContextMap : errorContextMap,
        success : successFn
    });
};


/**
 * Take the data that we've collected through the wizard and send it to the server
 */
GH.Welcome.createProject = function() {
    GH.Ajax.put({
        url : GH.Ajax.buildRestUrl('/welcome/createProject'),
        contentType : 'application/json',
        data : JSON.stringify(GH.Welcome.State.getCreateProjectRequest()),
        success : GH.Welcome.handleCreateResult
    });
};

/**
 * Take the data that we've collected through the wizard and send it to the server
 */
GH.Welcome.createView = function() {
    GH.Ajax.put({
        url : GH.Ajax.buildRestUrl('/welcome/createView'),
        contentType : 'application/json',
        data : JSON.stringify(GH.Welcome.State.getCreateViewRequest()),
        success : GH.Welcome.handleCreateResult
    });
};

/**
 * The server sends back most of the data we need to show the result step. Data is raw, so
 * we do some preprocessing here.
 */
GH.Welcome.handleCreateResult = function(result) {
    GH.storage.put('gh.welcomeMessage', result.success, true);
    
    // forward the user to the newly created rapid board
    GH.Welcome.redirectToRapidView(result.success.rapidView.id);
};

GH.Welcome.redirectToRapidView = function(rapidViewId) {
    var url = AJS.parseUri(window.location.href);
    window.location.href = url.protocol + '://' + url.authority + url.directory + 'RapidBoard.jspa?rapidView=' + rapidViewId;
};

GH.Welcome.redirectToImport = function() {
    var url = AJS.parseUri(window.location.href);
    window.location.href = url.protocol + '://' + url.authority + url.directory + 'admin/views/ExternalImport1.jspa';
};

GH.Welcome.completeWelcomeMat = function(callback) {
    GH.Ajax.put({
        url : GH.Ajax.buildRestUrl('/userData/seenWelcomeMat'),
        data: { "hasSeen": true },
        success: callback
    });
};

/**
 * suggest the first three characters as project key
 */
GH.Welcome.suggestProjectKey = function(event) {
    var projectName = AJS.$(event.target).val();
    var projectKey = AJS.$('#ghx-welcome-projectkey');

    if (projectName.length >= 3 && projectKey.val() == '') {
        projectKey.val(projectName.slice(0, 3).toUpperCase());
    }
};

/**
 * Common click-binding-to-dialog code
 */
GH.Welcome.bindOpenDialog = function(selector, openFn) {
    AJS.$(selector).click(function(event) {
        // it's a link, we don't want it to execute
        event.preventDefault();
        openFn();
    });
};

GH.Welcome.generateCancelHandler = function(dialog) {
    // we bind the dialog in this closure (rather than providing it as a parameter) as the two
    //  places we use it have different prototypes. This is why we have this generate function
    return function() {
        // analytics for cancel tracking
        AJS.$(AJS).trigger("gh.welcome.dialog.change", {
            action: "cancel",
            category: dialog.eventCategory,
            label: dialog.curpage.toString()
        });

        dialog.remove();
        AJS.$('#ghx-video').removeClass('ghx-hidden');

        // unbind the hide handler
        AJS.unbind("remove.dialog");

        // return false to stop propogation of events
        return false;
    }
};

// until we get AUI 3.5, which fires an event on pressing 'esc', we have to manually handle the keyup
GH.Welcome.bindHideHandler = function (dialog) {
    var handler = GH.Welcome.generateCancelHandler(dialog);

    // post AUI 3.5: this
    // AJS.bind("hide.dialog", handler);
    // until then...
    AJS.$(document).bind("keyup", function(e) {
        // esc ascii code...
        if(e.keyCode == 27) {
            handler();
            AJS.$(this).unbind(e);
        }
    });
};

/**
 * When setting up a step of a dialog, we always need to hide the video underneath and add
 * a 'Cancel' link which removes the dialog and reveals the video.
 */
GH.Welcome.hideVideoAndBindCancel = function(dialog) {
    AJS.$('#ghx-video').addClass('ghx-hidden');
    dialog.addCancel(AJS.I18n.getText('gh.rapid.operations.cancel'), GH.Welcome.generateCancelHandler(dialog));
};

GH.Welcome.init = function() {
    // load the model and kick off to the first step
    GH.Welcome.loadModel();

    // workaround for https://studio.atlassian.com/browse/AJS-703
    var dim = AJS.dim;
    AJS.dim = function() {
        try {
            dim.apply(this, arguments);
        } catch (err) {
            // do nothing
        }
    };
};
